/*
 * {app} - {purpose, description}
 *
 * Copyright (c) 2005-2006 Rack-Soft, LLC. All rights reserved.
 *
 * $Header: //itsd/devel/platform/voipnow/251/low_level/kamailio_extra_sources/1.5.4/db/db_4psa.c#1 $
 *
 * Modified: $DateTime: 2010/09/28 18:16:02 $ by $Author: bogdanc $ with $Change: 71991 $
 */




#define _GNU_SOURCE



#include "../dprint.h"
#include "../sr_module.h"
#include "../mem/mem.h"
#include "../ut.h"
#include "db_cap.h"
#include "db_id.h"
#include "db_pool.h"
#include "db.h"
#include "../4psa_mem_wrappers.h"

#define MEMC_BEHAVIOR_SERVER_FAILURE_LIMIT					5
#define MEMC_BEHAVIOR_SND_TIMEOUT							500000
#define MEMC_BEHAVIOR_RCV_TIMEOUT							500000
#define MEMC_BEHAVIOR_CONNECT_TIMEOUT						10

extern struct redisurl_list_t * g_mem_list;


static struct redis_client * init_redis_client(struct redis_client * source) {

	struct redis_client * redis = NULL;
//	redis_mm_t *allocator = redis_mm_create(malloc_f, calloc_f, realloc_f, free_f);

	redis = redis_client_create(REDIS_HASH_MD5,\
        REDIS_DISTRIBUTION_CONSISTENT_KETAMA, NULL, 0);

	if (g_log_redis) {
		struct redis_log_levels levels = {
			.error = __LOG_ERROR,
			.debug = __LOG_DEBUG,
			.event = __LOG_EVENT,
			.notice = __LOG_NOTICE,
			.warning = __LOG_WARNING,
			.none = __LOG_NONE,
		};
		redis_enable_log(redis, redis_log_wrap, &levels, REDIS_LEVEL_ERROR |
			REDIS_LEVEL_DEBUG | REDIS_LEVEL_EVENT | REDIS_LEVEL_NOTICE |
			REDIS_LEVEL_WARNING);
	}

	if (!redis) {
		LM_WARN("Unable to initialize the Redis client structure!");
	}

	return redis;
}

static redis_return_t add_redis_server(struct redis_client * mem, const char * redis_url) {
	redis_return_t ret = REDIS_RET_ERROR;
	char * url = NULL;
	char * port = NULL;
	int p = 0;

	if (!mem || !redis_url)
		goto _end;

	url = strdup(redis_url);
	port = strrchr(url, ':');

	if (port) {
		*port++ = '\0';

		p = atoi(port);
		
		if (p <= 0 || p > 65535)
			goto _end;
	}
	ret = redis_server_add(mem, url, p, NULL);

_end:
	free(url);

	return ret;
}

struct redis_client *db_red_init(void) {
	redis_return_t ret = REDIS_RET_ERROR;
	struct redis_client *result = NULL;
	struct redisurl_list_t *redis_url = g_mem_list;

	result = init_redis_client(NULL);

	if (!result) {
		return NULL;
	}

	while (redis_url) {
		ret = add_redis_server(result, redis_url->url);

		if (ret != REDIS_RET_SUCCESS) {
			LM_WARN("Adding cache server %s gave error (%d:%s)\n", redis_url->url, ret, redis_strerror(ret));
		} else {
			LM_DBG("Adding cache server %s was successfull.\n", redis_url->url);
		}
		
		redis_url = redis_url->next;
	}

	return result;
}

void db_red_close(struct redis_client *mem) {

	redis_client_destroy(mem);
}

