/*
 * {app} - {purpose, description}
 *
 * Copyright (c) 2005-2006 Rack-Soft, LLC. All rights reserved.
 *
 * $Header: //itsd/devel/platform/voipnow/251/low_level/kamailio_extra_sources/1.5.4/db/db_ut_4psa.c#1 $
 *
 * Modified: $DateTime: 2010/09/28 18:16:02 $ by $Author: bogdanc $ with $Change: 71991 $
 */




#include "db_ut.h"

#include "../mem/mem.h"
#include "../dprint.h"
#include <limits.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>

#define DBVAL2STR_SZ0 24
#define REDIS_MAX_KEY	255

static redis_return_t safe_redis_set_add(struct redis_client * mem, db_redq_pair_t * memkey, const char *memval, unsigned ttl);

void _print_memres(char ** vals, int n) {
	int i;

	if (!vals)
		return;

	for (i = 0; i<n; ++i) {
		if (vals[i]) LM_DBG("val[%d]: %s\n", i, vals[i]);
	}
}


void _print_dbval(db_val_t * val) {
	char buf[DBVAL2STR_SZ0];
	size_t len = DBVAL2STR_SZ0;
	const char * v = NULL;

	v = db_dbval2str(val, buf, &len);
	LM_DBG("val: type:%d, nul:%d, free:%d, val: %s\n", val->type, val->nul, val->free, (v==NULL)? "null":v);
}

void _print_dbrow(db_row_t * row) {
	int i;

	LM_DBG("cols: %d\n", ROW_N(row));
	for (i = 0; i < ROW_N(row); ++i) {
		LM_DBG("col %d:", i);
		_print_dbval(&(ROW_VALUES(row)[i]));
	}
}

void _print_dbres(db_res_t * res) {
	int i = 0;

	LM_DBG("rows: %d\n", RES_ROW_N(res));
	for (i = 0; i < RES_ROW_N(res); ++i) {
		LM_DBG("row %d:", i);
		_print_dbrow(&(RES_ROWS(res)[i]));
	}

}

/* inline int grow_arr_pair_(void ** arr1, void ** arr2, int * sz) {{{ */
inline int grow_arr_pair_(void ** arr1, void ** arr2, int * sz) {
	void * p = NULL;

	*sz *= 2;

	p = realloc(*arr1, *sz * sizeof(void*));
	if (!p) return -1;
	*arr1 = p;
	p = NULL;

	p = realloc(*arr2, *sz * sizeof(void*));
	if (!p) return -1;
	*arr2 = p;

	return 0;
} /* grow_arr_pair_ }}} */


/* db_res_t * redis2dbres(db_redq_pair_t * memkeys, char ** vals, size_t * lens, int n) {{{ */
db_res_t * redis2dbres(db_redq_pair_t * memkeys, char ** vals, \
			size_t *lens, int n, int cols, int *failed_idx) {
	db_res_t * ret = NULL;
	db_res_t * res = NULL;
	db_row_t * row = NULL;
	int i, rows = 1, col = -1, r = -1;

	res = db_new_result();
	
	if (cols < 0)
		goto _end;

	if (cols == 0) {
		ret = res;
		goto _end;
	}

	if (!vals || !lens) goto _end;

	/* if (n%cols && cols > 1) goto _end; */

	for (i = 0; i < n; ++i) {
		if (memkeys[i].col <= col) rows++;
		col = memkeys[i].col;
	}

	if (db_allocate_columns(res, cols) != 0) {
		LM_ERR("could not allocate columns\n");
		goto _end;
	}
	RES_COL_N(res) = cols;
	bzero(RES_NAMES(res), cols * sizeof(db_key_t));
	bzero(RES_TYPES(res), cols * sizeof(db_type_t));

	RES_ROW_N(res) = rows;
	RES_NUM_ROWS(res) = rows;
	RES_ROWS(res) = (struct db_row*)pkg_malloc(sizeof(db_row_t) * rows);
	if (!RES_ROWS(res)) {
		LM_ERR("no memory left\n");
		goto _end;;
	}

	res->redis = 1;

	_print_memres(vals, n);
	/* make sure u enter the new row creation from the first place */
	r = -1;
	for (i = 0; i < n; ++i) {
		
		if ((i == 0) || 
			memkeys[i].col <= col) {
			/* new row */
			++r;

			/* TODO - if invalid index what shall we do ? */
			if (r >= RES_ROW_N(res)) break;
			if (memkeys[i].col >= RES_COL_N(res)) break;

			row = &(RES_ROWS(res)[r]);
			ROW_VALUES(row) = (db_val_t*)pkg_malloc(cols * sizeof(*ROW_VALUES(row)));
			if (!ROW_VALUES(row)) {
				LM_ERR("no private memory left\n");
				goto _end;
			}
			memset(ROW_VALUES(row), 0, cols * sizeof(*ROW_VALUES(row)));
			ROW_N(row) = RES_COL_N(res);
		}
		
		col = memkeys[i].col;
		//put res on memkeys[i].col column
		//we could put col name too
		if (!RES_NAMES(res)[col]) {
			RES_NAMES(res)[col] = (str*)pkg_malloc(sizeof(str));
			RES_NAMES(res)[col]->s = "<redis>";
			RES_NAMES(res)[col]->len = 7;
			RES_TYPES(res)[col] = memkeys[i].type;
		}

		VAL_TYPE(&ROW_VALUES(row)[col]) = memkeys[i].type;

		LM_DBG("memkey %d of %d (%s:%s)\n", i, n, memkeys[i].memkey, (vals[i]==NULL)? "null":vals[i]);
		if (db_mysql_str2val(memkeys[i].type, &(ROW_VALUES(row)[col]),
					vals[i], lens[i]) < 0) {
			LM_ERR("failed to convert value %s\n", vals[i]);
			if (failed_idx) *failed_idx = i;
			goto _end;
		}
		/* free the values converted rather than aliased */
		if (memkeys[i].type == DB_INT || memkeys[i].type == DB_DOUBLE \
				|| memkeys[i].type == DB_DATETIME || memkeys[i].type == DB_BITMAP ) {

			safe_free(vals[i]);
		}
	}

	ret = res;
	if (failed_idx) *failed_idx = n;
_end:
	if (!ret && res) {
		if (res->redis) redis_dbres_free(res);
		if (db_free_result(res) < 0) LM_ERR("unable to free result structure\n");
	}

	if (ret) _print_dbres(ret);

	return ret;
} /* redis2dbres }}} */


/* 4PSA: transform from arr of memkey strcuts into 2 arr of keys and lengths
 * retunrs no of keys put in the arr */
/* int db_keyst2keyarr(db_redq_pair_t * memkeys, char *** keystr, size_t ** len) {{{ */
int db_keyst2keyarr(db_redq_pair_t * memkeys, char *** keystr, size_t ** len) {
	int ret = -1;
	int i;
	int sz = MEMKEY_ARR_SIZE;

	*keystr = calloc(sz, sizeof(char *));
	*len = calloc(sz, sizeof(size_t));
	if (!*keystr || !*len) goto _end;

	for (i = 0; memkeys[i].memkey; ++i) {
		if (i == sz) {
			if (grow_arr_pair_((void **)keystr, (void **)len, &sz) == -1) goto _end;
		}

		(*keystr)[i] = memkeys[i].memkey;
		/* XXX - don't include the nul termination in the key
		 * for this confusses libredis */
		(*len)[i] = strlen(memkeys[i].memkey);
	}

	ret = i;
_end:
	if (ret == -1) {
		safe_free(*keystr);
		safe_free(*len);
	}

	return ret;
} /* db_keyst2keyarr }}} */


/* db_res_t * db_red_get(db_redq_pair_t * memkeys, char ** keys, size_t * lens, int n) {{{ */
db_res_t * db_red_get(struct redis_client * mem, db_redq_pair_t * memkeys, char ** keys, size_t * lens, int n) {
	db_res_t * ret = NULL;
	char **vals = NULL;
	char **vals_iter = NULL;
	size_t * val_lens = NULL;
	int i = 0, maxcol = -1, failed_idx = 0;
	redis_return_t redis_err = -1;
	unsigned char isMulti = 0;

 	if (!mem) goto _end;

 	/* check to see if n > 1 and none is a set operation */
 	if (n > 1) {
 		for (i = 0; i < n; i++)
 			if (memkeys[i].setkey) { isMulti = 1; break; }
 		isMulti = !isMulti;
 	}

 	/* issue all required redis queries */
	if (!isMulti) {
		vals     = (char**)pkg_malloc(n * sizeof(char*));
		val_lens = (size_t*)pkg_malloc(n * sizeof(size_t));
		if (!vals || !val_lens) {
			LM_ERR("No more private memory\n");
			return NULL;
		}
		memset(vals, 0, n * sizeof(char*));
		memset(val_lens, 0, n * sizeof(size_t));

		for (i = 0; i < n; i++) {
			redis_err = (memkeys[i].setkey)? /* if we should use the set API */
						( ((memkeys[i].flag == 0) || (memkeys[i].flag == 1))?
								  redis_sval_whole(mem, memkeys[i].memkey, strlen(memkeys[i].memkey), \
										memkeys[i].setkey, strlen(memkeys[i].setkey), \
										vals + i, val_lens + i, "\0", 1)
								: redis_svalfromip_whole(mem, memkeys[i].memkey, strlen(memkeys[i].memkey), \
										memkeys[i].setkey, strlen(memkeys[i].setkey), \
										vals + i, val_lens + i, "\0", 1) )
						: redis_get_whole(mem, memkeys[i].memkey, strlen(memkeys[i].memkey), \
								vals + i, val_lens + i, "\0", 1);

			if (redis_err == REDIS_RET_SET_EMPTY_VALUE) {
				vals[i] = NULL; val_lens[i] = 0;
				redis_err = REDIS_RET_SUCCESS;
			}
			if (redis_err != REDIS_RET_SUCCESS) {
				failed_idx = i;
				goto _end;
			}
		}
	} else {
		size_t vn = 0;
		LM_DBG("Redis get multi....\n");
		redis_err = redis_mget_whole(mem, (const char **)keys, n, lens, &vals, &vn, &val_lens, "\0", 1, REDIS_NULL_ON_FAIL);
		if ((redis_err != REDIS_RET_SUCCESS) || !vals || !val_lens || !vn) {
			failed_idx = 0;
			goto _final;
		}
	}

	i = 0;
	vals_iter = vals;
	while ((i < n) && *vals_iter) {
		LM_DBG("redis get: %s (%s), (val: %s;vallen: %lu)", keys[i], (memkeys[i].setkey)? memkeys[i].setkey : "", \
			*vals_iter, (unsigned long int)val_lens[i]);

		if (strcmp("__NULL", *vals_iter) == 0) {
			LM_DBG("redis key %s (%s) found with __NULL", keys[i], (memkeys[i].setkey)? memkeys[i].setkey : "");
			safe_free(vals[i]);
			if (val_lens) val_lens[i] = 0;
		} else if (strcmp("__NOROWS", *vals_iter) == 0) {
			LM_DBG("redis key %s (%s) found with __NOROWS", keys[i], (memkeys[i].setkey)? memkeys[i].setkey : "");
			break;
		}

		vals_iter++;
		i++;
	}
	if (i < n) {
		failed_idx = 0;
		goto _final;
	}

	/* now get a valid result set */
	for (i = 0; i < n; i++) {
		if (memkeys[i].col > maxcol) maxcol = memkeys[i].col;
	}
	maxcol++;
	
	/* debug found values */
	_print_memres(vals, n);

_end:
	ret = NULL;

	if (redis_err != REDIS_RET_SUCCESS) {
		int flag = memkeys[failed_idx].flag;
		maxcol = -1; /* create a null result set to query db */
		failed_idx = 0;
		LM_DBG("Current memkey flag is : %d, redis error %d\n", flag, (int)redis_err);
		switch (flag) {
		case 1:
		case 4:
			if (redis_err == REDIS_RET_NO_SET_KEY) {
				maxcol = 0; /* create am empty set so not to query the db */
			}
			break;
		case 5:
			if (redis_err == REDIS_RET_NOTFOUND) {
				maxcol = 0; /* create am empty set so not to query the db */
			} else if (redis_err == REDIS_RET_NO_SET_KEY) {
				/* special case - return -1 */
				ret = (db_res_t*)-1;
				goto _final;
			}
			break;
		case 0: /* a normal redis key will have a flag of 0 by default */
		default:
			break;
		}
	}

	/* convert the values to a result set */
	ret = redis2dbres(memkeys, vals, val_lens, n, maxcol, &failed_idx);

_final:
	if (vals) {
		for (i = failed_idx; i < n; i++)
			if (vals[i]) safe_free(vals[i]);
		safe_free(vals);
	}
	if (val_lens) safe_free(val_lens);
	return ret;
} /* db_red_get }}} */


/* void redis_dbres_free(db_res_t * _r) {{{ */
void redis_dbres_free(db_res_t * _r) {
	int i;

	for (i = 0; i < RES_COL_N(_r); ++i) {
		/* once upon a time redis successfuly returned
		 * some values; some were fixed width and were converted
		 * and the redis return could thus be free'd at once
		 * but some were aliased to the pointers in this structure */
		if (_r->rows[0].values[i].nul) continue;

		switch (_r->rows[0].values[i].type) {
			case DB_STRING:
			safe_free(_r->rows[0].values[i].val.string_val);
			break;

			case DB_STR:
			safe_free(_r->rows[0].values[i].val.str_val.s);
			_r->rows[0].values[i].val.str_val.len = 0;
			break;

			case DB_BLOB:
			safe_free(_r->rows[0].values[i].val.blob_val.s);
			_r->rows[0].values[i].val.blob_val.len = 0;
			break;

			default:
			break;
		}
	}
} /* redis_dbres_free }}} */


/* 4PSA: only fixed width vals (transformed to str) are placed in dst
 * existing strings are returned const char* .
 * the return alway points to "transformed" data, be it const
 * or freshly written (transformed) */
/* const char * db_dbval2str(db_val_t * val, char * dst, size_t * len) {{{ */
const char * db_dbval2str(const db_val_t * val, char * dst, size_t * len) {

	if (VAL_NULL(val)) {
		if (*len < sizeof("__NULL")) {
			LM_ERR("buffer too small\n");
			return NULL;
		}
		*len = snprintf(dst, *len, "__NULL");
		return dst;
	}

	switch(VAL_TYPE(val)) {
	case DB_INT:
		if (db_int2str(VAL_INT(val), dst, (int *)len) < 0) {
			LM_ERR("error while converting string to int\n");
			return NULL;
		} else {
			return dst;
		}

	case DB_BITMAP:
		if (db_int2str(VAL_BITMAP(val), dst, (int *)len) < 0) {
			LM_ERR("error while converting string to int\n");
			return NULL;
		} else {
			return dst;
		}

	case DB_DOUBLE:
		if (db_double2str(VAL_DOUBLE(val), dst, (int *)len) < 0) {
			LM_ERR("error while converting string to double\n");
			return NULL;
		} else {
			return dst;
		}

	case DB_STRING:
		*len = strlen(VAL_STRING(val));
		return VAL_STRING(val);

	case DB_STR:
		*len = VAL_STR(val).len;
		return VAL_STR(val).s;

	case DB_DATETIME:
		if (db_time2str(VAL_TIME(val), dst, (int *)len) < 0) {
			LM_ERR("error while converting string to time_t\n");
			return NULL;
		} else {
			return dst;
		}

	case DB_BLOB:
		*len = VAL_BLOB(val).len;
		return VAL_STR(val).s;

	default:
		return NULL;
	}
	/*return -8; --not reached*/
} /* db_dbval2str }}} */


/* int db_red_set_byres(struct redis_client * mem, db_redq_pair_t * memkeys, unsigned ttl, db_ret_t * _r) {{{ */
int db_red_set_byres(struct redis_client * mem, db_redq_pair_t * memkeys, unsigned ttl, db_res_t * _r) {
	int ret = -1;
	int i = 0, row = 0, col = -1;
	const char * strval = NULL;
	char buf[DBVAL2STR_SZ0];
	size_t len = DBVAL2STR_SZ0;
	redis_return_t err = 1;

	if (!mem) goto _end;
	LM_DBG("n rows in set by res: %d\n", _r->n);
	if (!_r->n) {

		for (i = 0; memkeys[i].memkey; ++i) {
			LM_DBG("redis key %s (%s) set up to __NOROWS", memkeys[i].memkey, (memkeys[i].setkey)? memkeys[i].setkey : "");
			err = (memkeys[i].setkey)? safe_redis_set_add(mem, memkeys + i, "__NOROWS", ttl) :					
					redis_set(mem, memkeys[i].memkey, strlen(memkeys[i].memkey), "__NOROWS", sizeof("__NOROWS"), ttl);
		}

		ret = 0;
		goto _end;
	}

	_print_dbres(_r);
	/* if the number of retrieved rows does not match the number of requested
	 * memkeys then there is no way to figure out what rows should be inserted
	 * in the hash; leave uncached  */

	for (i = 0; memkeys[i].memkey; ++i) {
		if (memkeys[i].col <= col) ++row;
		col = memkeys[i].col;
	}

	if (row != _r->n - 1) {
		LM_DBG("memkeys rows diff from res rows: %d:%d\n", row+1, _r->n);
		ret = 0;
		goto _end;
	}

	col = -1; row= 0;
	for (i = 0; memkeys[i].memkey; ++i) {
		char save = 0;

		if (memkeys[i].col <= col) ++row;
		col = memkeys[i].col;

		/* LM_DBG("col, row: %d, %d (supposed no of rows, as mysql told us about them: %d)\n", col, row, _r->rows[row].n); */

		/* TODO - if invalid index what shall we do ? */
		if (col >= _r->rows[row].n || col < 0) continue;
		
		/* never update the keys for some flag values */
		if (memkeys[i].flag > 0) continue;

		len = DBVAL2STR_SZ0;
		strval = db_dbval2str(&_r->rows[row].values[col], buf, &len);
		/* LM_DBG("value to be put in cache is: %s\n", strval); */
		if (!strval) continue;
		/* for their (may I say stupid) str type we have no \0 termination
		 * this could be invalid read/write however:
		 * 1. the mem is all prealloc'd
		 * 2. is private for this proc
		 * 3. the prealloc'd is extended so that we should never be at the margin
		 * and realy do invalid write
		 * note: if by coincidence the first byte after the str/blob
		 * is \0 then it should stay that way until we set the val to redis */
		if (!_r->rows[row].values[col].nul &&\
			(_r->rows[row].values[col].type == DB_STR || _r->rows[row].values[col].type == DB_BLOB)\
			&& _r->rows[row].values[col].val.str_val.s[len]) {

			save = _r->rows[row].values[col].val.str_val.s[len];
			_r->rows[row].values[col].val.str_val.s[len] = '\0';
		}

		/* XXX - we could count \0 in the key but redis will take it out
		 * and count it out too;
		 * when redis_[m]get()-ing we MUST NOT count \0 in the key however;
		 * thus we try to make o habbit out of leaving it out
		 * and leave it out even from the very redis_set() */
		LM_DBG("setting to mem %s(%s):%s\n", memkeys[i].memkey, (memkeys[i].setkey)? memkeys[i].setkey : "", strval);
		err = (memkeys[i].setkey)? safe_redis_set_add(mem, memkeys + i, (const char*)strval, ttl) :					
				redis_set(mem, memkeys[i].memkey, strlen(memkeys[i].memkey), (void *)strval, len+1, ttl);

		if (save) {
			_r->rows[row].values[col].val.str_val.s[len] = save;
			save = 0;
		}
		LM_DBG("set_byres: %d\n", err);
		if (err != REDIS_RET_SUCCESS) goto _end;
	}

	ret = 0;
_end:

	LM_DBG("set_byres ret: %d\n", ret);
	return ret;
} /* db_red_set_byres }}} */


/* int db_red_set_byval(struct redis_client * mem, db_redq_pair_t * memkeys, unsigned ttl, const db_val_t * _v, int n) {{{ */
int db_red_set_byval(struct redis_client * mem, db_redq_pair_t * memkeys, unsigned ttl, const db_val_t * _v, int n) {
	int ret = -1;
	int i = 0;
	const char * strval = NULL;
	char buf[DBVAL2STR_SZ0];
	size_t len = DBVAL2STR_SZ0;
	redis_return_t err = 1;

	if (!mem) goto _end;

	for (i = 0; memkeys[i].memkey; ++i) {
		char save = 0;

		/* TODO - if invalid index what shall we do ? */
		if (memkeys[i].col >= n || memkeys[i].col < 0) continue;
		
		/* never update the keys for some flag values */
		if (memkeys[i].flag > 0) continue;

		len = DBVAL2STR_SZ0;
		strval = db_dbval2str(&_v[memkeys[i].col], buf, &len);
		if (!strval) continue;
		/* for their (may I say stupid) str type we have no \0 termination
		 * this could be invalid read/write however:
		 * 1. the mem is all prealloc'd
		 * 2. is private for this proc
		 * 3. the prealloc'd is extended so that we should never be at the margin
		 * and realy do invalid write */
		if (!_v[memkeys[i].col].nul &&\
			(_v[memkeys[i].col].type == DB_STR || _v[memkeys[i].col].type == DB_BLOB)\
		   	 &&  _v[memkeys[i].col].val.str_val.s[len]) {

			save = _v[memkeys[i].col].val.str_val.s[len];
			_v[memkeys[i].col].val.str_val.s[len] = '\0';
		}

		/* XXX - we could count \0 in the key but redis will take it out
		 * and count it out too;
		 * when redis_[m]get()-ing we MUST NOT count \0 in the key however;
		 * thus we try to make o habbit out of leaving it out
		 * and leave it out even from the very redis_set() */
		LM_DBG("setting to mem (by val): %s(%s):%s\n", memkeys[i].memkey, (memkeys[i].setkey)? memkeys[i].setkey : "", strval);
		err = (memkeys[i].setkey)? safe_redis_set_add(mem, memkeys + i, (const char*)strval, ttl) :	
				redis_set(mem, memkeys[i].memkey, strlen(memkeys[i].memkey), (void *)strval, len+1, ttl);

		if (save) _v[memkeys[i].col].val.str_val.s[len] = save;
		LM_DBG("set_byval: %d (%d)\n", err, REDIS_RET_SUCCESS);
		if (err != REDIS_RET_SUCCESS) goto _end;

	}

	ret = 0;
_end:

	LM_DBG("set_byval ret: %d\n", ret);
	return ret;
} /* db_red_set_byval }}} */


/* int db_red_del(struct redis_client * mem, db_redq_pair_t * memkeys) {{{ */
int db_red_del(struct redis_client * mem, db_redq_pair_t * memkeys) {
	int ret = -1;
	int i = 0;

	if (!mem) goto _end;

	for (i = 0; memkeys[i].memkey; ++i) {
		if (memkeys[i].setkey) {
			LM_WARN("Will not delete a set key entry for an INSERT/UPDATE avp_redis_query_set query\n");
			continue;
		}
		ret = redis_del(mem, memkeys[i].memkey, strlen(memkeys[i].memkey)+1);
		if (ret != 0) goto _end;
	}

	ret = 0;
_end:
	return ret;
} /* db_red_del }}} */


static redis_return_t safe_redis_set_add(struct redis_client * mem, db_redq_pair_t * memkey, const char *memval, unsigned ttl)
{
	redis_return_t err = 0;
	int try = 1;
	int key_len = 0, setkey_len = 0, val_len;
	
	if (!mem || !memkey || !memkey->memkey || !memkey->setkey || !memval)
		return err;
	
	key_len = strlen(memkey->memkey);
	setkey_len = strlen(memkey->setkey);
	val_len = strlen(memval);
	/* first try to add the new key-value entry to the set
	 * if the set does not exists it will be created */
	while (try > 0) {
		err = redis_saddval(mem, memkey->memkey, key_len, memkey->setkey, setkey_len, memval, val_len);
		if ((err == REDIS_RET_ALREADY_EXISTS) && (try > 0)) {
			/* delete the key-value set entry so we can try to add it once more */
			LM_WARN("Duplicate set key %s (%s), deleting...\n",  memkey->memkey,  memkey->setkey);
			err = redis_srem(mem, memkey->memkey, key_len, memkey->setkey, setkey_len);
		} else { try++; break; }
		try--;
	}
	
	if ((try > 0) && (ttl > 0)) {
		if (redis_set_ttl(mem, memkey->memkey, key_len, ttl) == REDIS_RET_SUCCESS) {
			LM_DBG("Setting ttl %d for set %s\n", ttl, memkey->memkey);
		}
	}

	return err;
}
