/*
 * 4PSA VoipNow - Implements a custom memory allocator for shared memory
 *                allocation based on the private memory SLAB allocator.
 *                Pratically the allocation logic is unchanged, the only
 *                difference is in the allocation of shared memory segments.
 *                We allocate a SLAB to a SLAB class from a previously mapped
 *                shared memory. The allocation of the shared memory is lazy
 *                and is done on a demand basis (when shm_slab_malloc needs
 *                additional shared memory) or on a SIGFAULT signal.
 *                Valgrind debugging is also supported.
 *
 * Copyright (c) 2009 Rack-Soft, LLC. All rights reserved..
 *
 * $Header: //itsd/devel/platform/voipnow/251/low_level/kamailio_extra_sources/1.5.4/mem/shm_slab_malloc.h#1 $
 *
 * Modified: $DateTime: 2010/09/28 18:16:02 $ by $Author: bogdanc $ with $Change: 71991 $
 */

#if !defined(SHM_SLAB_MALLOC_H) && defined(SHM_SLAB_MALLOC)
#define SHM_SLAB_MALLOC_H

#include "meminfo.h"

/* ============================ Kamailio shared memory SLAB API  ========================= */
/* Initializes the SLAB allocation system. Must be called before any other method is used.
 * Set the desired the SLAB class SLOT factor (how much a SLOT size will increase in
 * succesive SLAB classes. E.g for 2.0 slot classes will be 128b, 256b, 521b...), and if you'd
 * like to preallocate the SLABS in each class.
 * Default Kamailio implementation is (SHM_SLOT_SLAB_FACTOR 2.0), macro defined in config.h*/
int     shm_slabs_init(const double factor);

/* Allocate object of given length in shared memory. 0 on error */
void*   shm_slabs_malloc(const size_t size);

/* Free a previously allocated object in shared memory */
void    shm_slabs_free(void *ptr);

/* Reallocate object to a given length in shared memory. 0 on error */
void*   shm_slabs_realloc(void *ptr, const size_t size);

/* fills a malloc info structure with info about the SLAB shared memory allocator  */
void    shm_slabs_info(struct mem_info* info);

/* dump status information about each SLAB class and associated SLOTS in shared memory */
void    shm_slabs_status(void);

#ifdef STATISTICS
typedef enum {SIZE, USED, RUSED, MUSED, FREE, FRAGS} shm_slab_statistics_t;
unsigned long shm_slab_get(shm_slab_statistics_t type);
#endif

#ifdef __VALGRIND__
void valgrind_signal_handler();
#define VALGRIND_SIGNAL_HANDLER valgrind_signal_handler()
#endif
/* ======================================================================================= */
#endif
