/*
 * 4PSA VoipNow - Implements a custom memory allocator based on the Memcached
 *                server implementation of a SLAB allocation.
 *                Our own implementation uses mmap and munmap for SLABs and
 *                internal dynamic data manipulation, and SLOT overhead
 *                logic for SLOT to SLAB class association (a SLOT overhead
 *                is a slot's additional data information added to each SLOT
 *                pointer address to be able to quickly identify it's parent
 *                SLAB class or SLAB address (only used for Valgrind profilling)
 *
 * Copyright (c) 2009 Rack-Soft, LLC. All rights reserved..
 *
 * $Header: //itsd/devel/platform/voipnow/251/low_level/kamailio_extra_sources/1.5.4/mem/slab_malloc.h#1 $
 *
 * Modified: $DateTime: 2010/09/28 18:16:02 $ by $Author: bogdanc $ with $Change: 71991 $
 */

#if !defined(SLAB_MALLOC_H) && defined(SLAB_MALLOC)
#define SLAB_MALLOC_H

#include "meminfo.h"

/* =========================== Kamailio private memory SLAB API  ========================= */
/* Initializes the SLAB allocation system. Must be called before any other method is used.
 * Set the desired the SLAB class SLOT factor (how much a SLOT size will increase in
 * succesive SLAB classes. E.g for 2.0 slot classes will be 128b, 256b, 521b...), and if you'd
 * like to preallocate the SLABS in each class.
 * Default Kamailio implementation is (SLOT_SLAB_FACTOR 2.0), macro defined in config.h    */
void    slabs_init(const double factor);

/* Allocate object of given length. 0 on error */
void*   slabs_malloc(const size_t size);

/* Free a previously allocated object */
void    slabs_free(void *ptr);

/* Reallocate object to a given length. 0 on error */
void*   slabs_realloc(void *ptr, const size_t size);

/* fills a malloc info structure with info about the SLAB allocator  */
void    slabs_info(struct mem_info* info);

/* dump status information about each SLAB class and associated SLOTS */
void    slabs_status(void);
/* ======================================================================================= */
#endif
