/*
 * 4PSA VoipNow - Miscellaneous functions
 *
 * Copyright (c) 2005-2009 Rack-Soft, LLC. All rights reserved.
 *
 * $Header: //itsd/devel/platform/voipnow/251/low_level/kamailio_extra_sources/1.5.4/misc_4psa.c#2 $
 *
 * Modified: $DateTime: 2010/10/27 19:46:48 $ by $Author: mihaim $ with $Change: 72773 $
 */

#define _GNU_SOURCE
#include <stdio.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "misc_4psa.h"
#include "modules/presence/presence.h"

#define MAX_KEY_LEN 200
char cache_key[MAX_KEY_LEN];
#define MAX_VALUE_LEN 512
char cache_value[MAX_KEY_LEN];

/* Cache the value of extension publish during a request 
 * to avoid multiple cache queries for it */
int g_cached_extpublish = 0;
str g_cached_ext = {NULL, 0};

void reset_cached_extension_publish()
{
	g_cached_extpublish = 0;
	pkg_free(g_cached_ext.s);
	g_cached_ext.s = NULL;
	g_cached_ext.len = 0;
}

/* int get_extension_publish(struct redis_client *mem, str extno) {{{ */
int get_extension_publish(struct redis_client *mem, str extno, db_func_t *dbf, db_con_t *dbcon) {
	/* return false by default */
	int ret = 0;
	db_redq_pair_t memkeys[2] = {{0, 0, 0, 0, 0}, {0, 0, 0, 0, 0}};
	/* extension.<extno>.extension_publish */
	size_t memkey_len = 0;
	str query = {0, -1};
	db_res_t *result= NULL;
	db_val_t *val = NULL;

	if ((g_cached_ext.len == extno.len) && !memcmp(g_cached_ext.s, extno.s, extno.len)) {
		return g_cached_extpublish;
	}

	pkg_strcpy(&g_cached_ext, &extno);

	memkey_len = snprintf(cache_key, MAX_KEY_LEN, EXT_PART"%.*s"PUBLISH_PART, extno.len, extno.s);
	memkeys[0].memkey = cache_key;
	memkeys[0].type = DB_STRING;
	memkeys[0].col = 0;

	query.len = asprintf(&query.s
			, "SELECT value FROM extension, extension_prefs "
			"  WHERE extended_number = '%.*s' "
			"	AND extension.id = extension_prefs.extension_id "
			"	AND param = 'extension_publish' ", extno.len, extno.s);
	if (query.len <= 0) goto _end;

	LM_DBG("Quering for 'extenion_publish' status "
			"using redis key: %s and mysql query: %s\n"\
			, memkeys[0].memkey, query.s);

	if (dbf->red_raw_query (mem, memkeys, EXTENSION_PUBLISH_TTL, dbcon, &query, &result) < 0) {
		LM_WARN("failed to query for 'extension_publish' status for extension %.*s\n"\
			, extno.len, extno.s);
	}

	/* assume false if nothing came out from DB */
	if ( !result || !RES_ROWS(result)|| !ROW_N(RES_ROWS(result)) ||
			((val = ROW_VALUES(RES_ROWS(result))) && val->nul) ) {
		/* add the no publish key in Redis so not to query DB again for the key */
		char *ext_publish = "0"; /* Set the default to publish by Kamailio if nothing is found */
		redis_set(mem, memkeys[0].memkey, memkey_len, ext_publish, 1, EXTENSION_PUBLISH_TTL);
		goto _end;
	}

	if (val->val.string_val && val->val.string_val[0]=='1') {
		ret = 1;
	}
	
	/* leave with false the default return if not instructed otherwise above */
_end:
	if (result) dbf->free_result(dbcon, result);
	if (query.s) free(query.s);
	
	g_cached_extpublish = ret;

	return ret;
} /* get_extension_publish }}} */

/* static enum ext_state_t intstate(const char * state) {{{ */
static enum ext_state_t intstate(const char * state) {
	if (!state || !state[0]) return ST_OFFLINE;

	if (strcasecmp(state, ST_RINGING_S) == 0) return ST_RINGING;
	else if (strcasecmp(state, ST_BUSY_S) == 0) return ST_BUSY;
	else if (strcasecmp(state, ST_ONLINE_S) == 0) return ST_ONLINE;
	else if (strcasecmp(state, ST_OFFLINE_S) == 0) return ST_OFFLINE;
	else if (strcasecmp(state, ST_REGISTER_S) == 0) return ST_REGISTER;

	return ST_ERR;
} /* intstate }}} */

/* const char * strstate(enum ext_state_t state) {{{ */
const char * strstate(enum ext_state_t state) {
	switch (state) {
		case ST_OFFLINE:	return ST_OFFLINE_S;
		case ST_RINGING:	return ST_RINGING_S;
		case ST_BUSY:		return ST_BUSY_S;
		case ST_ONLINE:		return ST_ONLINE_S;
		case ST_REGISTER:	return ST_REGISTER_S;
		case ST_ERR:		return "";
	}
	return "";
} /* strstate }}} */

const char *convert_presence_state_to_dlginfo(const char *presence_state) {
	if (!strcasecmp(presence_state, ST_BUSY_S) || !strcasecmp(presence_state, ST_OFFLINE_S)) {
		return ST_CONFIRMED_S;
	}
	if (!strcasecmp(presence_state, ST_RINGING_S)) {
		return ST_EARLY_S;
	}
	if (!strcasecmp(presence_state, ST_ONLINE_S)) {
		return ST_TERMINATED_S;
	}

	return NULL;
}

/* Get busy or ringing counter from redis */
static int get_counter(struct redis_client *mem, str extno, int busy) {
	int ret = -1;	
	redis_return_t err = -1;
	int bc = -1; 
	int count = 0;
	size_t val_len = 0;
	char *cached_state = NULL;

	/* extension.<full_ext_no>.busy */
	if (busy) {
		bc = snprintf(cache_key, MAX_KEY_LEN, EXT_PART"%.*s."BUSY, extno.len, extno.s);
	} else {
		bc = snprintf(cache_key, MAX_KEY_LEN, EXT_PART"%.*s."RINGING, extno.len, extno.s);
	}
	if (bc > 0 && bc < MAX_KEY_LEN) {
		err = redis_get_whole(mem, cache_key, bc, &cached_state, &val_len, "\0", 1);
		if ((err != REDIS_RET_SUCCESS) || !cached_state) {
			LM_DBG("redis does not know the busy counters of the extension %.*s . redis error: (%d:%s)\n" \
					, extno.len, extno.s, err, redis_strerror(err));
			return ret;
		}
		if (cached_state) {
			sscanf(cached_state, "%d", &count);
			free(cached_state);
			cached_state = NULL;
			ret = count;
		}
	}

	return ret;
}

/*!\fn int presentity_allow_state_switch(str extno, const char * newstate) {{{
 * Checks if an extension can switch states depending on state counters */
const char *presentity_allow_state_switch(struct redis_client *mem, str extno, const char *newstate) {
	const char *ret = NULL; /* Always default to switching is allowed */
	int count = 0;
	int bc = -1; 
	redis_return_t err = -1;
	char *cached_state = NULL;
	size_t val_len = 0;
	enum ext_state_t newst = ST_ERR, curr_st = ST_ERR;

	if (!mem || !extno.s || !newstate) goto _end;

	newst = intstate(newstate);
	if (newst == ST_REGISTER) 
		ret = ST_ONLINE_S;
	else
		ret = strstate(newst);

	bc = snprintf(cache_key, MAX_KEY_LEN, EXT_PART"%.*s"STATE_PART, extno.len, extno.s);
	if (bc > 0 && bc < MAX_KEY_LEN) {
		/* get curr state from redis only; key has no \0 counted to it */
		err = redis_get_whole(mem, cache_key, bc, &cached_state, &val_len, "\0", 1);
		if (err) {
			LM_WARN("redis does not know the presence status of the extension %.*s . redis error: (%d:%s)\n" \
					, extno.len, extno.s, err, redis_strerror(err));
		}
	}

	/* assume true if nothing came out from DB */
	if (cached_state) {
		curr_st = intstate(cached_state);
	} else if ((err == REDIS_RET_CONNECTION_FAILURE) || (err == REDIS_RET_SERVER_DOWN))	{
		curr_st = ST_ONLINE; /* set the state online if redis is down */
		ret = ST_ONLINE_S;
		goto _end;
	} else {
		curr_st = ST_OFFLINE;
	}

	if (cached_state) free(cached_state);
	cached_state = NULL;

	switch (newst) {
		case ST_ONLINE:
		case ST_BUSY:
		case ST_RINGING:
		{
			/* Get ringing counter */
			count = get_counter(mem, extno, 0);
			if (count < 0) {
				/* Set online if redis problem */
				ret = ST_ONLINE_S;
				goto _end;
			}
			
			if (count) {
				if (newst == ST_RINGING) {
					/* Switch to ringing as requested */
					goto _end;
				}
				LM_WARN("Can't switch to %s. Ringing=%d\n", strstate(newst), count);
				ret = NULL;
				goto _end;
			}
			
			/* Get busy counter */	
			count = get_counter(mem, extno, 1);
			if (count < 0) {
				/* Set online if redis problem */
				ret = ST_ONLINE_S;
				goto _end;
			}
				
			if (count) {
				if (newst == ST_BUSY) {
					/* Switch to busy as requested */
					goto _end;
				} else if (newst == ST_ONLINE) {
					/* Switch to busy as all ringing might be terminated */
					ret = ST_BUSY_S;
					goto _end;
				}
				LM_WARN("Can't switch to %s. Busy=%d\n", strstate(newst), count);
				ret = NULL;
				goto _end;
			}
		}
		break;

		case ST_OFFLINE:
		case ST_REGISTER:
		{
			count = get_counter(mem, extno, 1);
			if (count < 0) {
				ret = NULL;
				goto _end;
			}
				
			/* can't be online/offline; we are still busy on other lines */
			if (count) {
				LM_WARN("Can't switch to %s. Busy=%d\n", strstate(newst), count);
				ret = NULL;
				goto _end;
			}
			
			count = get_counter(mem, extno, 0);
			if (count < 0) {
				ret = NULL;
				goto _end;
			}
				
			/* can't be online/offline; we are still ringing on other lines */
			if (count) {
				LM_WARN("Can't switch to %s. Ringing=%d\n", strstate(newst), count);
				ret = NULL;
				goto _end;
			}
		}
		break;
		case ST_ERR:
			ret = NULL;
		break;
	}

_end:
	LM_DBG(" + red: Allow state switch to %s (wanted %s) for %.*s? %s.\n", ret?ret:"<nochange>", strstate(newst), extno.len, extno.s, ret?"yes":"no");
	if (cached_state) pkg_free(cached_state);

	return ret;
#undef len_
} /* }}} presentity_allow_state_switch */


/* void reset_extension_presence_state(struct redis_client *mem, str *extno) {{{
 * This is implemented as a callback when a contact is INSERTED or UPDATED in usrloc */
typedef struct redis_client* (*redis_connection_t)(); /* The pua_dialoginfo Redis connection - private per process */
void reset_extension_presence_state(ucontact_t *c, int type, void *param) {
	int bc = -1;
	redis_return_t err = -1;
	struct redis_client *mem =  (*(*(redis_connection_t*)param))();
	str * extno = NULL;

	if (!c || !c->aor || !mem)
		return;
	extno = c->aor;

	/* set presence state online */
	bc = snprintf(cache_key, MAX_KEY_LEN, EXT_PART"%.*s"STATE_PART, extno->len, extno->s);
	if (bc > 0 && bc < MAX_KEY_LEN) {
		if (redis_exists(mem, cache_key, bc) == REDIS_RET_SUCCESS) {
			LM_DBG(" + red: Status key for %.*s alredy there on usrloc update/insert - no reset needed.\n", extno->len, extno->s);
		} else {
			err = redis_set(mem, cache_key, bc, ST_ONLINE_S, sizeof(ST_ONLINE_S), MAKE_NON_SEMIDURABLE(PHONE_STATE_TTL));
		}
	}

	/* set busycalls counter to 0 */
	bc = snprintf(cache_key, MAX_KEY_LEN, EXT_PART"%.*s."BUSY, extno->len, extno->s);
	if (bc > 0 && bc < MAX_KEY_LEN) {
		if (redis_exists(mem, cache_key, bc) == REDIS_RET_SUCCESS) {
			LM_DBG(" + red: Busy key for %.*s alredy there on usrloc update/insert - no reset needed.\n", extno->len, extno->s);
		} else {
			err = redis_set(mem, cache_key, bc, (char *)"0", 1, MAKE_NON_SEMIDURABLE(PHONE_STATE_TTL));
		}
	}

	/* set ringingcalls counter to 0 */
	bc = snprintf(cache_key, MAX_KEY_LEN, EXT_PART"%.*s."RINGING,extno->len, extno->s);
	if (bc > 0 && bc < MAX_KEY_LEN) {
		if (redis_exists(mem, cache_key, bc) == REDIS_RET_SUCCESS) {
			LM_DBG(" + red: Ringing key for %.*s alredy there on usrloc update/insert - no reset needed.\n", extno->len, extno->s);
		} else {
			err = redis_set(mem, cache_key, bc, (char *)"0", 1, MAKE_NON_SEMIDURABLE(PHONE_STATE_TTL));
		}
	}

} /* }}} reset_extension_presence_state */

/* void delete_extension_presence_state(struct redis_client *mem, str *extno) {{{
 * Use this when extension unregisters. */
void delete_extension_presence_state(struct redis_client *mem, str *extno) {
	int bc = -1;

	/* remove presence state */
	bc = snprintf(cache_key, MAX_KEY_LEN, EXT_PART"%.*s"STATE_PART, extno->len, extno->s);
	if (bc > 0 && bc < MAX_KEY_LEN) {
		redis_del(mem, cache_key, bc);
	}

	/* remove busycalls counter */
	bc = snprintf(cache_key, MAX_KEY_LEN, EXT_PART"%.*s."BUSY, extno->len, extno->s);
	if (bc > 0 && bc < MAX_KEY_LEN) {
		redis_del(mem, cache_key, bc);
	}

	/* remove ringingcalls counter */
	bc = snprintf(cache_key, MAX_KEY_LEN, EXT_PART"%.*s."RINGING, extno->len, extno->s);
	if (bc > 0 && bc < MAX_KEY_LEN) {
		redis_del(mem, cache_key, bc);
	}

} /* }}} delete_extension_presence_state */

/* void update_extension_publish(struct redis_client *mem, str *extno) {{{
 * Use this to update if possible the extension's state */
const char *update_extension_publish(struct redis_client *mem, str extno, const char *presence_state)
{
	int bc = -1;
	redis_return_t err = -1;
	const char *newstate;
	if (!mem || !extno.s || !presence_state)
		return NULL;
	/* 4PSA: if this publish is from the server (on behalf of the extension)
	 * we know the extension state changed
	 * what we don't know yet is if the new state is in parallel with another one
	 * (parallel lines)
	 * for instance, if line 2 hangs up, the new state goes to online
	 * but if line 1 is still in a call the extension should remain busy */
	if ((newstate = presentity_allow_state_switch(mem, extno, presence_state))) {
		/* 4PSA: this change is allowed, settup the presence state in redis */
		bc = snprintf(cache_key, MAX_KEY_LEN, EXT_PART"%.*s"STATE_PART, extno.len, extno.s);
		if (bc > 0 && bc < MAX_KEY_LEN) {
			err = redis_set(mem, cache_key, bc, newstate, strlen(newstate) + 1, MAKE_NON_SEMIDURABLE(PHONE_STATE_TTL));
			if (err && (err != REDIS_RET_TTL_ALREADY_SET)) {
				LM_ERR("Error while updating presence state in redis (%d:%s)\n", err, redis_strerror(err));
			}
		}
		return newstate;
	}
	
	return NULL;
} /* }}} update_extension_publish */

#define RED_SET_STR(str, ttl) { if (str.s && (str.len > 0)) { err = redis_set(mem, cache_key, bc, str.s, str.len, ttl);} else { err = redis_set(mem, cache_key, bc, "__NULL", strlen("__NULL"), ttl); } }
#define CONTACT_TTL 600

void insert_cached_contact(struct redis_client *mem, str *extno, ucontact_t *c) /* {{{ */ {
	int bc = -1;
	redis_return_t err = -1;
	
	bc = snprintf(cache_key, MAX_KEY_LEN, EXT_PART"%.*s"CONTACT_PART, extno->len, extno->s);
	if (bc > 0 && bc < MAX_KEY_LEN) {
		RED_SET_STR(c->c, CONTACT_TTL);
	}

	bc = snprintf(cache_key, MAX_KEY_LEN, EXT_PART"%.*s"EXPIRES_PART, extno->len, extno->s);
	if (bc > 0 && bc < MAX_KEY_LEN) {
		redis_del(mem, cache_key, bc);
	}

	bc = snprintf(cache_key, MAX_KEY_LEN, EXT_PART"%.*s"RECEIVED_PART, extno->len, extno->s);
	if (bc > 0 && bc < MAX_KEY_LEN) {
		RED_SET_STR(c->received, CONTACT_TTL);
	}

	bc = snprintf(cache_key, MAX_KEY_LEN, EXT_PART"%.*s"SOCKET_PART, extno->len, extno->s);
	if (bc > 0 && bc < MAX_KEY_LEN) {
		RED_SET_STR(c->sock->sock_str, CONTACT_TTL);
	}
	
	bc = snprintf(cache_key, MAX_KEY_LEN, EXT_PART"%.*s"CFLAGS_PART, extno->len, extno->s);
	if (bc > 0 && bc < MAX_KEY_LEN) {
		snprintf(cache_value, MAX_KEY_LEN, "%d", c->cflags);
		err = redis_set(mem, cache_key, bc, cache_value, strlen(cache_value), CONTACT_TTL);
	}

} /* }}} insert_cached_contact */

void update_cached_contact(struct redis_client *mem, str *extno, ucontact_t *c) /* {{{ */ {
	int bc = -1;
	redis_return_t err = -1;

	bc = snprintf(cache_key, MAX_KEY_LEN, EXT_PART"%.*s"EXPIRES_PART, extno->len, extno->s);
	if (bc > 0 && bc < MAX_KEY_LEN) {
		redis_del(mem, cache_key, bc);
	}

	bc = snprintf(cache_key, MAX_KEY_LEN, EXT_PART"%.*s"RECEIVED_PART, extno->len, extno->s);
	if (bc > 0 && bc < MAX_KEY_LEN) {
		RED_SET_STR(c->received, CONTACT_TTL);
	}

	bc = snprintf(cache_key, MAX_KEY_LEN, EXT_PART"%.*s"SOCKET_PART, extno->len, extno->s);
	if (bc > 0 && bc < MAX_KEY_LEN) {
		RED_SET_STR(c->sock->sock_str, CONTACT_TTL);
	}

	bc = snprintf(cache_key, MAX_KEY_LEN, EXT_PART"%.*s"CFLAGS_PART, extno->len, extno->s);
	if (bc > 0 && bc < MAX_KEY_LEN) {
		snprintf(cache_value, MAX_KEY_LEN, "%d", c->cflags);
		err = redis_set(mem, cache_key, bc, cache_value, strlen(cache_value), CONTACT_TTL);
	}

} /* }}} update_cached_contact */

void delete_cached_contact(struct redis_client *mem, str *extno) /* {{{ */ {
	int bc = -1;
	
	bc = snprintf(cache_key, MAX_KEY_LEN, EXT_PART"%.*s"CONTACT_PART, extno->len, extno->s);
	if (bc > 0 && bc < MAX_KEY_LEN) {
		redis_del(mem, cache_key, bc);
	}

	bc = snprintf(cache_key, MAX_KEY_LEN, EXT_PART"%.*s"EXPIRES_PART, extno->len, extno->s);
	if (bc > 0 && bc < MAX_KEY_LEN) {
		redis_del(mem, cache_key, bc);
	}

	bc = snprintf(cache_key, MAX_KEY_LEN, EXT_PART"%.*s"RECEIVED_PART, extno->len, extno->s);
	if (bc > 0 && bc < MAX_KEY_LEN) {
		redis_del(mem, cache_key, bc);
	}

	bc = snprintf(cache_key, MAX_KEY_LEN, EXT_PART"%.*s"SOCKET_PART, extno->len, extno->s);
	if (bc > 0 && bc < MAX_KEY_LEN) {
		redis_del(mem, cache_key, bc);
	}
	
	bc = snprintf(cache_key, MAX_KEY_LEN, EXT_PART"%.*s"CFLAGS_PART, extno->len, extno->s);
	if (bc > 0 && bc < MAX_KEY_LEN) {
		redis_del(mem, cache_key, bc);
	}

} /* }}} delete_cached_contact */

void update_presentity_watcherexpire(struct redis_client *mem, str pres_uri, int expire)
{
	int bc = -1;
	char *value = NULL;

	bc = snprintf(cache_key, MAX_KEY_LEN, EXT_PART"%.*s"ACTIVEWATCHERS_PART, pres_uri.len, pres_uri.s);
	if (bc > 0 && bc < MAX_KEY_LEN) {
		redis_return_t err = -1;
		size_t len  = 0;
		int pres_time = 0;
		/* get curr state from redis only; key has no \0 counted to it */
		err = redis_get_whole(mem, cache_key, bc, &value, &len, "\0", 1);
		/* parse the time_t entry */
		if ((err == REDIS_RET_CONNECTION_FAILURE) || (err == REDIS_RET_SERVER_DOWN))
			goto error;
		if (err == REDIS_RET_SUCCESS) {
			if (len < sscanf(value, "%d", &pres_time)) {
				goto error;
			}
		}

		expire = (expire > time(NULL))? expire : expire + time(NULL);
		
		/* update the key if required */
		if (expire > pres_time) {
			snprintf(cache_value, MAX_KEY_LEN, "%d", expire);
			err = redis_set(mem, cache_key, bc, cache_value, strlen(cache_value), 0);
		}
	}

error:
	if (value) free(value);
}

unsigned char presentity_has_active_watchers(struct redis_client *mem, str pres_uri)
{
	int bc = -1;
	char *value = NULL;
	int ret = 1; /* watchers found */
	
	bc = snprintf(cache_key, MAX_KEY_LEN, EXT_PART"%.*s"ACTIVEWATCHERS_PART, pres_uri.len, pres_uri.s);
	if (bc > 0 && bc < MAX_KEY_LEN) {
		redis_return_t err = -1;
		size_t len  = 0;
		int pres_time = 0;
		/* get curr state from redis only; key has no \0 counted to it */
		err = redis_get_whole(mem, cache_key, bc, &value, &len, "\0", 1);
		if ((err == REDIS_RET_CONNECTION_FAILURE) || (err == REDIS_RET_SERVER_DOWN)) {
			/* if Redis server is down consider that there are watchers */
			goto error;
		}
		if (err == REDIS_RET_SUCCESS) {
			if (len < sscanf(value, "%d", &pres_time)) {
				/* also if there is not a valid value defaults to having watchers */
				goto error;
			}
		}
		if (pres_time < time(NULL)) {
			/* the last watcher expired or there was none set */
			ret = 0;
		}
	}

error:

	LM_DBG("Presence extension %.*s %s watchers\n",pres_uri.len, pres_uri.s, (ret)? "HAS" : "DOES NOT HAVE");
	if (value) free(value);
	return ret;
}

/**
 * \brief Determines whether an event present in redis or change_audit table has changed it's id (db_id > private_id)
 * If the private value is smaller than the redis value it is updated to the stored value
 * If the key does not exists in redis or private_id > redis_id then the database will be queried
 * and the private AND redis value will be set to the database value
 * In any case if the private value is updated or if the database is queried it will be considered an event change
 *  
 * \param mem redis connection
 * \param dbf database api
 * \param dbcon database connection handle
 * \param event event name as found in database table
 * \param rediskey event name as found in redis
 * \param event_id private event id stored by the process. Will be checked and updated with the redis/db value
 * 
 * \return 1 if a change is detected (db_id > private_id), 0 otherwise
 */
unsigned char check_set_event(struct redis_client *mem, db_func_t *dbf, db_con_t *dbcon, const char *event, const char *rediskey, int *event_id)
{
	int bc = -1, current_id = 0;
	char *value      = NULL;
	db_res_t *result = NULL;
	db_val_t *val    = NULL;
	str query = {0, -1};
	unsigned char changed = 0;
	redis_return_t err = -1;

	if (!mem || !dbf || !dbcon || !event || !rediskey || !event_id)
		goto done;
	
	LM_DBG("Checking if %s event changed.", event);
		
	bc = strlen(rediskey);
	if (bc > 0 && bc < MAX_KEY_LEN) {
		size_t len  = 0;
		err = redis_get_whole(mem, rediskey, bc, &value, &len, "\0", 1);
		if (err == REDIS_RET_SUCCESS) {
			if ((len > 0) && (sscanf(value, "%d", &current_id) > 0)) {
				/* if the event key exists check if this needs update */
				if (*event_id == current_id) {
					goto done;
				}
				if (*event_id < current_id) {
					*event_id = current_id;
					changed   = 1; /* we need to update our event related information */
					goto done;
				}
			}
		}
	}
	
	/* fallback to query the database */
	query.len = asprintf(&query.s, "SELECT change_audit.update FROM change_audit WHERE event=\"%s\" AND client_id IS NULL", event);
	if (query.len <= 0) goto done;

	LM_INFO("Quering table change_audit for %s event id", event);

	if (dbf->raw_query (dbcon, &query, &result) < 0) {
		LM_WARN("failed to query table change_audit for %s event id", event);
	}

	/* compute the new id value from DB */
	if ( result && RES_ROWS(result) && ROW_N(RES_ROWS(result)) &&
		(val = ROW_VALUES(RES_ROWS(result))) && (val->type == DB_INT)) {
		changed = 1; /* we need to update our event related information */
		current_id = val->val.int_val;
		*event_id  = current_id;
	}
	
	/* create the key in redis in any case */
	snprintf(cache_value, MAX_VALUE_LEN, "%d", current_id);
	if ((err = redis_set(mem, rediskey, bc, cache_value, strlen(cache_value), EVENT_UPDATE_TTL)) != REDIS_RET_SUCCESS) {
		LM_WARN("Redis %s key could not be set to %s. Libredis error:%d", rediskey, cache_value, (int)err);
	}

done:
	if (result)  dbf->free_result(dbcon, result);
	if (value)   free(value);
	if (query.s) free(query.s);
	if (changed) LM_INFO("Event %s changed. Current private value is set to %d.", event, *event_id);
	return changed;
}

/*!\fn int localip(const char * ip, const char * net, const char * mask) {{{
 * \brief Determine whether ip is part of the net:mask subnet or not.
 *  
 * \param ip IP
 * \param net network address
 * \param mask mask
 * 
 * \return 1 if it does
 * \return 0 otherwise
 */
int localip(const char *ip, const char *net, const char *mask) {
	struct in_addr intip, intnet, intmask;
	
	
	if (!inet_aton(ip, &intip) || !inet_aton(net, &intnet)\
			|| !inet_aton(mask, &intmask)) {

		LM_DBG("Non ip given; considering extern (%s, %s, %s)\n", ip, net, mask);
		return 0;
	}
	if ((intip.s_addr & intmask.s_addr) == (intnet.s_addr & intmask.s_addr)) return 1;
	return 0;
} /* }}} localip */

str *pkg_strcpy(str *dst, str *src)
{
	int len = src->len; 

	if (dst->s) {
		pkg_free(dst->s);
		dst->s = NULL;
		dst->len = 0;
	}
		
	if (!len) {
		return dst;
	}
	
	dst->len = len;
	dst->s = (char *)pkg_malloc(len + 1);
	if (dst->s == NULL) {
		LM_ERR("no more pkg memory!\n");
		return NULL;
	}
	memcpy(dst->s, src->s, len);
	dst->s[len] = 0;
	return dst;
}

