/*
 * 4PSA VoipNow - Miscellaneous functions
 *
 * Copyright (c) 2005-2009 Rack-Soft, LLC. All rights reserved.
 *
 * $Header: //itsd/devel/platform/voipnow/251/low_level/kamailio_extra_sources/1.5.4/misc_4psa.h#3 $
 *
 * Modified: $DateTime: 2011/05/06 14:54:01 $ by $Author: mihaim $ with $Change: 79900 $
 */

#ifndef __MISC_4PSA__
#define __MISC_4PSA__

#include "ip_addr.h"
#include "modules/usrloc/ucontact.h"

#include "str.h"
#include "db/db.h"

#define ST_OFFLINE_S	"offline"
#define ST_RINGING_S	"ringing"
#define ST_BUSY_S		"busy"
#define ST_ONLINE_S		"online"
#define ST_REGISTER_S		"register"

#define ST_TERMINATED_S	"terminated"
#define ST_CONFIRMED_S	"confirmed"
#define ST_TRYING_S	"trying"
#define ST_EARLY_S	"early"

#define RINGING "ringingcalls"
#define BUSY	"busycalls"

#define EXT_PART "extension."
#define STATE_PART ".state"
#define PUBLISH_PART ".extension_publish"

#define EXPIRES_PART ".expires"
#define CONTACT_PART ".contact"
#define RECEIVED_PART ".received"
#define SOCKET_PART ".socket"
#define CFLAGS_PART ".cflags"

#define ACTIVEWATCHERS_PART ".active_watchers_expire"

enum ext_state_t {
	ST_OFFLINE = 0,
	ST_RINGING,
	ST_BUSY,
	ST_ONLINE,
	ST_REGISTER,
	ST_ERR = -1
};

#define PHONE_STATE_TTL	36000

#define EXTENSION_PUBLISH_TTL 14400

#define EVENT_UPDATE_TTL 84000

/* fix for using older libredis */
#ifndef MAKE_NON_SEMIDURABLE
#warning "Please use the new 4psa redis library for non semidurable entries"
#define MAKE_NON_SEMIDURABLE(ttl) ( ((long int)(ttl)) | (int)(1<<31) ) /**< To be used to set TTL for non-semidurable entries. */
#endif


int get_extension_publish(struct redis_client *mem, str extno, db_func_t *dbf, db_con_t *dbcon);

void reset_cached_extension_publish();

void reset_extension_presence_state(ucontact_t *c, int type, void *param);
void delete_extension_presence_state(struct redis_client *mem, str *extno);
const char *update_extension_publish(struct redis_client *mem, str extno, const char *presence_state);
const char *convert_presence_state_to_dlginfo(const char *presence_state);

const char *presentity_allow_state_switch(struct redis_client *mem, str extno, const char * newstate);
void insert_cached_contact(struct redis_client *mem, str *extno, ucontact_t *c);
void update_cached_contact(struct redis_client *mem, str *extno, ucontact_t *c);
void delete_cached_contact(struct redis_client *mem, str *extno);

/* The API is for setting the last expire for a watcher based on a presentity uri */
void update_presentity_watcherexpire(struct redis_client *mem, str pres_uri, int expire);
unsigned char presentity_has_active_watchers(struct redis_client *mem, str pres_uri);

/**
 * \brief Determines whether an event present in redis or change_audit table has changed it's id (db_id > private_id)
 * If the private value is smaller than the redis value it is updated to the stored value
 * If the key does not exists in redis or private_id > redis_id then the database will be queried
 * and the private AND redis value will be set to the database value
 * In any case if the private value is updated or if the database is queried it will be considered an event change
 *  
 * \param mem redis connection
 * \param dbf database api
 * \param dbcon database connection handle
 * \param event event name as found in database table
 * \param rediskey event name as found in redis
 * \param event_id private event id stored by the process. Will be checked and updated with the redis/db value
 * 
 * \return 1 if a change is detected (db_id > private_id), 0 otherwise
 */
unsigned char check_set_event(struct redis_client *mem, db_func_t *dbf, db_con_t *dbcon, const char *event, const char *rediskey, int *event_id);

/*!\fn int localip(const char * ip, const char * net, const char * mask)
 * \brief Determine whether ip is part of the net:mask subnet or not.
 *  
 * \param ip IP
 * \param net network address
 * \param mask mask
 * 
 * \return 1 if it does
 * \return 0 otherwise
 */
int localip(const char *ip, const char *net, const char *mask);

str *pkg_strcpy(str *dst, str *src);


#endif /* __MISC_4PSA__ */

