/*
 * 4PSA VoipNow - VoipNow authentication module
 *
 * Copyright (c) 2005-2009 Rack-Soft, LLC. All rights reserved.
 *
 * $Header: //itsd/devel/platform/voipnow/251/low_level/kamailio_extra_sources/1.5.4/modules/auth_vn/auth_vn.c#1 $
 *
 * Modified: $DateTime: 2010/09/28 18:16:02 $ by $Author: bogdanc $ with $Change: 71991 $
 */


#include <stdio.h>
#include <string.h>

#include "../../sr_module.h"
#include "../../db/db.h"
#include "../../dprint.h"
#include "../../error.h"
#include "../../mod_fix.h"
#include "../../mem/mem.h"
#include "../auth/api.h"
#include "../sl/sl_api.h"
#include "../auth_db/aaa_avps.h"

#include "auth_vn_cmds.h"


MODULE_VERSION


#define DEFAULT_CRED_LIST "rpid"

#define USER_COL "username"
#define USER_COL_LEN (sizeof(USER_COL) - 1)

#define DOMAIN_COL "domain"
#define DOMAIN_COL_LEN (sizeof(DOMAIN_COL) - 1)



/*
 * Module destroy function prototype
 */
static void destroy(void);


/*
 * Module child-init function prototype
 */
static int child_init(int rank);


/*
 * Module initialization function prototype
 */
static int mod_init(void);

/*
 * Module parameter variables
 */
static str db_url = {DEFAULT_RODB_URL, DEFAULT_RODB_URL_LEN};
int calc_ha1                = 0;
int use_domain              = 0; /* Use also domain when looking up in table */


db_con_t* auth_vn_db_handle = 0; /* database connection handle */
db_func_t auth_vn_dbf;
struct redis_client *pa_mem = NULL;

char *credentials_list      = DEFAULT_CRED_LIST;
struct aaa_avp *credentials = 0; /* Parsed list of credentials to load */
int credentials_n           = 0; /* Number of credentials in the list */
str user_column             = {USER_COL, USER_COL_LEN};
str domain_column           = {DOMAIN_COL, DOMAIN_COL_LEN};

/*
 * Exported functions
 */
static cmd_export_t cmds[] = {
	{"vn_authorize",   (cmd_function)auth_vn_authorize_cfg, 0,\
		0, 0, REQUEST_ROUTE},
	{0, 0, 0, 0, 0, 0}
};

/*
 * Exported parameters
 */
static param_export_t params[] = {
	{"db_url",            STR_PARAM, &db_url.s           },
	{"user_column",       STR_PARAM, &user_column.s      },
	{"domain_column",     STR_PARAM, &domain_column.s    },
	{"calculate_ha1",     INT_PARAM, &calc_ha1           },
	{"load_credentials",  STR_PARAM, &credentials_list   },
	{"use_domain",        INT_PARAM, &use_domain         },
	{0, 0, 0}
};

/*
 * Module interface
 */
struct module_exports exports = {
	"auth_vn", 
	DEFAULT_DLFLAGS, /* dlopen flags */
	cmds,       /* Exported functions */
	params,     /* Exported parameters */
	0,          /* exported statistics */
	0,          /* exported MI functions */
	0,          /* exported pseudo-variables */
	0,          /* extra processes */
	mod_init,   /* module initialization function */
	0,          /* response function */
	destroy,    /* destroy function */
	child_init  /* child initialization function */
};



auth_api_t auth_api;

/** SL binds */
struct sl_binds slb;


static int child_init(int rank) {
	auth_vn_db_handle = auth_vn_dbf.init(&db_url);
	if (auth_vn_db_handle == 0){
		LM_ERR("unable to connect to the database\n");
		return -1;
	}

	pa_mem = auth_vn_dbf.red_init();
	if (pa_mem == 0){
		LM_WARN("unable to connect to the redis server(s)\n");
		return 0;
	}

	return 0;
}

static int mod_init(void) {

	bind_auth_t bind_auth;

	db_url.len = strlen(db_url.s);

	/* Find a database module */
	if (db_bind_mod(&db_url, &auth_vn_dbf) < 0){
		LM_ERR("unable to bind to a database driver\n");
		return -1;
	}

	/* bind to auth module and import the API */
	bind_auth = (bind_auth_t)find_export("bind_auth", 0, 0);
	if (!bind_auth) {
		LM_ERR("unable to find bind_auth function. Check if you load the auth module.\n");
		return -2;
	}

	if (bind_auth(&auth_api) < 0) {
		LM_ERR("unable to bind auth module\n");
		return -3;
	}

	/* load the SL API */
	if (load_sl_api(&slb)!=0) {
		LM_ERR("can't load SL API\n");
		return -1;
	}

	/* process additional list of credentials */
	if (parse_aaa_avps( credentials_list, &credentials, &credentials_n)!=0) {
		LM_ERR("failed to parse credentials\n");
		return -5;
	}

	return 0;
}

static void destroy(void) {
	if (auth_vn_db_handle) {
		auth_vn_dbf.close(auth_vn_db_handle);
		auth_vn_db_handle = 0;
	}
	if (pa_mem) {
		auth_vn_dbf.red_close(pa_mem);
		pa_mem = NULL;
	}
}
