/*
 * 4PSA VoipNow - Perform authentication using the UDF MySQL functions vngetpass and vnauth.
 *
 * Copyright (c) 2005-2009 Rack-Soft, LLC. All rights reserved.
 *
 * $Header: //itsd/devel/platform/voipnow/251/low_level/kamailio_extra_sources/1.5.4/modules/auth_vn/auth_vn_api.c#1 $
 *
 * Modified: $DateTime: 2010/09/28 18:16:02 $ by $Author: bogdanc $ with $Change: 71991 $
 */

#include <stdio.h>
#include <string.h>

#include "../../sr_module.h"
#include "../../db/db.h"
#include "../../dprint.h"
#include "../../error.h"
#include "../../mod_fix.h"
#include "../../mem/mem.h"
#include "../../parser/digest/digest.h"
#include "../../parser/hf.h"
#include "../../parser/parser_f.h"
#include "../auth_db/aaa_avps.h"

#include "auth_vn.h"
#include "auth_vn_api.h"

str *get_pass(str *username) {
	db_res_t* res = NULL;
	db_row_t row;
	str query = {0, 0};
	size_t resp_len = 0;

	str *ret = NULL;

#define QUERY_FMT "CALL vngetpass('%.*s', 'plain', 'kamailio')"

	/* counting out the length of the %.*s plus the terminated 0 from sizeof */
	query.len = username->len + sizeof(QUERY_FMT) - 5;

	query.s = (char *)pkg_malloc(query.len + 1);
	if (!query.s) {
		LM_ERR("not enough memory!");
		goto _end;
	}
	snprintf(query.s, query.len + 1, QUERY_FMT, username->len, username->s);

#undef QUERY_FMT

	
	if (auth_vn_dbf.raw_query(auth_vn_db_handle, &query, &res) != 0 || !res) {
		LM_ERR("query unsuccessful: %.*s\n", query.len, query.s);
		goto _end;
	}

    if (RES_ROW_N(res) < 1) goto _end;

    row = RES_ROWS(res)[0];
	if (!row.values || (row.n != 2)) {
		LM_ERR("Malformed query\n");
		goto _end;
	}

	resp_len = strlen(row.values[0].val.string_val);
	ret = (str *)pkg_malloc(sizeof(str));
	ret->s = (char *)pkg_malloc(resp_len + 1);
	ret->len = resp_len;
	memcpy(ret->s, row.values[0].val.string_val, resp_len);
	ret->s[resp_len] = 0;

_end:	

	if (res) auth_vn_dbf.free_result(auth_vn_db_handle, res);

	return ret;
}


int get_ha1(struct username* _username, str* _domain,
			  char* _ha1) {
	str *pass = NULL;
	int ret = -1;

	pass = get_pass(&_username->user);
	if (!pass) goto _end;

	if (calc_ha1) {
		/* Only plaintext passwords are stored in database,
		 * we have to calculate HA1 */
		auth_api.calc_HA1(HA_MD5, &_username->whole, _domain, pass,
				0, 0, _ha1);
		LM_DBG("HA1 string calculated: %s\n", _ha1);
	} else {
		memcpy(_ha1, pass->s, pass->len);
		_ha1[pass->len] = '\0';
	}

	ret = 0;

_end:
	
	if (pass) {
		pkg_free(pass->s);
		pkg_free(pass);
	}

	return ret;
}

int get_subscriber_data(struct username* _username, str* _domain, db_res_t** res) {

	struct aaa_avp *cred;
	db_key_t keys[2];
	db_val_t vals[2];
	db_key_t *col = NULL;
	str table = {"ser_subscriber", sizeof("ser_subscriber") - 1};
	db_redq_pair_t *memkeys = NULL;
	int n, nc;

	col = pkg_malloc(sizeof(*col) * (credentials_n + 1));
	if (col == NULL) {
		LM_ERR("no more pkg memory\n");
		return -1;
	}

	memkeys = pkg_malloc(sizeof(*memkeys) * (credentials_n + 2));
	if (memkeys == NULL) {
		LM_ERR("no more pkg memory\n");
		return -1;
	}
	memset(memkeys, 0, sizeof(*memkeys) * (credentials_n + 2));

	keys[0] = &user_column;
	keys[1] = &domain_column;
	
	memkeys[0].memkey = (char*)pkg_malloc((strlen("extension.")+_username->user.len+strlen(".ser_password") + 1)*sizeof(char)) ;
	if (memkeys[0].memkey == NULL){
		LM_ERR("no more pkg memory\n");
		return -1;
	}
	
	memcpy(memkeys[0].memkey,"extension.",strlen("extension."));
	memcpy(memkeys[0].memkey+strlen("extension."),_username->user.s,_username->user.len);
	memcpy(memkeys[0].memkey+strlen("extension.")+_username->user.len,".ser_password",strlen(".ser_password"));
	memkeys[0].memkey[strlen("extension.")+_username->user.len+strlen(".ser_password")]='\0';
	LM_DBG("memkey: %s\n",memkeys[0].memkey);
	memkeys[0].col = 0;
	memkeys[0].type = DB_STRING;
	memkeys[0].setkey = NULL;
	memkeys[0].flag = 0;

	for (n = 0, cred=credentials; cred ; n++, cred=cred->next) {
		col[n] = &cred->attr_name;
		
		memkeys[1 + n].memkey = (char*)pkg_malloc((strlen("extension.")+_username->user.len+1+cred->attr_name.len + 1)*sizeof(char)) ;
		if (memkeys[1 + n].memkey == NULL){
			LM_ERR("no more pkg memory to re-allocate \n");
			return -1;
		}

		memcpy(memkeys[1 + n].memkey,"extension.",strlen("extension."));
		memcpy(memkeys[1 + n].memkey+strlen("extension."),_username->user.s,_username->user.len);
		memcpy(memkeys[1 + n].memkey+strlen("extension.")+_username->user.len,".",1);
		memcpy(memkeys[1 + n].memkey+strlen("extension.")+_username->user.len+1,cred->attr_name.s,cred->attr_name.len);
		memkeys[1 + n].memkey[strlen("extension.")+_username->user.len+1+cred->attr_name.len]='\0';
		LM_DBG("memkey: %s\n",memkeys[1 + n].memkey);
		memkeys[1 + n].col = 1 + n;
		memkeys[1 + n].type = DB_STRING;
		memkeys[1 + n].setkey = NULL;
		memkeys[1 + n].flag = 0;
	}

	memkeys[1 + n].memkey = NULL;

	VAL_TYPE(vals) = VAL_TYPE(vals + 1) = DB_STR;
	VAL_NULL(vals) = VAL_NULL(vals + 1) = 0;

	VAL_STR(vals).s = _username->user.s;
	VAL_STR(vals).len = _username->user.len;

	if (_username->domain.len) {
		VAL_STR(vals + 1) = _username->domain;
	} else {
		VAL_STR(vals + 1) = *_domain;
	}

	n = (use_domain ? 2 : 1);
	nc = credentials_n;
	
	if (auth_vn_dbf.use_table(auth_vn_db_handle, &table) < 0) {
		LM_ERR("failed to use_table\n");
		goto error;
	}

	if (auth_vn_dbf.red_query(pa_mem, memkeys, 7200, auth_vn_db_handle, keys, 0, vals, col, n, nc, 0, res) < 0) {
		LM_ERR("failed to query database\n");
		goto error;
	}
	
	pkg_free(col);
	if (memkeys[0].memkey) pkg_free(memkeys[0].memkey);
	for (n = 0, cred=credentials; cred ; n++, cred=cred->next) 
		if (memkeys[ n + 1 ].memkey) pkg_free(memkeys[ n + 1 ].memkey);
	if (memkeys) pkg_free(memkeys);

	if (RES_ROW_N(*res) == 0) {
		LM_DBG("no result for user \'%.*s@%.*s\'\n",
				_username->user.len, ZSW(_username->user.s),
			(use_domain ? (_domain->len) : 0), ZSW(_domain->s));
		return 1;
	}

	return 0;

error:
	if (col) pkg_free(col);
	if (memkeys[0].memkey) pkg_free(memkeys[0].memkey);
	for (n = 0, cred=credentials; cred ; n++, cred=cred->next) 
		if (memkeys[ n + 1 ].memkey) pkg_free(memkeys[ n + 1 ].memkey);
	if (memkeys) pkg_free(memkeys);
	return -1;	
}
