/*
 * 4PSA VoipNow - VoipNow authentication module config file commands
 *
 * Copyright (c) 2005-2009 Rack-Soft, LLC. All rights reserved.
 *
 * $Header: //itsd/devel/platform/voipnow/251/low_level/kamailio_extra_sources/1.5.4/modules/auth_vn/auth_vn_cmds.c#1 $
 *
 * Modified: $DateTime: 2010/09/28 18:16:02 $ by $Author: bogdanc $ with $Change: 71991 $
 */


#include "../auth/api.h"
#include "../../sr_module.h"
#include "../auth_db/aaa_avps.h"
#include "../../parser/hf.h"

#include "auth_vn.h"
#include "auth_vn_api.h"

static str auth_500_err = str_init("Server Internal Error");

/*
 * Generate AVPs from the database result
 */
static int generate_avps(db_res_t* result) {
	struct aaa_avp *cred;
	int_str ivalue;
	int i;

	for (cred=credentials, i=0; cred; cred=cred->next, i++) {
		switch (result->col.types[i]) {
		case DB_STR:
			ivalue.s = VAL_STR(&(result->rows[0].values[i]));

			if (VAL_NULL(&(result->rows[0].values[i])) ||
			ivalue.s.s == NULL || ivalue.s.len==0)
				continue;

			if (add_avp(cred->avp_type|AVP_VAL_STR,cred->avp_name,ivalue)!=0){
				LM_ERR("failed to add AVP\n");
				return -1;
			}

			LM_DBG("set string AVP \"%s\"/%d = \"%.*s\"\n",
				(cred->avp_type&AVP_NAME_STR)?cred->avp_name.s.s:"",
				(cred->avp_type&AVP_NAME_STR)?0:cred->avp_name.n,
				ivalue.s.len, ZSW(ivalue.s.s));
			break;
		case DB_STRING:
			ivalue.s.s = (char*)VAL_STRING(&(result->rows[0].values[i]));

			if (VAL_NULL(&(result->rows[0].values[i])) ||
			ivalue.s.s == NULL || (ivalue.s.len=strlen(ivalue.s.s))==0 )
				continue;

			if (add_avp(cred->avp_type|AVP_VAL_STR,cred->avp_name,ivalue)!=0){
				LM_ERR("failed to add AVP\n");
				return -1;
			}

			LM_DBG("set string AVP \"%s\"/%d = \"%.*s\"\n",
				(cred->avp_type&AVP_NAME_STR)?cred->avp_name.s.s:"",
				(cred->avp_type&AVP_NAME_STR)?0:cred->avp_name.n,
				ivalue.s.len, ZSW(ivalue.s.s));
			break;
		case DB_INT:
			if (VAL_NULL(&(result->rows[0].values[i])))
				continue;

			ivalue.n = (int)VAL_INT(&(result->rows[0].values[i]));

			if (add_avp(cred->avp_type, cred->avp_name, ivalue)!=0) {
				LM_ERR("failed to add AVP\n");
				return -1;
			}

			LM_DBG("set int AVP \"%s\"/%d = %d\n",
				(cred->avp_type&AVP_NAME_STR)?cred->avp_name.s.s:"",
				(cred->avp_type&AVP_NAME_STR)?0:cred->avp_name.n,
				ivalue.n);
			break;
		default:
			LM_ERR("subscriber table column `%.*s' has unsuported type. "
				"Only string/str or int columns are supported by"
				"load_credentials.\n", result->col.names[i]->len, result->col.names[i]->s);
			break;
		}
	}

	return 0;
}
int auth_vn_authorize_cfg(struct sip_msg* _m) {

	char ha1[256];
	int res;
	struct hdr_field* h;
	auth_body_t* cred;
	auth_result_t ret;
	str domain;
	db_res_t* result = NULL;

	domain.len = 0;
	domain.s = 0;

	ret = auth_api.pre_auth(_m, &domain, HDR_AUTHORIZATION_T, &h);

	if (ret != DO_AUTHORIZATION)
		return ret;

	cred = (auth_body_t*)h->parsed;

	res = get_ha1(&cred->digest.username, &domain, ha1);
	if (res < 0) {
		/* Error while accessing the database */
		if (slb.send_reply(_m, 500, &auth_500_err) == -1) {
			LM_ERR("failed to send 500 reply\n");
		}
		return ERROR;
	}

	/* Recalculate response, it must be same to authorize successfully */
	if (!auth_api.check_response(&(cred->digest),
				&_m->first_line.u.request.method, ha1)) {
		ret = auth_api.post_auth(_m, h);
		if (ret == AUTHORIZED) {
			get_subscriber_data(&cred->digest.username, &domain, &result);
			generate_avps(result);
			auth_vn_dbf.free_result(auth_vn_db_handle, result);
		}
		return ret;
	}

	return INVALID_PASSWORD;

}

