/*
 * {app} - {purpose, description}
 *
 * Copyright (c) 2005-2006 Rack-Soft, LLC. All rights reserved.
 *
 * $Header: //itsd/devel/platform/voipnow/251/low_level/kamailio_extra_sources/1.5.4/modules/avpops/avpops_impl_4psa.c#1 $
 *
 * Modified: $DateTime: 2010/09/28 18:16:02 $ by $Author: bogdanc $ with $Change: 71991 $
 */


#define _GNU_SOURCE

#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <regex.h>
#include <fnmatch.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>


#include <4psa_string.h>

#include "../../ut.h"
#include "../../dprint.h"
#include "../../usr_avp.h"
#include "../../action.h"
#include "../../ip_addr.h"
#include "../../config.h"
#include "../../dset.h"
#include "../../data_lump.h"
#include "../../data_lump_rpl.h"
#include "../../pvar.h"
#include "../../parser/parse_from.h"
#include "../../parser/parse_uri.h"
#include "../../mem/mem.h"
#include "avpops_impl.h"
#include "avpops_db.h"

extern char printbuf[];

/* int expand_memkeys_pv(struct sip_msg* msg, db_redq_pair_t * memkeys) {{{ */
int expand_memkeys_pv(struct sip_msg* msg, db_redq_pair_t * memkeys, char ** save) {
	int ret = -1;
	int i;
	int j = 0;

	for (i = 0; memkeys[i].memkey; ++i) {
		LM_DBG("key %d (%s)\n", i, memkeys[i].memkey);
		expand_pv(msg, &memkeys[i].memkey, '.', &save[j], ALLOC_GLIBC);
		LM_DBG("key (after expand) %d (%s)\n", i, memkeys[i].memkey);
		j++; /* go to the setkey save */

		if (memkeys[i].setkey) {
			LM_DBG("setkey %d (%s)\n", i, memkeys[i].setkey);
			expand_pv(msg, &memkeys[i].setkey, '.', &save[j], ALLOC_GLIBC);
			LM_DBG("setkey (after expand) %d (%s)\n", i, memkeys[i].setkey);
		}
		j++; /* go to the memkey save */
	}

	ret = 0;
/* _end: */

	return ret;
} /* expand_memkeys_pv }}} */

/* int restore_memkeys_pv(db_redq_pair_t * memkeys, char ** save) {{{ */
int restore_memkeys_pv(db_redq_pair_t * memkeys, char ** save) {
	int ret = -1;
	int i;
	int j = 0;

	for (i = 0; memkeys[i].memkey; ++i) {
		if (save[j]) {
			free(memkeys[i].memkey);
			memkeys[i].memkey = save[j];
			save[j] = NULL;
		}
		j++; /* go to the setkey save */
		if (save[j]) {
			free(memkeys[i].setkey);
			memkeys[i].setkey = save[j];
			save[j] = NULL;
		}
		j++; /* go to the memkey save */
	}

	ret = 0;

	return ret;
} /* restore_memkeys_pv }}} */



/* int ops_memquery_avps(struct sip_msg* msg, pv_elem_t* query,pvname_list_t* dest, db_redq_pair_t * memkeys, unsigned ttl) {{{ */
int ops_memquery_avps(struct sip_msg* msg, pv_elem_t* query\
		, pvname_list_t* dest, db_redq_pair_t * memkeys, unsigned ttl) {
	
	int ret = -1;
	int printbuf_len;
	struct red_ctx memctx;
	int n = 0, i = 0;
	char ** save_keys = NULL;

	if(msg==NULL || query==NULL || memkeys==NULL)
	{
		LM_ERR("bad parameters\n");
		return -1;
	}
	
	while (memkeys[i].memkey) {
		n += 2; /* 4PSA: save memkey and setkey as well */
		i++;
	}
	save_keys = calloc(n, sizeof(char *));
	if (!save_keys) {
		LM_ERR("no more malloc mem\n");
		return -1;
	}

	/* make sure to restore after this expanding */
	expand_memkeys_pv(msg, memkeys, save_keys);

	printbuf_len = AVP_PRINTBUF_SIZE-1;
	if(pv_printf(msg, query, printbuf, &printbuf_len)<0 || printbuf_len<=0)
	{
		LM_ERR("cannot print the query\n");
		goto _end;
	}

	LM_DBG("query [%s]\n", printbuf);
	
	memctx.keys = memkeys;
	memctx.ttl = ttl;
	if ((ret = db_query_avp(msg, printbuf, dest, &memctx)) !=0 ) goto _end;

	ret = 1;
_end:
	restore_memkeys_pv(memkeys, save_keys);
	free(save_keys);

	return ret;
} /* ops_memquery_avps }}} */


/* int ops_comasplit_avps(struct sip_msg* msg, char * src, pvname_list_t* n, pvname_list_t* dest) {{{ */
int ops_comasplit_avps(struct sip_msg* msg, char * src, pvname_list_t* n, pvname_list_t* dest) {

	int ret = -1;
	safestr_t * spl = NULL;
	int i, j;
	int_str avp_val_n;
	int_str avp_name_n;
	unsigned short avp_type_n = 0;
	int_str avp_val;
	int_str avp_name;
	unsigned short avp_type = 0;
	char * another_stupid_copy = NULL;
	/* char * save_key = NULL; */


	if(msg == NULL || n == NULL || dest == NULL)
	{
		LM_ERR("bad parameters\n");
		return -1;
	}

	LM_DBG("fixup complete. do the work\n");
	another_stupid_copy = strdup(src);
	expand_pv(msg, &another_stupid_copy, ',', NULL, ALLOC_GLIBC);
	LM_DBG("working on: %s\n", another_stupid_copy);

	if(pv_get_avp_name(msg, &dest->sname.pvp, &avp_name, &avp_type)!=0) {
		LM_ERR("cant get avp name\n");
		goto _end;
	}

	spl = safestr_split_bychar(SAFESTR_TEMP(another_stupid_copy), ',');
	for (i = 0; spl[i]; ++i) {
		safestr_trim(spl[i], SAFESTR_TRIM_BOTH);
	}


	/* we are aliasing those, since they will be copied again in shm */
	for(--i, j = 0; i >= 0; --i) {
		LM_DBG("value splinter: %s\n", (const char *)spl[i]);
		if (!((const char *)spl[i])[0]) continue;

		avp_type |= AVP_VAL_STR;
		avp_val.s.s = (char *)spl[i];
		avp_val.s.len=safestr_length(spl[i]);

		if(add_avp(avp_type, avp_name, avp_val)!=0) {
			LM_ERR("unable to add avp\n");
			goto _end;
		}

		++j;
	}

	if(pv_get_avp_name(msg, &n->sname.pvp, &avp_name_n, &avp_type_n)!=0) {
		LM_ERR("cant get avp name\n");
		goto _end;
	}
	avp_val_n.n = j;
	LM_DBG("aarg 2 will be set to: %d\n", j);
	if(add_avp(avp_type_n, avp_name_n, avp_val_n)!=0) {
		LM_ERR("unable to add avp\n");
		goto _end;
	}
	
	/* sleep(300); */
	ret = 1;
	LM_DBG("good\n");
_end:
	LM_DBG("end\n");
	if (spl) safestr_freelist(spl);
	if (another_stupid_copy) free(another_stupid_copy);
	/* if src is not copied, make sure to save and restore the pv */

	return ret;
} /* ops_comasplit_avps }}} */


inline int ops_checklocalip_avps(struct sip_msg* msg, pv_elem_p ip1, pv_elem_p ip2, pv_elem_p mask) {
	int ret = -1;
	struct in_addr intip1, intip2, intmask;
	int errPrintPV = 0, errCastIP = 0;
	int printbuf_len;

	printbuf_len = AVP_PRINTBUF_SIZE-1;
	if (!errCastIP) errPrintPV |= pv_printf(msg, ip1, printbuf, &printbuf_len);
	if (!errPrintPV) errCastIP |= !inet_aton(printbuf, &intip1);

	printbuf_len = AVP_PRINTBUF_SIZE-1;
	if (!errCastIP) errPrintPV |= pv_printf(msg, ip2, printbuf, &printbuf_len);
	if (!errPrintPV) errCastIP |= !inet_aton(printbuf, &intip2);

	printbuf_len = AVP_PRINTBUF_SIZE-1;
	if (!errCastIP) errPrintPV |= pv_printf(msg, mask, printbuf, &printbuf_len);
	if (!errPrintPV) errCastIP |= !inet_aton(printbuf, &intmask);

	if (errPrintPV) {
		LM_NOTICE("cannot resolve one of the ips from cfg data to real string value\n");
		goto _end;
	}
	if (errCastIP) {
		LM_NOTICE("this is not a valid ip: %s\n", printbuf);
		goto _end;
	}
	
	/* XXX - don't ret 0 !! (cfg route route will exit or something like that) */
	if ((intip1.s_addr & intmask.s_addr) == (intip2.s_addr & intmask.s_addr)) ret = 1;
	else ret = -1;

_end:

	return ret;
}
