/*
 * 4PSA VoipNow - Caching module
 *
 * Copyright (c) 2005-2009 Rack-Soft, LLC. All rights reserved.
 *
 * $Header: //itsd/devel/platform/voipnow/251/low_level/kamailio_extra_sources/1.5.4/modules/cache/cache.c#1 $
 *
 * Modified: $DateTime: 2010/09/28 18:16:02 $ by $Author: bogdanc $ with $Change: 71991 $
 */

#include <libredis/redis.h>

#include "../../sr_module.h"
#include "cache_fixup.h"
#include "cache_api.h"
#include "cache_cmds.h"
#include "cache_bind.h"

#include "../../4psa_mem_wrappers.h"

/* List of redis servers */
extern struct redisurl_list_t * g_mem_list;

/* Cache server handler */
struct redis_client *cache_handle = NULL;

static int cache_mod_init(void);

MODULE_VERSION


/*! \brief
 * MySQL database module interface
 */
static cmd_export_t cmds[] = {
	{"bind_cache",   (cmd_function)bind_cache,   1,     0,         0,  0},
	{"cache_get",            (cmd_function)cache_get_cfg,          2,
	fixup_char_pvar, fixup_free_char_pvar,
		REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|LOCAL_ROUTE},
	{"cache_set",            (cmd_function)cache_set_cfg,          3,
	fixup_char_char_uint, 0,
		REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|LOCAL_ROUTE},
	{"cache_inc",            (cmd_function)cache_inc_cfg,          1,
	0, 0,
		REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|LOCAL_ROUTE},
	{"cache_incby",            (cmd_function)cache_incby_cfg,          2,
	fixup_char_uint, 0,
		REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|LOCAL_ROUTE},
	{"cache_dec",            (cmd_function)cache_dec_cfg,          1,
	0, 0,
		REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|LOCAL_ROUTE},
	{"cache_decby",            (cmd_function)cache_decby_cfg,          2,
	fixup_char_uint, 0,
		REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|LOCAL_ROUTE},
	{"cache_del",            (cmd_function)cache_del,          1,
	0, 0,
		REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|LOCAL_ROUTE},
	{"cache_sadd",            (cmd_function)cache_sadd_cfg,          2,
	0, 0,
		REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|LOCAL_ROUTE},
	{"cache_srem",            (cmd_function)cache_srem_cfg,          2,
	0, 0,
		REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|LOCAL_ROUTE},
	{"cache_scard",            (cmd_function)cache_scard_cfg,          2,
	fixup_char_pvar, fixup_free_char_pvar,
		REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|LOCAL_ROUTE},
	{"cache_sismember",        (cmd_function)cache_sismember_cfg,          2,
	0,0,
		REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|LOCAL_ROUTE},
	{"cache_smembers",        (cmd_function)cache_smembers_cfg,          2,
	fixup_char_pvlist, 0,
		REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|LOCAL_ROUTE},
	{0, 0, 0, 0, 0, 0}
};


/*! \brief
 * Exported parameters
 */
static param_export_t params[] = {
	{0, 0, 0}
};


struct module_exports exports = {	
	"cache",
	DEFAULT_DLFLAGS, /* dlopen flags */
	cmds,
	params,          /* module parameters */
	0,               /* exported statistics */
	0,               /* exported MI functions */
	0,               /* exported pseudo-variables */
	0,               /* extra processes */
	cache_mod_init,  /* module initialization function */
	0,               /* response function*/
	0,               /* destroy function */
	0                /* per-child init function */
};


static int cache_mod_init(void) {

	redis_return_t ret = REDIS_RET_ERROR;
	char *url = NULL;
	char *port = NULL;
	int p = 0;
	struct redisurl_list_t * redis_url = g_mem_list;
//	redis_mm_t *allocator = redis_mm_create(malloc_f, calloc_f, realloc_f, free_f);

	/* Initialize the cache handler */
	cache_handle = redis_client_create(REDIS_HASH_MD5, \
		REDIS_DISTRIBUTION_CONSISTENT_KETAMA, NULL, 0);
	if (!cache_handle)
		goto _end;

	if (g_log_redis) {
		struct redis_log_levels levels = {
			.error = __LOG_ERROR,
			.debug = __LOG_DEBUG,
			.event = __LOG_EVENT,
			.notice = __LOG_NOTICE,
			.warning = __LOG_WARNING,
			.none = __LOG_NONE,
		};
		redis_enable_log(cache_handle, redis_log_wrap, &levels, REDIS_LEVEL_ERROR |
			REDIS_LEVEL_DEBUG | REDIS_LEVEL_EVENT | REDIS_LEVEL_NOTICE |
			REDIS_LEVEL_WARNING);
	}

	while (redis_url) {

		url = strdup(redis_url->url);
		port = strrchr(url, ':');

		if (port) {
			*port++ = '\0';
	
			p = atoi(port);
			if (p <= 0 || p > 65535) {
				free(url);
				redis_url = redis_url->next;
				continue;
			}
		}
		
		ret = redis_server_add(cache_handle, url, p, NULL);
		
		if (ret != REDIS_RET_SUCCESS) {
			LM_WARN("Adding cache server %s gave error (%d:%s)\n", redis_url->url, ret, redis_strerror(ret));
		} else {
			LM_DBG("Adding cache server %s was successfull.\n", redis_url->url);
		}
			
		free(url);
		redis_url = redis_url->next;
	}

_end:

	return 0;
}

