/*
 * {app} - {purpose, description}
 *
 * Copyright (c) 2005-2009 Rack-Soft, LLC. All rights reserved.
 *
 * $Header: //itsd/devel/platform/voipnow/251/low_level/kamailio_extra_sources/1.5.4/modules/cache/cache_api.h#1 $
 *
 * Modified: $DateTime: 2010/09/28 18:16:02 $ by $Author: bogdanc $ with $Change: 71991 $
 */



#ifndef CACHE_API_H__
#define CACHE_API_H__

#include <libredis/redis.h>


/* Cache API */

/*!\fn redis_return_t cache_get(char *key, size_t key_size, char **val, size_t *val_size)
 *
 * Fetch the value of a key from the cache server.
 *
 * \param key Key
 * \param key_size Key size
 * \param val Storage space for the value, will be allocated
 * \param val_size Storage space for the value size (can be NULL)
 *
 * \return REDIS_RET_SUCCESS on success
 * \return RED_RET_NOTFOUND if the key is not found
 * \return some other error code in any other case.
 */
redis_return_t cache_get(char *key, size_t key_size, char **val, size_t *val_size);

/*!\fn redis_return_t cache_set(char *key, size_t key_size, char *val, size_t val_size, size_t ttl)
 *
 * Set the value of a key on the cache server.
 *
 * \param key Key
 * \param key_size Key size
 * \param val Value
 * \param val_size Value size
 * \param ttl Lifespan of the key-value pair
 *
 * \return REDIS_RET_SUCCESS on success
 * \return some other error code in any other case.
 */
redis_return_t cache_set(char *key, size_t key_size, char *val, size_t val_size, size_t ttl);

/*!\fn redis_return_t cache_inc(char *key, size_t key_size, long *incremented_value)
 *
 * Increment the value of a key by 1 on the cache server.
 *
 * \param key Key
 * \param key_size Key size
 * \param incremented_value Storage for the incremented value (can be NULL).
 *
 * \return REDIS_RET_SUCCESS on success
 * \return some other error code in any other case.
 */
redis_return_t cache_inc(char *key, size_t key_size, int *incremented_value);

/*!\fn redis_return_t cache_incby(char *key, size_t key_size, long by, int *incremented_value)
 *
 * Increment the value of a key by the specified amount on the cache server.
 *
 * \param key Key
 * \param key_size Key size
 * \param by Amount
 * \param incremented_value Storage for the incremented value (can be NULL).
 *
 * \return REDIS_RET_SUCCESS on success
 * \return some other error code in any other case.
 */
redis_return_t cache_incby(char *key, size_t key_size, long by, int *incremented_value);

/*!\fn redis_return_t cache_dec(char *key, size_t key_size, int *decremented_value)
 *
 * Decrement the value of a key by 1 on the cache server.
 *
 * \param key Key
 * \param key_size Key size
 * \param decremented_value Storage for the decremented value (can be NULL).
 *
 * \return REDIS_RET_SUCCESS on success
 * \return some other error code in any other case.
 */
redis_return_t cache_dec(char *key, size_t key_size, int *decremented_value);

/*!\fn redis_return_t cache_decby(char *key, size_t key_size, long by, int *decremented_value)
 *
 * Decrement the value of a key by the specified amount on the cache server.
 *
 * \param key Key
 * \param key_size Key size
 * \param by Amount
 * \param decremented_value Storage for the decremented value (can be NULL).
 *
 * \return REDIS_RET_SUCCESS on success
 * \return some other error code in any other case.
 */
redis_return_t cache_decby(char *key, size_t key_size, long by, int *decremented_value);

/*!\fn redis_return_t cache_del(char *key, size_t key_size)
 *
 * Remove the value of a key from the cache server.
 *
 * \param key Key
 * \param key_size Key size
 *
 * \return REDIS_RET_SUCCESS on success
 * \return some other error code in any other case.
 */
redis_return_t cache_del(char *key, size_t key_size);

/*!\fn redis_return_t cache_sadd(char *key, size_t key_size, char *val, size_t val_size)
 *
 * Add a value to a set
 *
 * \param key Key
 * \param key_size Key size
 * \param val Value
 * \param val_size Value size
 *
 * \return REDIS_RET_SUCCESS on success
 * \return some other error code in any other case.
 */
redis_return_t cache_sadd(char *key, size_t key_size, char *val, size_t val_size);

/*!\fn redis_return_t cache_srem(char *key, size_t key_size, char *val, size_t val_size)
 *
 * Remove a value from a set
 *
 * \param key Key
 * \param key_size Key size
 * \param val Value
 * \param val_size Value size
 *
 * \return REDIS_RET_SUCCESS on success
 * \return some other error code in any other case.
 */
redis_return_t cache_srem(char *key, size_t key_size, char *val, size_t val_size);

/*!\fn redis_return_t cache_scard(char *key, size_t key_size, int *card)
 *
 * Return the number of elements of a set.
 *
 * \param key Key
 * \param key_size Key size
 * \param card Placeholder for the number of elements.
 *
 * \return REDIS_RET_SUCCESS on success
 * \return some other error code in any other case.
 */
redis_return_t cache_scard(char *key, size_t key_size, int *card);

/*!\fn redis_return_t cache_sismember(char *key, size_t key_size, char *val, size_t val_size)
 *
 * Check whether a value is member of a set or not.
 *
 * \param key Key
 * \param key_size Key size
 * \param val Value
 * \param val_size Value size
 *
 * \return REDIS_RET_SUCCESS on success
 * \return some other error code in any other case.
 */
redis_return_t cache_sismember(char *key, size_t key_size, char *val, size_t val_size);


/*!\fn redis_return_t cache_smembers(char *key, size_t key_size, char ***val, size_t *val_len, \
 *                                 size_t **ind_val_len);
 *
 * Return all the members in a set.
 *
 * \param key Key
 * \param key_size Key size
 * \param val Array of Values
 * \param val_len Number of values
 * \param ind_val_len Array holding the size of each value
 *
 * \return REDIS_RET_SUCCESS on success
 * \return some other error code in any other case.
 */
redis_return_t cache_smembers(char *key, size_t key_size, char ***val, size_t *val_len, \
								size_t **ind_val_len);


#define cache_free(ptr) if (cache_handler) {cache_handler->mm->rfree(ptr); ptr = NULL;}

#endif /* CACHE_API_H__ */

