/*
 * {app} - {purpose, description}
 *
 * Copyright (c) 2005-2009 Rack-Soft, LLC. All rights reserved.
 *
 * $Header: //itsd/devel/platform/voipnow/251/low_level/kamailio_extra_sources/1.5.4/modules/cache/cache_cmds.c#1 $
 *
 * Modified: $DateTime: 2010/09/28 18:16:02 $ by $Author: bogdanc $ with $Change: 71991 $
 */


#include <libredis/redis.h>

#include "cache.h"
#include "cache_api.h"
#include "cache_cmds.h"

#include "../../sr_module.h"
#include "../../pvar.h"


int cache_get_cfg(struct sip_msg* msg, char *key, char *result) {

	int ret = -1;
	pv_spec_t *sp_dest = (pv_spec_t *)result;
	pv_value_t pv_val;
	char *val = NULL;
	size_t val_size = 0;
	redis_return_t err = REDIS_RET_ERROR;

	if (!msg  || !key || !result) goto _end;
	
	err = cache_get(key, strlen(key), &val, &val_size);
	if (!val || (err != REDIS_RET_SUCCESS)) goto _end;
	
	memset(&pv_val, 0, sizeof(pv_value_t));
	pv_val.flags = PV_VAL_STR;
	pv_val.rs.s = val;
	pv_val.rs.len = val_size;

	if (sp_dest->setf(msg, &sp_dest->pvp, (int)EQ_T, &pv_val)<0) {
		LM_ERR("setting profile PV failed\n");
		goto _end;
	}

	ret = 1;

_end:
	

	return ret;

}

int cache_set_cfg(struct sip_msg *msg, char *key, char *value, char *ttl) {

	int ret = -1;

	if (!msg  || !key || !value || !ttl) goto _end;
	
	ret = cache_set(key, strlen(key), value, strlen(value), \
				(unsigned int)(unsigned long)ttl);
	
	if (ret == REDIS_RET_SUCCESS) ret = 1;

_end:

	return ret;

}

int cache_inc_cfg(struct sip_msg *msg, char *key) {

	int ret = -1;

	if (!msg  || !key) goto _end;
	
	ret = cache_inc(key, strlen(key), NULL);
	
	if (ret == REDIS_RET_SUCCESS) ret = 1;

_end:

	return ret;
}

int cache_incby_cfg(struct sip_msg *msg, char *key, char *by) {

	int ret = -1;

	if (!msg  || !key || !by) goto _end;
	
	ret = cache_incby(key, strlen(key), (long)by, NULL);
	
	if (ret == REDIS_RET_SUCCESS) ret = 1;

_end:

	return ret;
}

int cache_dec_cfg(struct sip_msg *msg, char *key) {

	int ret = -1;

	if (!msg  || !key) goto _end;
	
	ret = cache_dec(key, strlen(key), NULL);
	
	if (ret == REDIS_RET_SUCCESS) ret = 1;

_end:

	return ret;
}

int cache_decby_cfg(struct sip_msg *msg, char *key, char *by) {

	int ret = -1;

	if (!msg  || !key || !by) goto _end;
	
	ret = cache_decby(key, strlen(key), (long)by, NULL);
	
	if (ret == REDIS_RET_SUCCESS) ret = 1;

_end:

	return ret;
}

int cache_del_cfg(struct sip_msg *msg, char *key) {

	int ret = -1;

	if (!msg  || !key) goto _end;
	
	ret = cache_del(key, strlen(key));
	
	if (ret == REDIS_RET_SUCCESS) ret = 1;

_end:

	return ret;
}

int cache_sadd_cfg(struct sip_msg *msg, char *key, char *val) {

	int ret = -1;

	if (!msg  || !key || !val) goto _end;
	
	ret = cache_sadd(key, strlen(key), val, strlen(val));
	
	if (ret == REDIS_RET_SUCCESS) ret = 1;

_end:

	return ret;
}

int cache_srem_cfg(struct sip_msg *msg, char *key, char *val) {

	int ret = -1;

	if (!msg  || !key || !val) goto _end;
	
	ret = cache_srem(key, strlen(key), val, strlen(val));
	
	if (ret == REDIS_RET_SUCCESS) ret = 1;

_end:

	return ret;
}

int cache_scard_cfg(struct sip_msg* msg, char *key, char *result) {

	int ret = -1;
	pv_spec_t *sp_dest = (pv_spec_t *)result;
	pv_value_t pv_val;
	int card = 0;
	redis_return_t err = REDIS_RET_ERROR;

	if (!msg  || !key || !result) goto _end;
	
	err = cache_scard(key, strlen(key), &card);
	if (err != REDIS_RET_SUCCESS) goto _end;
	
	memset(&pv_val, 0, sizeof(pv_value_t));
	pv_val.flags = PV_VAL_INT;
	pv_val.ri = card;

	if (sp_dest->setf(msg, &sp_dest->pvp, (int)EQ_T, &pv_val)<0) {
		LM_ERR("setting profile PV failed\n");
		goto _end;
	}

	ret = 1;

_end:

	return ret;

}

int cache_sismember_cfg(struct sip_msg *msg, char *key, char *val) {

	int ret = -1;

	if (!msg  || !key || !val) goto _end;
	
	ret = cache_sismember(key, strlen(key), val, strlen(val));
	
	if (ret == REDIS_RET_SUCCESS) ret = 1;

_end:

	return ret;
}

int cache_smembers_cfg(struct sip_msg *msg, char *key, char *dest) {

	int ret = -1;
	pvname_list_t *pvlist = (pvname_list_t *)dest;
	int_str avp_val;
	int_str avp_name;
	unsigned short avp_type = 0;
	char **members = NULL;
	size_t n = 0;
	size_t *lens = NULL;
	redis_return_t err = REDIS_RET_ERROR;
	int i = 0;

	if (!msg || !key || !dest) goto _end;
	
	err = cache_smembers(key, strlen(key), &members, &n, &lens);

	if (err != REDIS_RET_SUCCESS) goto _end;

	if (pv_get_avp_name(msg, &pvlist->sname.pvp, &avp_name, &avp_type)!=0) {
		LM_ERR("cant get avp\n");
		goto _end;
	}

	for (i = 0; i < n; i ++) {

		avp_type |= AVP_VAL_STR;
		avp_val.s.s = members[i];
		avp_val.s.len = strlen(members[i]);

		if (add_avp(avp_type, avp_name, avp_val)!=0) {
			LM_ERR("unable to add avp\n");
			goto _end;
		}

	}

	ret = 1;

_end:
	
	cache_free(lens);
	for (i = 0; i < n; i ++) cache_free(members[i]);
	cache_free(members);

	return ret;

}
