/*
 * {app} - {purpose, description}
 *
 * Copyright (c) 2005-2009 Rack-Soft, LLC. All rights reserved.
 *
 * $Header: //itsd/devel/platform/voipnow/251/low_level/kamailio_extra_sources/1.5.4/modules/cache/cache_cmds.h#1 $
 *
 * Modified: $DateTime: 2010/09/28 18:16:02 $ by $Author: bogdanc $ with $Change: 71991 $
 */


#ifndef CACHE_CMDS_H__
#define CACHE_CMDS_H__

#include "../../sr_module.h"


/* Config file command handlers */

/*!\fn int cache_get_cfg(struct sip_msg *msg, char *key, char *result)
 *
 * Fetch the value of a key from the cache server.
 *
 * \param key Key
 * \param result A struct pv_value_t that'll hold the result
 *
 * \return 1 on success
 * \return -1 on error
 */
int cache_get_cfg(struct sip_msg* msg, char *key, char *result);

/*!\fn int cache_set_cfg(struct sip_msg *msg, char *key, char *value, char *ttl);
 *
 * Set the value of a key on the cache server.
 *
 * \param key Key
 * \param value Key value
 * \param ttl Lifespan of the key-value pair
 *
 * \return 1 on success
 * \return redis error code on error
 */
int cache_set_cfg(struct sip_msg *msg, char *key, char *value, char *ttl);

/*!\fn int cache_inc_cfg(struct sip_msg *msg, char *key);
 *
 * Increment the value of a key by 1 on the cache server.
 *
 * \param key Key
 *
 * \return 1 on success
 * \return redis error code on error
 */
int cache_inc_cfg(struct sip_msg *msg, char *key);

/*!\fn int cache_incby_cfg(struct sip_msg *msg, char *key, char *by);
 *
 * Increment the value of a key by the specified amount on the cache server.
 *
 * \param key Key
 *
 * \return 1 on success
 * \return redis error code on error
 */
int cache_incby_cfg(struct sip_msg *msg, char *key, char *by);

/*!\fn int cache_dec_cfg(struct sip_msg *msg, char *key);
 *
 * Decrement the value of a key by 1 on the cache server.
 *
 * \param key Key
 * \param by Amount
 *
 * \return 1 on success
 * \return redis error code on error
 */
int cache_dec_cfg(struct sip_msg *msg, char *key);

/*!\fn int cache_decby_cfg(struct sip_msg *msg, char *key, char *by);
 *
 * Decrement the value of a key by the specified amount on the cache server.
 *
 * \param key Key
 * \param by Amount
 *
 * \return 1 on success
 * \return redis error code on error
 */
int cache_decby_cfg(struct sip_msg *msg, char *key, char *by);

/*!\fn int cache_del_cfg(struct sip_msg *msg, char *key);
 *
 * Remove a key from the cache server.
 *
 * \param key Key
 *
 * \return 1 on success
 * \return redis error code on error
 */
int cache_del_cfg(struct sip_msg *msg, char *key);

/*!\fn int cache_sadd_cfg(struct sip_msg *msg, char *key, char *val);
 *
 * Add a value to a set
 *
 * \param key Key
 * \param val Value
 *
 * \return 1 on success
 * \return redis error code on error
 */
int cache_sadd_cfg(struct sip_msg *msg, char *key, char *val);

/*!\fn int cache_srem_cfg(struct sip_msg *msg, char *key, char *val);
 *
 * Remove a value from a set
 *
 * \param key Key
 * \param val Value
 *
 * \return 1 on success
 * \return redis error code on error
 */
int cache_srem_cfg(struct sip_msg *msg, char *key, char *val);

/*!\fn int cache_scard_cfg(struct sip_msg *msg, char *key, char *result);
 *
 * Return the number of elements in a set.
 *
 * \param key Key
 * \param val Value
 *
 * \return 1 on success
 * \return redis error code on error
 */
int cache_scard_cfg(struct sip_msg *msg, char *key, char *result);

/*!\fn int cache_sismember_cfg(struct sip_msg *msg, char *key, char *val);
 *
 * Check whether the value is a member of the given set or not.
 *
 * \param key Key
 * \param val Value
 *
 * \return 1 on success
 * \return redis error code on error
 */
int cache_sismember_cfg(struct sip_msg *msg, char *key, char *val);

/*!\fn int cache_smembers_cfg(struct sip_msg *msg, char *key, char *result);
 *
 * Get the list of members of a set.
 *
 * \param key Key
 * \param result AVP array placeholder for the result
 *
 * \return 1 on success
 * \return redis error code on error
 */
int cache_smembers_cfg(struct sip_msg *msg, char *key, char *result);


#endif /* CACHE_CMDS_H__ */
