/*
 * 4PSA VoipNow - cache module fixup functions
 *
 * Copyright (c) 2005-2009 Rack-Soft, LLC. All rights reserved.
 *
 * $Header: //itsd/devel/platform/voipnow/251/low_level/kamailio_extra_sources/1.5.4/modules/cache/cache_fixup.c#1 $
 *
 * Modified: $DateTime: 2010/09/28 18:16:02 $ by $Author: bogdanc $ with $Change: 71991 $
 */

#include "../../mod_fix.h"
#include "../../error.h"

/*! \brief fixup for char*, pvar */
int fixup_char_pvar(void **param, int param_no)
{
	if (param_no == 1) return 0;
	if (param_no != 2)
	{
	    LM_ERR("invalid parameter number %d\n", param_no);
	    return E_UNSPEC;
	}
	return fixup_pvar_null(param, 1);
}

/*! \brief fixup free for char*, pvar */
int fixup_free_char_pvar(void **param, int param_no)
{
	if(param_no == 1) return 0;
	if (param_no != 2) {
	    LM_ERR("invalid parameter number %d\n", param_no);
	    return E_UNSPEC;
	}
	return fixup_free_str_null(param, 1);
}

/*! \brief fixup for char*,char*,int */
int fixup_char_char_uint(void **param, int param_no)
{
	if(param_no == 1 || param_no == 2) return 0;
	if (param_no != 3) {
	    LM_ERR("invalid parameter number %d\n", param_no);
	    return E_UNSPEC;
	}
	return fixup_uint_null(param, 1);
}

/*! \brief fixup for char*,unsigned int(used as long) */
int fixup_char_uint(void **param, int param_no)
{
	if(param_no == 1) return 0;
	if (param_no != 2) {
	    LM_ERR("invalid parameter number %d\n", param_no);
	    return E_UNSPEC;
	}
	return fixup_uint_null(param, 1);
}

/*! \brief fixup for char*,pvname_list_t */
int fixup_char_pvlist(void **param, int param_no) {

	pvname_list_t *anlist = NULL;
	str s;

	if (param_no == 1) return 0;
	if (param_no != 2) {
	    LM_ERR("invalid parameter number %d\n", param_no);
	    return E_UNSPEC;
	}

	s.s = (char*)(*param);

	if (s.s==NULL) {
		LM_ERR("null format in P%d\n", param_no);
		return E_UNSPEC;
	}

	s.len = strlen(s.s);
	anlist = parse_pvname_list(&s, PVT_AVP);
	if (anlist==NULL) {
		LM_ERR("bad format in P%d [%s]\n", param_no, s.s);
		return E_UNSPEC;
	}
	*param = (void*)anlist;
	return 0;

}
