/*
 * {app} - {purpose, description}
 *
 * Copyright (c) 2005-2006 Rack-Soft, LLC. All rights reserved.
 *
 * $Header: //itsd/devel/platform/voipnow/251/low_level/kamailio_extra_sources/1.5.4/modules/db_mysql/dbase_4psa.c#2 $
 *
 * Modified: $DateTime: 2013/03/12 09:21:46 $ by $Author: bogdanc $ with $Change: 96896 $
 */




#include <stdio.h>
#include <string.h>
#include <mysql.h>
#include <errmsg.h>
#include <mysql_version.h>
#include "../../mem/mem.h"
#include "../../dprint.h"
#include "../../db/db_query.h"
#include "../../db/db_ut.h"
#include "val.h"
#include "my_con.h"
#include "res.h"
#include "row.h"
#include "db_mysql.h"
#include "dbase.h"


extern int db_mysql_submit_query(const db_con_t* _h, const str* _s);
extern int db_mysql_store_result(const db_con_t* _h, db_res_t** _r);
/**
 * \brief Interact with cache, and only if necessary interact with db.
 */

/* Per process redis connection */
struct redis_client *g_redis_client = NULL;
/* Reference count for the redis connection */
int g_redis_client_refcount = 0;

struct redis_client *db_red_mysql_init(void)
{
	/* Reuse existing connection if found or create a new one */
	if (!g_redis_client) {
		g_redis_client = db_red_init();
	}
	if (g_redis_client) {
		/* Increase the refcount of the created connection */
		g_redis_client_refcount++;
	}
	return g_redis_client;
}

void db_red_mysql_close(struct redis_client *client)
{
	if (g_redis_client) {
		if (client != g_redis_client) {
			return;
		}
		g_redis_client_refcount--;
		
		if (g_redis_client_refcount == 0) {
			/* Close connection to Redis when refcount reaches 0 */
			db_red_close(g_redis_client);
			g_redis_client = NULL;
		}
	}
}

unsigned long long db_affected_rows(const db_con_t* _h) {
	return mysql_affected_rows(CON_CONNECTION(_h));
}

/* the idea here is not to update any of the cached keys if a single one could not be set to sql
 * there will be inconsistencies either way, but this kind of inconsistency is the "regular" one */
static inline int affected_rows_matches_upd_memkeys(const db_con_t* _h, db_redq_pair_t * memkeys, int cols) {
	int ret = 0;
	int i, mem_rows = 1, last_col = -1;
	unsigned long long rows;

	if (!memkeys[0].memkey || !cols) goto _end;
	rows = mysql_affected_rows(CON_CONNECTION(_h));
	if (rows == -1ULL || rows == 0) goto _end;

	for (i = 0; memkeys[i].memkey; last_col = memkeys[i++].col) {
		if (memkeys[i].col >= cols || memkeys[i].col < 0) goto _end;
		/* new row (always fails at first iteration, last being -1) */
		if (last_col >= memkeys[i].col) ++mem_rows;
	}

	if (mem_rows == rows) ret = 1;

_end:
	return ret;
}

/* the idea here is (opposite from the above) to not to delete the cached keys
 * only if none of them were actually deleted
 * if at least one key could be deleted from sql it would be closer to correcteness
 * to delete all the keys from cache rather than leave all of them in the cache
 * the keys that incorrectly wanishes will be retrieved back on the first ocasion */
static inline int affected_rows_matches_del_memkeys(const db_con_t* _h, db_redq_pair_t * memkeys) {
	unsigned long long rows = mysql_affected_rows(CON_CONNECTION(_h));

	if (rows == -1ULL || rows == 0) return 0;

	/* if at least one key was deleted from sql, go ahead and delete them al from cache (return true) */
	return 1;
}


/** 4PSA: same as db_mysql_query but with redis support
 * \param memkey the string key by wich the result may have been cached */
/* int db_red_mysql_query(struct redis_client * mem, db_redq_pair_t * memkeys, int ttl,const db_con_t* _h, const db_key_t* _k, const db_op_t* _op, const db_val_t* _v, const db_key_t* _c, const int _n, const int _nc, const db_key_t _o, db_res_t** _r) {{{ */
int db_red_mysql_query(struct redis_client * mem, db_redq_pair_t * memkeys, int ttl\
		, const db_con_t* _h, const db_key_t* _k, const db_op_t* _op\
		, const db_val_t* _v, const db_key_t* _c, const int _n, const int _nc\
		, const db_key_t _o, db_res_t** _r) {
	
	int ret = -1;
	char ** key_strs = NULL;
	size_t * key_lengths = NULL;
	size_t alias = 0;
	int n;
	
	if (!_r || !memkeys[0].memkey || !memkeys[0].memkey[0]) goto _end;
	
	if (!memkeys[1].memkey) {
		/* if (!memkeys[0].memkey[0]) goto _end; */
		n = 1;
		key_strs = &memkeys[0].memkey;
		alias = strlen(key_strs[0]);
		key_lengths = &alias;
	} else if ((n = db_keyst2keyarr(memkeys, &key_strs, &key_lengths)) <= 0) {
		goto _end;
	}
	
	if ((*_r = db_red_get(mem, memkeys, key_strs, key_lengths, n))) {
		if (*_r == (db_res_t*)-1) {
			ret = -5;
			*_r = NULL;
		} else
			ret = 0;
		goto _end;
	} else {
		LM_DBG("sql branch");
		ret = db_do_query(_h, _k, _op, _v, _c, _n, _nc, _o, _r,
		db_mysql_val2str, db_mysql_submit_query, db_mysql_store_result);

		if (ret == 0 && *_r) _print_dbres(*_r);
		else LM_DBG("no result from sql");
		LM_DBG("sql branch end");
		if (ret == 0) db_red_set_byres(mem, memkeys, ttl, *_r);

	}

	
_end:
	if (!alias) {
		safe_free(key_strs);
		safe_free(key_lengths);
	}
	
	return ret;
} /* db_red_mysql_query }}} */



/* int db_red_mysql_raw_query(struct redis_client * mem, db_redq_pair_t * memkeys, int ttl, const db_con_t* _h, const str* _s, db_res_t** _r) {{{ */
int db_red_mysql_raw_query(struct redis_client * mem, db_redq_pair_t * memkeys, int ttl, const db_con_t* _h, const str* _s, db_res_t** _r) {
	int ret = -1;
	char ** key_strs = NULL;
	size_t * key_lengths = NULL;
	size_t alias = 0;
	int n;
	
	if (!memkeys[0].memkey || !memkeys[0].memkey[0]) goto _end;
	
	/* a select; behave like db_red_mysql_query() */
	if (_r) {
		if (!memkeys[1].memkey) {
			n = 1;
			key_strs = &memkeys[0].memkey;
			alias = strlen(key_strs[0]);
			key_lengths = &alias;
		} else if ((n = db_keyst2keyarr(memkeys, &key_strs, &key_lengths)) <= 0) {
			goto _end;
		}
		
		if ((*_r = db_red_get(mem, memkeys, key_strs, key_lengths, n))) {
			if (*_r == (db_res_t*)-1) {
				ret = -5;
				*_r = NULL;
			} else
				ret = 0;
			goto _end;
		} else {
			ret = db_do_raw_query(_h, _s, _r, db_mysql_submit_query,
				db_mysql_store_result);

			if (ret == 0) db_red_set_byres(mem, memkeys, ttl, *_r);
		}
	} else {
		/* an insert/update etc OR delete (no result queries) */
		/* 4PSA: we do a hash delete in this case since we don't know the new values
		 * were there any. next time somebody will try to fetch some of this keys
		 * the result will be queried from database and hash will be resync'd */

		ret = db_do_raw_query(_h, _s, _r, db_mysql_submit_query,
				db_mysql_store_result);
		if (ret == 0 && affected_rows_matches_del_memkeys(_h, memkeys)) {
			ret = db_red_del(mem, memkeys);
		}
	}
	
_end:
	if (!alias) {
		safe_free(key_strs);
		safe_free(key_lengths);
	}

	return ret;
} /* db_red_mysql_raw_query }}} */


/* int db_red_mysql_insert(struct redis_client * mem, db_redq_pair_t * memkeys, int ttl, const db_con_t* _h, const db_key_t* _k, const db_val_t* _v, const int _n) {{{ */
int db_red_mysql_insert(struct redis_client * mem, db_redq_pair_t * memkeys, int ttl\
, const db_con_t* _h, const db_key_t* _k, const db_val_t* _v, const int _n) {
	int ret = -1;
	
	ret = db_do_insert(_h, _k, _v, _n, db_mysql_val2str,
	db_mysql_submit_query);
	
	if (ret == 0) db_red_set_byval(mem, memkeys, ttl, _v, _n);

	return ret;
} /* db_red_mysql_insert }}} */


/* int db_red_mysql_delete(struct redis_client * mem, db_redq_pair_t * memkeys, const db_con_t* _h, const db_key_t* _k, const db_op_t* _o, const db_val_t* _v, const int _n) {{{ */
int db_red_mysql_delete(struct redis_client * mem, db_redq_pair_t * memkeys\
		, const db_con_t* _h, const db_key_t* _k, const db_op_t* _o\
		, const db_val_t* _v, const int _n) {
	
	int ret = -1;
	
	ret = db_do_delete(_h, _k, _o, _v, _n, db_mysql_val2str,
	db_mysql_submit_query);
	
	if (ret == 0 && affected_rows_matches_del_memkeys(_h, memkeys)) db_red_del(mem, memkeys);

	return ret;
} /* db_red_mysql_delete }}} */


/* int db_red_mysql_update(struct redis_client * mem, db_redq_pair_t * memkeys, int ttl, const db_con_t* _h, const db_key_t* _k, const db_op_t* _o, const db_val_t* _v, const db_key_t* _uk, const db_val_t* _uv, const int _n, const int _un) {{{ */
int db_red_mysql_update(struct redis_client * mem, db_redq_pair_t * memkeys\
		, int ttl, const db_con_t* _h, const db_key_t* _k, const db_op_t* _o\
		, const db_val_t* _v, const db_key_t* _uk, const db_val_t* _uv\
		, const int _n, const int _un) {
	
	int ret = -1;
	
	ret = db_do_update(_h, _k, _o, _v, _uk, _uv, _n, _un, db_mysql_val2str,
	db_mysql_submit_query);
	

	if (ret == 0 && affected_rows_matches_upd_memkeys(_h, memkeys, _un)) {
		db_red_set_byval(mem, memkeys, ttl, _uv, _un);
	}

	return ret;
} /* db_red_mysql_update }}} */


/* int db_red_mysql_replace(struct redis_client * mem, db_redq_pair_t * memkeys, int ttl, const db_con_t* _h, const db_key_t* _k, const db_val_t* _v, const int _n) {{{ */
int db_red_mysql_replace(struct redis_client * mem, db_redq_pair_t * memkeys, int ttl\
, const db_con_t* _h, const db_key_t* _k, const db_val_t* _v, const int _n) {
	
	int ret = -1;
	
	ret = db_do_replace(_h, _k, _v, _n, db_mysql_val2str,
	db_mysql_submit_query);
	
	if (ret == 0) db_red_set_byval(mem, memkeys, ttl, _v, _n);

	return ret;
} /* db_red_mysql_replace }}} */


/* int db_red_insert_update(struct redis_client * mem, db_redq_pair_t * memkeys, int ttl, const db_con_t* _h, const db_key_t* _k, const db_val_t* _v, const int _n) {{{ */
int db_red_insert_update(struct redis_client * mem, db_redq_pair_t * memkeys, int ttl\
, const db_con_t* _h, const db_key_t* _k, const db_val_t* _v, const int _n) {

	int ret = -1;

	ret = db_insert_update(_h, _k, _v, _n);
	
	if (ret == 0) db_red_set_byval(mem, memkeys, ttl, _v, _n);
	
	return ret;
} /* db_red_insert_update }}} */

