/*
 * {app} - {purpose, description}
 *
 * Copyright (c) 2005-2006 Rack-Soft, LLC. All rights reserved.
 *
 * $Header: //itsd/devel/platform/voipnow/251/low_level/kamailio_extra_sources/1.5.4/modules/dialog/dlg_puastate_4psa.c#1 $
 *
 * Modified: $DateTime: 2010/09/28 18:16:02 $ by $Author: bogdanc $ with $Change: 71991 $
 */


#define _GNU_SOURCE

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>


#include "../../locking.h"
#include "../../parser/parse_uri.h"
#include "dlg_timer.h"
#include "dlg_hash.h"
#include "dlg_profile.h"
#include "dlg_db_handler.h"
#include "../../misc_4psa.h"

static int red_statecounter(str extno, const char * state, int up);
extern gen_lock_t * g_mem_state_counter_lock;
extern struct redis_client * g_dlg_mem;
extern db_con_t* dialog_db_handle; /* database connection handle */
extern db_func_t dialog_dbf;


/* int dlg_statetracking_upd(str uri, unsigned flags, int newstate, int oldstate) {{{ */
int dlg_statetracking_upd(str uri, unsigned flags, int newstate, int oldstate) {
	int ret = -1;
	struct sip_uri uri_;
	int ringing = 0;
	int busy = 0;

	if (newstate == oldstate)
		return 0;

	if (!uri.len || !uri.s) 
		goto _end;

	if (parse_uri(uri.s, uri.len, &uri_) < 0) {
		LM_ERR("failed to parse from uri: %.*s\n", uri.len, uri.s);
		goto _end;
	}

	/* if extension is allowed to publish (phone publish) then we don't meddle */
	if (get_extension_publish(g_dlg_mem, uri_.user, &dialog_dbf, dialog_db_handle)) {
		ret = 0;
		goto _end;
	}

	/* determine states to decrement */
	switch (oldstate) {
		case DLG_STATE_EARLY: /* initial invite */
		/* was busy/ringing */
		if (flags & DLG_FLAG_SENDCALLER) {
			busy--;
		} else if (flags & DLG_FLAG_SENDCALLEE) {
			ringing--;
		}
		break;

		case DLG_STATE_CONFIRMED: /* ACK received */
		/* was busy */
		busy--;
		break;
	}

	/* determine states to increment */
	switch (newstate) {
		case DLG_STATE_EARLY:
		/* becomes busy/ringing */
		if (flags & DLG_FLAG_SENDCALLER) {
			busy++;
		} else if (flags & DLG_FLAG_SENDCALLEE) {
			ringing++;
		}
		break;

		case DLG_STATE_CONFIRMED:
		/* becomes busy */
		busy++;

		break;
	}

	ret = 0;
	if (ringing) {
		ret = red_statecounter(uri_.user, RINGING, ringing);
	}
	if (!ret && busy) {
		ret = red_statecounter(uri_.user, BUSY, busy);
	}

_end:
	return ret;
} /* dlg_statetracking_upd }}} */

/* static int red_statecounter(str extno, const char * state, int up) {{{ */
static int red_statecounter(str extno, const char * state, int up) {
	char *val = NULL;
	int count = 0;
   	int bc = -1;
	size_t val_len = 0;
	redis_return_t err = -1;
	int ret = -1;
#define MAX_COUNTER_LEN 64
	static char memkey[MAX_COUNTER_LEN];
	
#define MAX_EXTENSION_LINES 6
	
	if (!g_dlg_mem || !g_mem_state_counter_lock) return -1;

	/* extension.<full_ext_no>.<state> */
	bc = snprintf(memkey, MAX_COUNTER_LEN, "extension.%.*s.%s", extno.len, extno.s, state);
	if (bc < 0 || bc >= MAX_COUNTER_LEN) {
		LM_ERR("state counting memkey size %d excedeed (ret: %d)\n", MAX_COUNTER_LEN, bc);
		return ret;
	}

	lock_get(g_mem_state_counter_lock);
	
	err = redis_get_whole(g_dlg_mem, memkey, bc, &val, &val_len, "\0", 1);
	
	if ((err == REDIS_RET_SUCCESS) && val) {
		sscanf(val, "%d", &count);
	}
	if (val) free(val);
	val = NULL;

	/* increase/decrease counter */
	count = count + up;
	/* always force any value to be >= 0 */
	if (count < 0) count = 0;
	else if (count > MAX_EXTENSION_LINES) {
		LM_WARN("The maximum count of extension lines have been reached. Reseting\n");
		count = 0;
	}
	
	if (asprintf(&val, "%d", count) < 0)
		goto _end;

	if (val) {
		err = redis_set(g_dlg_mem, memkey, bc, val, strlen(val), MAKE_NON_SEMIDURABLE(PHONE_STATE_TTL));
		free(val);
	}

	ret = 0;
_end:
	lock_release(g_mem_state_counter_lock);
	return ret;

#undef MAX_COUNTER_LEN
} /* red_statecounter }}} */

#undef DOWN
#undef UP


#define NODEID_F	"/etc/voipnownode/nodeid"
#define NODEID_SZ	64

/* int dlg_fs_nodeid(void) {{{ */
int dlg_fs_nodeid(void) {
	static char buf[NODEID_SZ + 1], *p = buf;
	int fd = open(NODEID_F, O_RDONLY);
	long len = NODEID_SZ, bc = 0;

	if (fd < 0) return -1;

	buf[0] = '\0';
	while (len) {
		bc = read(fd, p, len);
		if (bc == -1 && errno == EINTR) continue;
		/* stop at eof */
		if (!bc) break;

		p[bc] = '\0';
		p += (unsigned long)bc;
		len -= bc;
	}
	--p;
	while (isspace(*p)) *p-- = '\0';

	g_nodeid_str.len = strlen(buf);
	g_nodeid_str.s = buf;

	return 1;
} /* dlg_fs_nodeid }}} */

