/*
 * 4PSA VoipNow - Htable variables cleanup based on dialog destruction
 *
 * Copyright (c) 2005-2009 Rack-Soft, LLC. All rights reserved.
 *
 * $Header: //itsd/devel/platform/voipnow/251/low_level/kamailio_extra_sources/1.5.4/modules/htable/dialog_4psa.c#1 $
 *
 * Modified: $DateTime: 2010/09/28 18:16:02 $ by $Author: bogdanc $ with $Change: 71991 $
 */

#include "ht_api.h"
#include "ht_var.h"

#include "../../mem/mem.h"

#include "../dialog/dlg_load.h"
#include "../dialog/dlg_hash.h"

#include "../tm/tm_load.h"

struct dlg_binds dlg_api;
struct tm_binds tm_api;
extern ht_t *_ht_root;

int ht_dlg_key_expire_on_bye = -1;

static void dialog_created_4psa(struct dlg_cell *dlg, int type, struct dlg_cb_params *_params);
static void dialog_done_4psa(struct dlg_cell *dlg, int type, struct dlg_cb_params *_params);


int dialog_garbage_init(void)
{
	/* bind to the dialog API */
	if (load_dlg_api(&dlg_api)!=0) {
		LM_ERR("failed to find dialog API - is dialog module loaded?\n");
		return -1;
	}
	/* load the TM API */
	if (load_tm_api(&tm_api)!=0) {
		LM_ERR("can't load TM API\n");
		return -1;
	}
	/* register dialog creation callback */
	if (dlg_api.register_dlgcb(NULL, DLGCB_CREATED, dialog_created_4psa, NULL, NULL) != 0) {
		LM_ERR("cannot register callback for dialog creation\n");
		return -1;
	}

	return 0;
}

static void dialog_created_4psa(struct dlg_cell *dlg, int type, struct dlg_cb_params *params)
{
	/* Register dialog callbacks for when the dialog will be terminated and destroyed */
	if (dlg_api.register_dlgcb(dlg, DLGCB_TERMINATED | DLGCB_DESTROY,
		dialog_done_4psa, NULL, NULL) != 0) {
		LM_ERR("cannot register callback for dialog destructor\n");
	}
}

static void dialog_hashtable_cleanup_4psa(struct cell* t, int type, struct tmcb_params *params)
{
	ht_t *ht;
	str callid;
	
	if ((params->code < 200) || (params->code >= 300)) {
		/* Only interested in 2xx codes */
		LM_ERR("Got reply to bye with code %d. Not cleaning hastables.\n", params->code);
		return;
	}
	
	callid.s = (char *)(*params->param);
	callid.len = strlen(callid.s);
	
	LM_DBG("Transaction %p for dialog with callid %.*s received event %d.\n", t, callid.len, callid.s, type);
	
	ht = _ht_root;
	while(ht) {
		if (ht_del_cell(ht, &callid) != 0) {
			LM_ERR("Error on deleting callid \"%.*s\" from hashtable %.*s\n", callid.len, callid.s, ht->name.len, ht->name.s);
		}
		ht = ht->next;
	}
	
	return;
}

static char *shm_strdup(str *src) {
	char *res;

	if (!src || !src->s)
		return NULL;
	if (!(res = (char *) shm_malloc(src->len + 1)))
		return NULL;
	strncpy(res, src->s, src->len);
	res[src->len] = 0;
	return res;
}

static void release_info_4psa(void *info)
{
	char *callid = (char*)info;
	shm_free(callid);
}

static void dialog_done_4psa(struct dlg_cell *dlg, int type, struct dlg_cb_params *params)
{
	int_str expires;
	ht_t *ht;
	char *callid_dup;
	
	if (!dlg || !dlg->callid.len) {
		return;
	}
	
	LM_DBG("Dialog %p with callid %.*s received event %d.\n", dlg, dlg->callid.len, dlg->callid.s, type);
	
	if (dlg_api.lookup_similar_dlg(dlg)) {
		/* Do not remove hash table entries as there is another dialog that needs them */
		LM_DBG("Not removing hash table entries for dialog %p as there is another active dialog with the same callid.\n", dlg);
		return;
	}

	if (ht_dlg_key_expire_on_bye > 0) {
		LM_DBG("Setting expire to %d for callid key %.*s.\n", ht_dlg_key_expire_on_bye, dlg->callid.len, dlg->callid.s);
		expires.n = ht_dlg_key_expire_on_bye;
		ht = _ht_root;
		while(ht) {
			if (ht_set_cell_expire(ht, &dlg->callid, 0, &expires) != 0) {
				LM_ERR("Error on expiring callid \"%.*s\" in hashtable %.*s\n", dlg->callid.len, dlg->callid.s, ht->name.len, ht->name.s);
			}
			ht = ht->next;
		}
	}
	if (params->msg) {
		LM_DBG("Registering for transaction response notification\n");
		callid_dup = shm_strdup(&dlg->callid);
		/* The sip dialog has not actualy ended, only a BYE was received.
		 * A 200 OK might come so register for the replies to the current transaction.
		 */
		if ( tm_api.register_tmcb( params->msg, 0, TMCB_RESPONSE_PRE_OUT|TMCB_RESPONSE_FWDED, dialog_hashtable_cleanup_4psa,
						(void*)callid_dup, release_info_4psa) < 0 ) {
			LM_ERR("Failed to register TMCB\n");
		}
	}
}

