/*
 * $Id: //itsd/devel/platform/voipnow/251/low_level/kamailio_extra_sources/1.5.4/modules/load_balancer/lb_data.c#1 $
 *
 * load balancer module - complex call load balancing
 *
 * Copyright (C) 2009 Voice Sistem SRL
 *
 * This file is part of opensips, a free SIP server.
 *
 * opensips is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * opensips is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * History:
 * --------
 *  2009-02-01 initial version (bogdan)
 */



#include <stdio.h>

#include "../../mem/shm_mem.h"
#include "lb_parser.h"
#include "lb_data.h"
#include "lb_db.h"

extern db_con_t* lb_db_handle;
extern db_func_t lb_dbf;

str kam_strdup(const char * s, size_t len) {
	str ret = {0,0};

	if (!s) goto _end;

	ret.len = len ? len : strlen(s);

	ret.s = pkg_malloc(ret.len + 1);
	if (!ret.s) {
		ret.len = 0;
		goto _end;
	}

	ret.s[ret.len] = '\0';

	strncpy(ret.s, s, ret.len);

_end:

	return ret;
}

static str choose_destination(struct lb_res_str_list *rl, db_res_t* res) {
	str node = {0,0};
	db_row_t row;
	db_val_t nod, prio, free1, free2;

	if (RES_ROW_N(res) < 1) goto _end;

	row = RES_ROWS(res)[0];
	if (!row.values || row.n < 2) {
		LM_ERR("Malformed query; less than two columns selected\n");
		goto _end;
	}

	nod = row.values[0];
	free1 = row.values[1];
	if (nod.nul || free1.nul) goto _end;

	switch (rl->n) {
		case 1:
		/* can the value be less than 0 ? */
		if (free1.val.int_val == 0) goto _end;
		else {
			node = kam_strdup(nod.val.string_val, 0);
			goto _end;
		}
		break;

		case 2:
		if (row.n < 3) goto _end;

		prio = row.values[2];
		if (prio.nul) goto _end;

		if (prio.val.int_val == 0 && free1.val.int_val == 0) goto _end;
		else {
			node = kam_strdup(nod.val.string_val, 0);
			goto _end;
		}
		break;

		case 3:
		if (row.n < 4) goto _end;
		free2 = row.values[2];
		prio = row.values[3];
		if (prio.nul || free2.nul) goto _end;

		if (prio.val.int_val == 0 && free1.val.int_val == 0\
				&& free2.val.int_val == 0) goto _end;
		else {
			node = kam_strdup(nod.val.string_val, 0);
			goto _end;
		}
		break;
	}

_end:
	
	return node;
}

int do_load_balance(struct sip_msg *req, struct lb_res_str_list *rl) {
	/* what does -1 mean? is something bad; should we always ret -2 */
	int ret = -1;
	db_res_t* res = NULL;
	str dst = {0,0};

	if (rl->n < 1) {
		LM_ERR("empty resource list recived\n");
		goto _end;
	}

	/* make query from rl arr */
	res = lb_db_server_resources(rl);
	if (!res) {
		LM_ERR("problem quering database\n");
		goto _end;
	}

	/* walk result and decide route (dst) */
	dst = choose_destination(rl, res);

	ret = -2;
	if (!dst.s || !dst.len) {
		LM_WARN("no node available\n");
		goto _end;
	}

	/* set dst uri */
	if (dst.s && set_dst_uri(req, &dst) != 0) {
		LM_WARN("failed to set duri\n");
		goto _end;
	}

	//return 0, -1, -2 (??)
	ret = 0;
_end:

	if (res) lb_dbf.free_result(lb_db_handle, res);
	if (dst.s) pkg_free(dst.s);

	return ret;
}

