/*
 * $Id: //itsd/devel/platform/voipnow/251/low_level/kamailio_extra_sources/1.5.4/modules/load_balancer/lb_db.c#1 $
 *
 * load balancer module - complex call load balancing
 *
 * Copyright (C) 2009 Voice Sistem SRL
 *
 * This file is part of opensips, a free SIP server.
 *
 * opensips is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * opensips is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * History:
 * --------
 *  2009-02-01 initial version (bogdan)
 */




#include "../../db/db.h"
#include "lb_db.h"

#define LB_TABLE_VERSION  1
#define LB_FETCH_ROWS     100

str lb_id_column			=	str_init(LB_ID_COL);
str lb_grpid_column			=	str_init(LB_GRP_ID_COL);
str lb_dsturi_column		=	str_init(LB_DST_URI_COL);
str lb_resource_column		=	str_init(LB_RESOURCES_COL);
str lb_table_name			=	str_init(LB_TABLE_NAME);

static struct {
	const char * arg;
	const char * cap;
	const char * use;
	const char * as;
} sql_fields[LB_SQL_LAST] = {
	{"sip", "sip_capacity", "sip_usage", "free_sip"},
	{"iax", "iax_capacity", "iax_usage", "iax_free"},
	{"pstn","pstn_capacity", "pstn_usage", "pstn_free"}
};

db_con_t* lb_db_handle    = 0; /* database connection handle */
db_func_t lb_dbf;


enum sql_fields_t sql_fields_idx(str arg_res) {
	int i;
	for (i = 0; i < LB_SQL_LAST; ++i) {
		if (strncmp(arg_res.s, sql_fields[i].arg, arg_res.len) == 0) {
			return i;
		}
	}
	return LB_SQL_ERR;
}


int lb_connect_db(const str *db_url)
{
	if (lb_db_handle) {
		LM_CRIT("BUG - db connection found already open\n");
		return -1;
	}
	if ((lb_db_handle = lb_dbf.init(db_url)) == 0)
		return -1;

	return 0;
}

void lb_close_db(void)
{
	if (lb_db_handle==NULL)
		return;

	lb_dbf.close(lb_db_handle);
	lb_db_handle = NULL;
}


int init_lb_db(const str *db_url, char *table)
{
	/* Find a database module */
	if (db_bind_mod(db_url, &lb_dbf) < 0){
		LM_ERR("Unable to bind to a database driver\n");
		return -1;
	}

	if (lb_connect_db(db_url)!=0){
		LM_ERR("unable to connect to the database\n");
		return -1;
	}

	if (table) {
		lb_table_name.s = table;
		lb_table_name.len = strlen(table);
	}

	if(db_check_table_version(&lb_dbf, lb_db_handle,
	&lb_table_name, LB_TABLE_VERSION) < 0) {
		LM_ERR("error during table version check.\n");
		return -1;
	}

	return 0;
}

db_res_t * lb_db_server_resources(struct lb_res_str_list *rl) {
	db_res_t* ret = NULL;
	db_res_t* res = NULL;
	int i, pstn_idx = -1;
	/* int have_pstn = 0; */
	str query = {0, 0};

	/* move pstn to the end if present */
	for (i = 0; i < rl->n; ++i) {
		rl->resources[i].sql_idx = sql_fields_idx(rl->resources[i].name);
		if (rl->resources[i].sql_idx == LB_SQL_ERR) {
			LM_ERR("not a valid resource name '%.*s'\n", rl->resources[i].name.len, rl->resources[i].name.s);
			goto _end;
		}

		if (rl->resources[i].sql_idx == LB_SQL_PSTN) pstn_idx = i;
	}
	if (pstn_idx != -1 && pstn_idx != rl->n - 1) {
		struct lb_res_str tmpr = rl->resources[rl->n - 1];

		rl->resources[rl->n - 1] = rl->resources[pstn_idx];
		rl->resources[pstn_idx] = tmpr;
	}

	/* this should be more than enough	 */
	query.len = 200 + 200 * rl->n;
	query.s = pkg_malloc(query.len + 1);
	if (!query.s) {
		LM_ERR("no more private mem left\n");
	}
	
	switch (rl->n) {
		case 1:
		query.len = snprintf(query.s, query.len\
			, "SELECT dst_uri, (%s - %s) as %s FROM ser_balancing "
			"  WHERE status=1 ORDER BY %s DESC"\
			, sql_fields[rl->resources[0].sql_idx].cap\
			, sql_fields[rl->resources[0].sql_idx].use\
			, sql_fields[rl->resources[0].sql_idx].as\
			, sql_fields[rl->resources[0].sql_idx].as);
		break;

		case 2:
		if (pstn_idx != -1) {
			query.len = snprintf(query.s, query.len\
				, "SELECT dst_uri, (%s - %s) as %s, (%s - %s) as %s"
				", ((%s-%s) + (%s-%s))*(%s-%s) as prio"
				"  FROM ser_balancing WHERE status=1 ORDER BY prio DESC, %s DESC"\
				, sql_fields[rl->resources[0].sql_idx].cap\
				, sql_fields[rl->resources[0].sql_idx].use\
				, sql_fields[rl->resources[0].sql_idx].as\
				, sql_fields[rl->resources[1].sql_idx].cap\
				, sql_fields[rl->resources[1].sql_idx].use\
				, sql_fields[rl->resources[1].sql_idx].as\
				, sql_fields[rl->resources[0].sql_idx].cap\
				, sql_fields[rl->resources[0].sql_idx].use\
				, sql_fields[rl->resources[1].sql_idx].cap\
				, sql_fields[rl->resources[1].sql_idx].use\
				, sql_fields[rl->resources[1].sql_idx].cap\
				, sql_fields[rl->resources[1].sql_idx].use\
				, sql_fields[rl->resources[0].sql_idx].as);
		} else {
			query.len = snprintf(query.s, query.len\
				, "SELECT dst_uri, (%s - %s) as %s, (%s - %s) as %s"
				", ((%s-%s) + (%s-%s)) as prio"
				"  FROM ser_balancing WHERE status=1 ORDER BY prio DESC, %s DESC"\
				, sql_fields[rl->resources[0].sql_idx].cap\
				, sql_fields[rl->resources[0].sql_idx].use\
				, sql_fields[rl->resources[0].sql_idx].as\
				, sql_fields[rl->resources[1].sql_idx].cap\
				, sql_fields[rl->resources[1].sql_idx].use\
				, sql_fields[rl->resources[1].sql_idx].as\
				, sql_fields[rl->resources[0].sql_idx].cap\
				, sql_fields[rl->resources[0].sql_idx].use\
				, sql_fields[rl->resources[1].sql_idx].cap\
				, sql_fields[rl->resources[1].sql_idx].use\
				, sql_fields[rl->resources[0].sql_idx].as);
		}
		break;

		case 3:
		if (pstn_idx != -1) {
			query.len = snprintf(query.s, query.len\
				, "SELECT dst_uri, (%s - %s) as %s, (%s - %s) as %s, (%s - %s) as %s"
				", ((%s-%s) + (%s-%s) + (%s-%s))*(%s-%s) as prio"
				"  FROM ser_balancing WHERE status=1 ORDER BY prio DESC, %s DESC, %s DESC"\
				, sql_fields[rl->resources[0].sql_idx].cap\
				, sql_fields[rl->resources[0].sql_idx].use\
				, sql_fields[rl->resources[0].sql_idx].as\
				, sql_fields[rl->resources[1].sql_idx].cap\
				, sql_fields[rl->resources[1].sql_idx].use\
				, sql_fields[rl->resources[1].sql_idx].as\
				, sql_fields[rl->resources[2].sql_idx].cap\
				, sql_fields[rl->resources[2].sql_idx].use\
				, sql_fields[rl->resources[2].sql_idx].as\
				, sql_fields[rl->resources[0].sql_idx].cap\
				, sql_fields[rl->resources[0].sql_idx].use\
				, sql_fields[rl->resources[1].sql_idx].cap\
				, sql_fields[rl->resources[1].sql_idx].use\
				, sql_fields[rl->resources[2].sql_idx].cap\
				, sql_fields[rl->resources[2].sql_idx].cap\
				, sql_fields[rl->resources[2].sql_idx].use\
				, sql_fields[rl->resources[2].sql_idx].use\
				, sql_fields[rl->resources[0].sql_idx].as\
				, sql_fields[rl->resources[1].sql_idx].as);
		} else {
			query.len = snprintf(query.s, query.len\
				, "SELECT dst_uri, (%s - %s) as %s, (%s - %s) as %s, (%s - %s) as %s"
				", ((%s-%s) + (%s-%s) + (%s-%s)) as prio"
				"  FROM ser_balancing WHERE status=1 ORDER BY prio DESC, %s DESC, %s DESC"\
				, sql_fields[rl->resources[0].sql_idx].cap\
				, sql_fields[rl->resources[0].sql_idx].use\
				, sql_fields[rl->resources[0].sql_idx].as\
				, sql_fields[rl->resources[1].sql_idx].cap\
				, sql_fields[rl->resources[1].sql_idx].use\
				, sql_fields[rl->resources[1].sql_idx].as\
				, sql_fields[rl->resources[2].sql_idx].cap\
				, sql_fields[rl->resources[2].sql_idx].use\
				, sql_fields[rl->resources[2].sql_idx].as\
				, sql_fields[rl->resources[0].sql_idx].cap\
				, sql_fields[rl->resources[0].sql_idx].use\
				, sql_fields[rl->resources[1].sql_idx].cap\
				, sql_fields[rl->resources[1].sql_idx].use\
				, sql_fields[rl->resources[2].sql_idx].cap\
				, sql_fields[rl->resources[2].sql_idx].use\
				, sql_fields[rl->resources[0].sql_idx].as\
				, sql_fields[rl->resources[1].sql_idx].as);
		}
		break;
	}

	if (query.len <= 0) goto _end;

	if (lb_dbf.raw_query(lb_db_handle, &query, &res) != 0 || !res) {
		LM_ERR("query unsuccessful: %.*s\n", query.len, query.s);
		goto _end;
	}


	ret = res;
_end:
	
	return ret;
}

/* db_res_t * get_sql_nodestate(void) {{{ */
db_res_t * get_sql_nodestate(void) {
	db_res_t* ret = NULL;
	db_res_t* res = NULL;
	str query = str_init("SELECT nodeid, dst_uri, status FROM ser_balancing "
			" WHERE status IN (1, 4)");


	if (lb_dbf.raw_query(lb_db_handle, &query, &res) != 0 || !res) {
		LM_ERR("query unsuccessful: %.*s\n", query.len, query.s);
		goto _end;
	}


	ret = res;
_end:

	return ret;
} /* get_sql_nodestate }}} */

/* int set_sql_newstate(str nodeid, int newstate) {{{ */
int set_sql_newstate(str nodeid, int newstate) {
	int ret = -1;
	str fmt = str_init("UPDATE `ser_balancing` SET `status` = %d "
			" WHERE `nodeid` = '%.*s'");
	str query;

	/* state is tinyint(3) */
	query.len = fmt.len + nodeid.len + 3;
	query.s = pkg_malloc(query.len + 1);
	if (!query.s) {
		LM_ERR("Not enough private mem\n");
		goto _end;
	}
	/* str_init is based on compiler generated constant strings - thus nul term */
	query.len = snprintf(query.s, query.len+1, fmt.s, newstate, nodeid.len, nodeid.s);
	if (query.len <= 0) {
		LM_ERR("snprintf error\n");
		goto _end;
	}


	if ((ret = lb_dbf.raw_query(lb_db_handle, &query, NULL)) < 0) {
		LM_ERR("query unsuccessful: %.*s\n", query.len, query.s);
		goto _end;
	} else ret = 0;

	
_end:
	if (query.s) pkg_free(query.s);
	
	return ret;
} /* set_sql_newstate }}} */

