/*
 * {app} - {purpose, description}
 *
 * Copyright (c) 2005-2006 Rack-Soft, LLC. All rights reserved.
 *
 * $Header: //itsd/devel/platform/voipnow/251/low_level/kamailio_extra_sources/1.5.4/modules/load_balancer/lb_prober.c#1 $
 *
 * Modified: $DateTime: 2010/09/28 18:16:02 $ by $Author: bogdanc $ with $Change: 71991 $
 */



#include "../../sr_module.h"
#include "../../db/db.h"
#include "../tm/tm_load.h"

#include "lb_prober.h"
#include "lb_db.h"


extern struct tm_binds tmb;
extern int lb_probing_threshold;

struct lb_node_list_t * g_nodes = NULL;
gen_lock_t * g_node_list_lock = NULL;


/* fwd decl */
int try_set_state(struct lb_node_list_t * node, int up);


void lb_node_list_destroy(struct lb_node_list_t * ptr) {
	if (ptr) {
		if (ptr->nodeid.s) shm_free(ptr->nodeid.s);
		if (ptr->dst_uri.s) shm_free(ptr->dst_uri.s);

		shm_free(ptr);
	}
}

struct lb_node_list_t * lb_node_list_init(const char * nodeid, int nodeid_len, const char * dst, int dst_len, int state) {
	struct lb_node_list_t * ret = NULL;
	struct lb_node_list_t * elem = shm_malloc(sizeof(struct lb_node_list_t));
	memset(elem, 0, sizeof(struct lb_node_list_t));

	if (nodeid) {
		elem->nodeid.len = nodeid_len ? nodeid_len : strlen(nodeid);
		elem->nodeid.s = shm_malloc(elem->nodeid.len);
		if (!elem->nodeid.s) {
			LM_ERR("Not enough shared mem.\n");
			goto _end;
		}
		strncpy(elem->nodeid.s, nodeid, elem->nodeid.len);
	}

	if (dst) {
		elem->dst_uri.len = dst_len ? dst_len : strlen(dst);
		elem->dst_uri.s = shm_malloc(elem->dst_uri.len);
		if (!elem->dst_uri.s) {
			LM_ERR("Not enough shared mem.\n");
			goto _end;
		}
		strncpy(elem->dst_uri.s, dst, elem->dst_uri.len);
	}

	elem->state = state;

	ret = elem;
_end:
	if (!ret && elem) lb_node_list_destroy(elem);

	return ret;
}

void lb_node_list_del(struct lb_node_list_t * elem, struct lb_node_list_t * prev) {

	if (!prev) return;
	prev->next = elem->next;
	elem->next = NULL;
}

inline void lb_node_list_add(struct lb_node_list_t * who) {
	who->next = g_nodes->next;
	g_nodes->next = who;
}


inline void lb_node_list_ins(struct lb_node_list_t * where, struct lb_node_list_t * who) {
	who->next = where->next;
	where->next = who;
}


static void lb_options_callback( struct cell *t, int type,
		        struct tmcb_params *ps ) {

	struct lb_node_list_t * node = NULL, *n;
	str uri = {0,0};
	int up;

	if (!*ps->param) {
		LM_DBG("No parameter provided, OPTIONS-Request was finished"
				" with code %d\n", ps->code);
		return;
	}

	node = (struct lb_node_list_t *)(*ps->param);

	/* The SIP-URI is taken from the Transaction.
	 * Remove the "To: " (s+4) and the trailing new-line (len - 4 (To: )
	 * - 2 (\r\n)). */
	uri.s = t->to.s + 4;
	uri.len = t->to.len - 6;
	
	NODE_LIST_LOCK;
	for (n = g_nodes->next; n; n = n->next) if (n == node) break;

	if (node != n) {
		LM_DBG("Node %p no longer found in list\n", node);
		NODE_LIST_UNLOCK;
		return;
	}

	LM_DBG("OPTIONS-Request was finished with code %d (to %.*s, nodeid %.*s)\n",
			ps->code, uri.len, uri.s, node->nodeid.len, node->nodeid.s);

	/* 200 is the normal answer
	 * asterisk, however, replys with 404 */
	if (ps->code == 200 || ps->code == 404) up = 1;
	else up = 0;

	try_set_state(node, up);

	NODE_LIST_UNLOCK;

	return;
};


int try_set_state(struct lb_node_list_t * node, int up) {
	int ret = -1;

//	LM_DBG("prober: node:%.*s: db_state:%d, history:st:%d, ping:%d, count:%d\n"\
	, node->dst_uri.len, node->dst_uri.s, node->state, node->growing_state, up, node->state_counter);
	/* nothing to change */
	if (node->state == node->growing_state && (node->state==4?0:1) == up) {
		/* LM_DBG("prober: the same\n"); */
		ret = 0;
		goto _end;
	}

	switch (node->growing_state) {
		case LB_DB_STATE_ACTIVE:
		if (up) ++node->state_counter;
		else {
			node->growing_state = LB_DB_STATE_PROBING;
			node->state_counter = 1;
		}
		break;


		case LB_DB_STATE_PROBING:
		if (up) {
			node->growing_state = LB_DB_STATE_ACTIVE;
			node->state_counter = 1;
		}
		else ++node->state_counter;
		break;


		default:
		if (up) node->growing_state = LB_DB_STATE_ACTIVE;
		else node->growing_state = LB_DB_STATE_PROBING;
		node->state_counter = 1;
	}

//	LM_DBG("prober: node inter:%.*s: db_state:%d, history:st:%d, ping:%d, count:%d\n"\
		, node->dst_uri.len, node->dst_uri.s, node->state, node->growing_state, up, node->state_counter);
	if (node->state_counter >= lb_probing_threshold) {
		if (set_sql_newstate(node->nodeid, node->growing_state) < 0) {
			LM_WARN("Failed to set new state: %d for node: %.*s\n"\
					, node->growing_state, node->nodeid.len, node->nodeid.s);
			goto _end;
		}
		LM_NOTICE("Node %.*s changed state form: %d to %d (counter was: %d)\n"\
				, node->nodeid.len, node->nodeid.s\
				, node->state, node->growing_state, node->state_counter);
		node->state = node->growing_state;
		node->state_counter = 1;
		ret = 1;
	} else ret = 0;


_end:
//	LM_DBG("prober: node exit:%.*s: db_state:%d, history:st:%d, ping:%d, count:%d\n"\
		, node->dst_uri.len, node->dst_uri.s, node->state, node->growing_state, up, node->state_counter);

	return ret;
}

/*! \brief Timer for checking inactive destinations
 * 
 * This timer is regularly fired.
 */
void lb_check_timer(unsigned int ticks, void* param) {
	db_row_t row;
	db_res_t* res = NULL;
	int i, found;
	struct lb_node_list_t * node, *prev;

	/* reset recent flag */
	NODE_LIST_LOCK;
	for (node = g_nodes->next; node; node = node->next) node->recent = 0;
	NODE_LIST_UNLOCK;
	
	if (!(res = get_sql_nodestate()) || RES_ROW_N(res) < 1) {
		LM_WARN("Error querying sql for for nodes\n");
		goto _end;
	}

	/* for each db node - consult list */
	for (i = 0, found = 0; i < RES_ROW_N(res); ++i, found = 0) {
		struct lb_node_list_t * newnode = NULL;
		row = RES_ROWS(res)[i];

		NODE_LIST_LOCK;
		for (node = g_nodes->next, prev = g_nodes; node; prev = node, node = node->next) {
			if (strncmp(node->nodeid.s, row.values[0].val.string_val, node->nodeid.len) == 0) {
				if (node->state != row.values[2].val.int_val) {
					node->state = row.values[2].val.int_val;
					/* watch out: this value is already used by state_disabled... */
					node->growing_state = 0;
					node->state_counter = 0;
				}
				node->recent = 1;
				NODE_LIST_UNLOCK;
				found = 1;
				break;
			}
		}
		if (!found) {
			newnode = lb_node_list_init(row.values[0].val.string_val, 0\
					, row.values[1].val.string_val, 0, row.values[2].val.int_val);
			/* TODO: what to do if err?? */
			if (!newnode) {
				NODE_LIST_UNLOCK;
				continue;
			}
			newnode->recent = 1;

			lb_node_list_add(newnode);
		}
		NODE_LIST_UNLOCK;
	}
	
	if (!g_nodes) {
		LM_WARN("No active/probing nodes found\n");
		goto _end;
	}

	/* foreach list node - register callback OR remove if not recent */
	NODE_LIST_LOCK;
	for (node = g_nodes->next, prev = g_nodes; node; prev = node, node = node->next) {
		
		/* if this elem was not found enabled in database then we remove it from our list */
		if (!node->recent) {
			lb_node_list_del(node, prev);
			lb_node_list_destroy(node);
			node = prev;

			continue;
		}
		LM_DBG("probing node %.*s, URI %.*s\n", node->nodeid.len, node->nodeid.s\
			, node->dst_uri.len, node->dst_uri.s);

		/* Send ping using TM-Module.
		 * int request(str* m, str* ruri, str* to, str* from, str* h,
		 *		str* b, str *oburi,
		 *		transaction_cb cb, void* cbp); */
		if (tmb.t_request(&lb_ping_method,
					&node->dst_uri,
					&node->dst_uri,
					&lb_ping_from,
					NULL,
					NULL,
					NULL,
					lb_options_callback,
					node) < 0) {

			LM_ERR("unable to ping [%.*s]\n",
					node->dst_uri.len, node->dst_uri.s);
		}
	}
	NODE_LIST_UNLOCK;

_end:

	return;
}


