/*
 * {app} - {purpose, description}
 *
 * Copyright (c) 2005-2006 Rack-Soft, LLC. All rights reserved.
 *
 * $Header: //itsd/devel/platform/voipnow/251/low_level/kamailio_extra_sources/1.5.4/modules/presence/presentity_4psa.c#1 $
 *
 * Modified: $DateTime: 2010/09/28 18:16:02 $ by $Author: bogdanc $ with $Change: 71991 $
 */


#define _GNU_SOURCE

#include <stdlib.h>
#include <string.h>
//#include "../../db/db.h"

#include "presence.h"
#include "presentity.h"
#include "notify.h"
#include "../../parser/parse_content.h"
#include "../pua/pidf.h"


/* char * body2state(const char * body) {{{ */
char * body2state(int event, const char * body) {
	char * ret = NULL;
	char * start = NULL, * end = NULL;
	char * tmp = NULL;
	unsigned long sz = 0;

	if (!body) goto _end;

	switch (event) {
		case EVENT_DIALOG:

		start = strstr(body, "<state>");
		if (!start) goto _end;
		start += 7;

		end = strstr(start, "</state>");
		if (!end) goto _end;
		sz = end - start;

		break;


		case EVENT_PRESENCE:
		/* there's no standard xml here;
		 * sorry, can't get state, no redis */
		goto _end;

		start = strstr(body, "<status>");
		if (!start) goto _end;
		start += 8;

		start = strstr(body, "<basic>");
		if (!start) goto _end;
		start += 7;

		end = strstr(start, "</basic>");
		if (!end) goto _end;

		tmp = strstr(start, "</status>");
		/* not our <basic> end */
		if (end > tmp) goto _end;

		sz = end - start;

		break;
	}

	ret = pkg_malloc(sz + 1);
	if (!ret) goto _end;

	memcpy(ret, start, sz);
	ret[sz] = '\0';
_end:


	return ret;
} /* body2state }}} */


/* int get_memsql_state(struct redis_client * mem, str pres_uri, str * state) {{{ */
int get_memsql_state(struct redis_client * mem, str pres_uri, str * state) {
	int ret = -1;
#define a_MAX 	40
	char key[a_MAX];
	int bc = -1;
	struct sip_uri uri;
	redis_return_t err = -1;
	/* this is used because str.len is an int, which sucks on x64 */
	size_t cool_sz;

	if (parse_uri(pres_uri.s, pres_uri.len, &uri)< 0) {
		LM_ERR("while parsing uri %.*s\n", pres_uri.len, pres_uri.s);
		goto _end;
	}

	/* form key: extension.<extno>.state */
	bc = snprintf(key, a_MAX, EXT_PART"%.*s"STATE_PART, uri.user.len, uri.user.s);
	if (bc <= 0 || bc >= a_MAX) {
		LM_ERR("Failed to form redis key for presence state using extension: %.*s\n"
				, uri.user.len, uri.user.s);
		goto _end;
	}

	/* get presence state */
	err = redis_get(mem, key, bc, &state->s, &cool_sz);
	state->len = (int)cool_sz;

	if (err && err != REDIS_RET_NOTFOUND) {
		LM_WARN("failed to query redis for presence state "
				"for extension %.*s, with key: %s (%d:%s)\n"
				, uri.user.len, uri.user.s, key, err, redis_strerror(err));
	}

	if (state->len < 1 || !state->s) {
		if ((err == REDIS_RET_CONNECTION_FAILURE) || (err == REDIS_RET_SERVER_DOWN)) {
			state->len = sizeof(ST_ONLINE_S) - 1;
			state->s = pkg_malloc(state->len + 1);
			strcpy(state->s, ST_ONLINE_S);
		} else {
			state->len = sizeof(ST_OFFLINE_S) - 1;
			state->s = pkg_malloc(state->len + 1);
			strcpy(state->s, ST_OFFLINE_S);
		}
		/* don't report the \0 term, it will confuse kamailio string comparison */
	} else --state->len;

	ret = 0;
_end:

	return ret;
#undef a_MAX
} /* get_memsql_state }}} */


#define a_MAX 1024
int customstate_xpidf_(char * body, str pres_uri, subs_t * s, str state);
int customstate_pidf_(char * body, str pres_uri, subs_t * s, str state);
int customstate_dialoginfo_(char * body, str pres_uri, subs_t * s, str state, int presence_initiator);

/* str * build_customstate_body(str pres_uri, subs_t * s, str state, int presence_initiator) {{{ */
str * build_customstate_body(str pres_uri, subs_t * s, str state, int presence_initiator) {
	char body[a_MAX];
	int len = -1;
	str * ret = NULL;
	str b = {0,0};
	
	int i = 0;
	short type, subtype, s_type, s_subtype, ctype = 0;
	int *supp_type = NULL;

	if (state.s == NULL)
		return NULL;

	if (s->event->evp->parsed == EVENT_DIALOG) {
		/* only one content type known */
		len = customstate_dialoginfo_(body, pres_uri, s, state, presence_initiator);
	} else { /* EVENT_PRESENCE */
		/* parse the current subscription accepted content types */
		if (!s->nact && s->accepted_content_types) {
			str accept = { s->accepted_content_types, strlen(s->accepted_content_types) };
			s->nact = parse_accept(&accept, &s->accepted_mime_types, s->mem_type);
		}
		/* identify the content type */
		supp_type = s->event->supp_content_types;
		while (*supp_type) {
			type = *supp_type >> 16;
			subtype = *supp_type & 0x00ff;
			supp_type++;
			if ((type == TYPE_UNKNOWN) || (subtype == SUBTYPE_UNKNOWN))
				continue;
			for (i = 0; i < s->nact; i ++) {
				s_type = s->accepted_mime_types[i] >> 16;
				s_subtype = s->accepted_mime_types[i] & 0x00ff;
				if ( ((s_type == type) || (s_type == TYPE_ALL) || (type == TYPE_ALL)) &&                   /* type check */
					((s_subtype == subtype) || (s_subtype == SUBTYPE_ALL) || (subtype == SUBTYPE_ALL)) ) { /* subtype check */
					/* found a common subtype */
					ctype = s_subtype;
					break;
				}
			}
		}
		
		/* build according to selected content type */
		switch(ctype) {
		case SUBTYPE_PIDFXML:  /* application/pidf+xml */
			s->new_content_type = "application/pidf+xml";
			len = customstate_pidf_(body, pres_uri, s, state);
			break;
		case SUBTYPE_XPIDFXML: /* application/xpidf+xml */
			len = customstate_xpidf_(body, pres_uri, s, state);
			s->new_content_type = "application/xpidf+xml";
			break;
		default:
			LM_WARN("Building content type %d not supported yet\n", ctype);
		}
	}

	if (len > 0) {
		b.len = len;
		b.s = pkg_malloc(b.len + 1);
		if (!b.s) {
			LM_ERR("Out of private mem!\n");
			return NULL;
		}
		strcpy(b.s, body);

		ret = pkg_malloc(sizeof(str));
		if (!ret) {
			LM_ERR("Out of private mem!\n");
			pkg_free(b.s);
			return NULL;
		}

		*ret = b;
	}

	return ret;
} /* build_customstate_body }}} */


/* int customstate_pidf_(char * body, str pres_uri, subs_t * s, str state) {{{ */
int customstate_pidf_(char * body, str pres_uri, subs_t * s, str state) {
	int ret = -1;
	int bc = 0;
	const char * status = "";
	const char * note = "";
	const char * basic_st = "";
	struct sip_uri uri;
	
	if (parse_uri(pres_uri.s, pres_uri.len, &uri)< 0) {
		LM_ERR("while parsing uri %.*s\n", pres_uri.len, pres_uri.s);
		goto _end;
	}

	if (strncasecmp(state.s, "online", state.len) == 0) {
		status = "";
		note = "Ready";
		basic_st = "open";
	} else if (strncasecmp(state.s, "offline", state.len) == 0) {
		status = "<ep:activities><ep:away/></ep:activities>";
		note = "Not online";
		basic_st = "closed";
	} else if (strncasecmp(state.s, "busy", state.len) == 0) {
		status = "<ep:activities><ep:busy/></ep:activities>";
		note = "On the phone";
		basic_st = "open";
	} else if (strncasecmp(state.s, "ringing", state.len) == 0) {
		status = "<ep:activities><ep:busy/></ep:activities>";
		note = "Ringing";
		basic_st = "open";
	} else goto _end;

	body[0] = body[a_MAX - 1] = '\0';
	bc = snprintf(body, a_MAX, \
		"<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n"
		"<presence xmlns=\"urn:ietf:params:xml:ns:pidf\"\n"
		"xmlns:pp=\"urn:ietf:params:xml:ns:pidf:person\"\n"
		"xmlns:es=\"urn:ietf:params:xml:ns:pidf:rpid:status:rpid-status\"\n"
		"xmlns:ep=\"urn:ietf:params:xml:ns:pidf:rpid:rpid-person\"\n"
		"entity=\"sip:%.*s@%.*s\">\n"
		"<pp:person><status>\n"
		"%s\n"
		"</status></pp:person>\n"
		"<note>%s</note>\n"
		"<tuple id=\"%.*s\">\n"
		"<contact priority=\"1\">sip:%.*s@%.*s</contact>\n"\
		"<status><basic>%s</basic></status>\n"
		"</tuple>\n"
		"</presence>\n"
		, s->from_user.len, s->from_user.s, s->from_domain.len, s->from_domain.s\
		, status, note, uri.user.len, uri.user.s\
		, uri.user.len, uri.user.s, uri.host.len, uri.host.s\
		, basic_st);
	/* don't return a non-nul terminated string */
	if (bc <= 0 || bc == a_MAX-1) goto _end;


	ret = bc;
_end:

	return ret;
} /* customstate_pidf_ }}} */


/* int customstate_xpidf_(char * body, str pres_uri, subs_t * s, str state) {{{ */
int customstate_xpidf_(char * body, str pres_uri, subs_t * s, str state) {
	int ret = -1;
	int bc = -1;
	const char * status = "";
	const char * substatus = "";
	struct sip_uri uri;

	if (parse_uri(pres_uri.s, pres_uri.len, &uri)< 0) {
		LM_ERR("while parsing uri %.*s\n", pres_uri.len, pres_uri.s);
		goto _end;
	}

	if (strncasecmp(state.s, "online", state.len)==0) {
		status = "open";
		substatus = "online";
	} else if (strncasecmp(state.s, "offline", state.len)==0) {
		status = "closed";
		substatus = "offline";
	} else if (strncasecmp(state.s, "ringing", state.len)==0\
			|| strncasecmp(state.s, "busy", state.len)==0) {
		status = "inuse";
		substatus = "onthephone";
	} else goto _end;

    body[0] = body[a_MAX - 1] = '\0';
    bc = snprintf(body, a_MAX, \
		"<?xml version=\"1.0\"?>\n"
		"<!DOCTYPE presence PUBLIC \"-//IETF//DTD RFCxxxx XPIDF 1.0//EN\" \"xpidf.dtd\">\n"
		"<presence>\n"
		"<presentity uri=\"sip:%.*s@%.*s;method=SUBSCRIBE\" />\n"
		"<atom id=\"%.*s\">\n"
		"<address uri=\"sip:%.*s@%.*s;user=ip\" priority=\"0.800000\">\n"
		"<status status=\"%s\" />\n"
		"<msnsubstatus substatus=\"%s\" />\n"
		"</address>\n"
		"</atom>\n"
		"</presence>\n"\
		, s->from_user.len, s->from_user.s, s->from_domain.len, s->from_domain.s\
		, uri.user.len, uri.user.s\
		, uri.user.len, uri.user.s, uri.host.len, uri.host.s\
		, status, substatus);
	/* don't return a non-nul terminated string */
	if (bc <= 0 || bc == a_MAX-1) goto _end;

	ret = bc;
_end:

	return ret;
} /* customstate_xpidf_ }}} */

/* int customstate_dialoginfo_(char * body, str pres_uri, subs_t * s, str state, int presence_initiator) {{{ */
int customstate_dialoginfo_(char * body, str pres_uri, subs_t * s, str state, int presence_initiator) {
	int ret = -1;
	int bc = -1;
	const char *status = "";
	const char *direction = "";
	int version = 0;
	struct sip_uri uri;

	if (parse_uri(pres_uri.s, pres_uri.len, &uri)< 0) {
		LM_ERR("while parsing uri %.*s\n", pres_uri.len, pres_uri.s);
		goto _end;
	}

	if (strncasecmp(state.s, "online", state.len)==0) {
		status = "terminated";
	} else if (strncasecmp(state.s, "offline", state.len)==0) {
		status = "confirmed";
	} else if (strncasecmp(state.s, "ringing", state.len)==0) {
		status = "early";
	} else if (strncasecmp(state.s, "busy", state.len)==0) {
		status = "confirmed";
	} else goto _end;

	/* get if the event was caused by the watched extension
	 * this will be used to set the dialog presence event direction */
	direction = (presence_initiator)? "initiator" : "recipient";

	/* Because of some buggy firmware some phones may need the version to always stay at zero and not get incremented */
	if (increment_dialog_info_version) {
		version = s->version;
	}
	
	body[0] = body[a_MAX - 1] = '\0';
	bc = snprintf(body, a_MAX, \
		"<?xml version=\"1.0\"?>\n"
		"<dialog-info xmlns=\"urn:ietf:params:xml:ns:dialog-info\" version=\"%d\" state=\"full\" entity=\"sip:%.*s@%.*s\">\n"
		"<dialog id=\"dummy\" call-id=\"dummy\" direction=\"%s\">\n"
		"<state>%s</state>\n"
	  	"</dialog>\n"
		"</dialog-info>\n"
		, version, uri.user.len, uri.user.s, uri.host.len, uri.host.s\
		, direction, status);
	/* don't return a non-nul terminated string */
	if (bc <= 0 || bc == a_MAX-1) goto _end;

	ret = bc;
_end:

	return ret;
} /* customstate_dialoginfo_ }}} */


int accepts_content(subs_t * s, int content_type)
{
	/* if no content is present accept anything */
	short subtype = content_type & 0x00ff;
	short type = content_type >> 16;
	short s_type, s_subtype;
	int i;
	if ((content_type == 0) || !s || (type == TYPE_UNKNOWN) || (subtype == SUBTYPE_UNKNOWN))
		return 1;
	
	if ((s->nact == 0) && s->accepted_content_types) {
		/* we need to parse the accepted content types */
		str accept = { s->accepted_content_types, strlen(s->accepted_content_types) };
		s->nact = parse_accept(&accept, &s->accepted_mime_types, s->mem_type);
	}
	
	if (s->nact == 0)
		return 1;
	
	/* now we need to see if any of the match */
	for (i = 0; i < s->nact; i++) {
		s_type = s->accepted_mime_types[i] >> 16;
		s_subtype = s->accepted_mime_types[i] & 0x00FF;
		if ( ((s_type == type)  || (s_type == TYPE_ALL) || (type == TYPE_ALL)) &&                   /* type check */
			((s_subtype == subtype) || (s_subtype == SUBTYPE_ALL) || (subtype == SUBTYPE_ALL)) ) {  /* subtype check */
			/* here we manually fix the accepted content type */
			switch(subtype) {
			case SUBTYPE_PIDFXML:       /* application/pidf+xml */
				s->new_content_type = "application/pidf+xml"; break;
			case SUBTYPE_XPIDFXML:      /* application/xpidf+xml */
				s->new_content_type = "application/xpidf+xml"; break;
			case SUBTYPE_DIALOGINFOXML: /*application/dialog-info+xml */
				s->new_content_type = "application/dialog-info+xml"; break;		
			default:
				s->new_content_type = NULL; break;
			}
			return 1;
		}
	}
	
	/* no content type accepted */
	return 0;
}


typedef struct {
	const char* stateValue;
	const char* psaState;
	unsigned char cmpLen;
} parsed_state_t;
/* The parsed presence state for each type of content type */
const parsed_state_t dialogStates[] = { {"terminated", ST_ONLINE_S, 10}, {"early", ST_RINGING_S, 5},
		{"confirmed", ST_BUSY_S, 9}, {0, 0, 0} };
const parsed_state_t pidfStates[]   = { {"open", ST_ONLINE_S, 5}, {"closed", ST_OFFLINE_S, 6}, {0, 0, 0} };
const parsed_state_t pidfNotes[]    = { {"Ready", ST_ONLINE_S, 5}, {"Ringing", ST_RINGING_S, 7},
		{"On the phone", ST_BUSY_S, 12}, {"Not online", ST_OFFLINE_S, 10}, {"Available", ST_ONLINE_S, 9},
		{"Busy", ST_BUSY_S, 4}, {"Lunch", ST_OFFLINE_S, 5}, {"Meeting", ST_BUSY_S, 7},
		{"Out of Office", ST_OFFLINE_S, 13}, {"Vacantion", ST_OFFLINE_S, 9}, {0, 0, 0} };
const parsed_state_t xpidfStates[]  = { {"online", ST_ONLINE_S, 6}, {"inuse", ST_BUSY_S, 5},
		{"closed", ST_OFFLINE_S, 6}, {0, 0, 0} };

int get_published_state(int event, int content_type, str *body, str *state)
{
	xmlDocPtr doc      = NULL;
	char *parsed_state = NULL;
	char *parsed_state_psa = NULL;
	parsed_state_t *toParse = NULL;
	short subtype = content_type & 0x00FF;
	static libxml_api_t xmlApi = { NULL, NULL, NULL, NULL };
	int result = -1;

	if (!body || !body->s || !state)
		goto error;
	
	/* One time only initialize libxml api */
	if (!xmlApi.xmlNodeGetNodeByName) {
		bind_libxml_t bind_libxml;
		if ( ((bind_libxml = (bind_libxml_t)find_export("bind_libxml_api", 1, 0)) == NULL) ||
			(bind_libxml(&xmlApi) == -1) ||
			!xmlApi.xmlNodeGetNodeByName || !xmlApi.xmlNodeGetNodeContentByName || !xmlApi.xmlNodeGetAttrContentByName) {
			LM_ERR("can not bind libxml api\n");
			goto error;
		}
	}
			
	/* we need to parse the xml for a specific body type */
	if (event == EVENT_DIALOG) {
		/* try to find the state in <state> node entry */
		if ( ((doc = xmlParseMemory(body->s, body->len)) == NULL) ||
			 ((parsed_state = xmlApi.xmlNodeGetNodeContentByName(doc->children, "state", NULL)) == NULL) ) {
			LM_WARN("Could not parse the current xml body for state or state node not found\n");
			goto error;
		}
		
		/* we need to convert the state to our own namespace name */
		toParse = (parsed_state_t*)dialogStates;
	} else {
		switch (subtype) {
		case SUBTYPE_PIDFXML:  /* application/pidf+xml */
			/* try to find the state in <note> node entry */
			if ((doc = xmlParseMemory(body->s, body->len)) == NULL) {
				LM_WARN("Could not parse the current pidf xml body\n");
				goto error;
			}

			/* look for note or basic entries */
			if ((parsed_state = xmlApi.xmlNodeGetNodeContentByName(doc->children, "note", NULL)) != NULL) {
				/* we need to convert the note node to our own namespace name */
				toParse = (parsed_state_t*)pidfNotes;
			} else if ((parsed_state = xmlApi.xmlNodeGetNodeContentByName(doc->children, "basic", NULL)) != NULL) {
				/* we need to convert the note node to our own namespace name */
				toParse = (parsed_state_t*)pidfStates;
			} else {
				LM_WARN("Could not find node or basic xml nodes\n");
				goto error;
			}
			break;
		case SUBTYPE_XPIDFXML: { /* application/xpidf+xml */
			xmlNodePtr node = NULL;
			/* try to find the state in state= attribute of the <state> node entry */
			if ( ((doc = xmlParseMemory(body->s, body->len)) == NULL) ||
				 ((node = xmlApi.xmlNodeGetNodeByName(doc->children, "state", NULL)) == NULL) ||
				 ((parsed_state = xmlApi.xmlNodeGetAttrContentByName(node, "state")) == NULL) ) {
				LM_WARN("Could not parse the current xml body or state attribute in state node not found\n");
				goto error;
			}

			/* we need to convert the state to our own namespace name */
			toParse = (parsed_state_t*)xpidfStates;
			break;
		}
		default:
			LM_WARN("Getting published state from this content type is not currently supported: 0x%X\n", (unsigned int)content_type);
			goto error;
		}
	}
	
	while (toParse && toParse->stateValue) {
		if (strncasecmp(parsed_state, toParse->stateValue, toParse->cmpLen)==0) {
			parsed_state_psa = (char*)toParse->psaState;
			break;
		}
		toParse++;
	}
	
	if (!parsed_state_psa) {
		LM_WARN("Could not identify the presence type state in this publish: %s\n", (parsed_state)? parsed_state : "");
		goto error;
	}

	/* copy the found dialog state */
	state->len = strlen(parsed_state_psa);
	state->s = (char*)pkg_malloc(state->len + 1);
	memcpy(state->s, parsed_state_psa, state->len + 1);

	result = 1;
error:
	if (doc) {
		xmlFreeDoc(doc);
		if(parsed_state) {
			xmlFree(parsed_state);
		}
		xmlCleanupParser();
		xmlMemoryDump();
	}
	
	return result;
}
