/*
 * Hash functions for cached publisher table
 */


#include "../../dprint.h"
#include "../../ut.h"
#include "../../hash_func.h"
#include "../../mem/shm_mem.h"
#include "../../mi/mi.h"
#include "presence.h"
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>

#define pub_hash(_s)  core_case_hash( _s, 0, PUB_HASH_SIZE)

/* Add publisher server to hash table */
int pub_hash_table_install (struct publisher_list **hash_table, char *publisher)
{
	struct publisher_list *np;
	unsigned int hash_val;

	np = (struct publisher_list *) shm_malloc(sizeof(*np));
	if (np == NULL) {
		LM_ERR("Cannot allocate memory for hash table entry\n");
		return -1;
	}

	np->publisher.len = strlen(publisher);
	np->publisher.s = (char *) shm_malloc(np->publisher.len);
	if (np->publisher.s == NULL) {
		LM_ERR("Cannot allocate memory for publisher string\n");
	        shm_free(np);
		return -1;
	}
	(void) strncpy(np->publisher.s, publisher, np->publisher.len);

	hash_val = pub_hash(&np->publisher);
	np->next = hash_table[hash_val];
	hash_table[hash_val] = np;

	return 1;
}


/* Check if publisher exists in hash table */
int pub_hash_table_lookup (str *ip)
{
	struct publisher_list *np;

	for (np = (*hash_table)[pub_hash(ip)]; np != NULL; np = np->next) {
		if ((np->publisher.len == ip->len) && 
		(strncasecmp(np->publisher.s, ip->s, np->publisher.len) == 0))
			return 1;
	}

	return -1;
}

/* Free contents of hash table */
void pub_hash_table_free (struct publisher_list **hash_table)
{
	int i;
	struct publisher_list *np, *next;
	
	if(hash_table==0)
		return;

	for (i = 0; i < PUB_HASH_SIZE; i++) {
		np = hash_table[i];
		while (np) {
			shm_free(np->publisher.s);
			next = np->next;
			shm_free(np);
			np = next;
		}
		hash_table[i] = NULL;
	}
}

/*
 * Check if publisher ip is found in the publisher hash table
 */
int is_publisher(struct sip_msg* _msg, char* _sp, char* _s2)
{
	pv_spec_t *sp;
	pv_value_t pv_val;

	sp = (pv_spec_t *)_sp;

	if (sp && (pv_get_spec_value(_msg, sp, &pv_val) == 0)) {
		if (pv_val.flags & PV_VAL_STR) {
			if (pv_val.rs.len == 0 || pv_val.rs.s == NULL) {
				LM_DBG("Missing publisher IP AVP\n");
				return -1;
			}
		return pub_hash_table_lookup(&(pv_val.rs));
		} else {
			LM_DBG("Pseudo variable value is not string\n");
			return -1;
		}
	} else {
		LM_DBG("Cannot get pseudo variable value\n");
		return -1;
    	}
}
