/*
 * 4PSA VoipNow - Implements a PV module constant. API for
 *                adding a constant to the PV cntset and
 *                getting it's value.
 *
 * Copyright (c) 2009 Rack-Soft, LLC. All rights reserved..
 *
 * $Header: //itsd/devel/platform/voipnow/251/low_level/kamailio_extra_sources/1.5.4/modules/pv/pv_cntv_4psa.c#1 $
 *
 * Modified: $DateTime: 2010/09/28 18:16:02 $ by $Author: bogdanc $ with $Change: 71991 $
 */

#include <string.h>

#include "../../dprint.h"
#include "../../mem/mem.h"
#include "../../mem/shm_mem.h"
#include "../../ut.h"
#include "../../pvar.h"

#include "pv_cntv_4psa.h"

static cnt_var_t  *cnt_vars    = 0;      /* global shared memory constants */
static str         cnt_cpy     = {0, 0}; /* local copy for returned result */

#define GUARD_ALLOC(f) \
	if((f) == 0) { \
		LM_ERR("out of shm mem\n"); \
		goto error; \
	}

#define GUARD_PRV_ALLOC(f) \
	if((f) == 0) { \
		LM_ERR("out of pkg mem\n"); \
		goto error; \
	}

/* adds a new constant variable to the global cnt_vars constant list */
static cnt_var_t* add_cnt_var(str *name, int_str *value, int flags);


int param_set_cntvar( modparam_t type, void* val)
{
	str s;
	char *p;
	int_str isv;
	int flags;
	int ival;

	/* parse variable type modparam("pv", "cntset",
	 * "CNT_NAME=[sS]:value" for strings
	 * "CNT_NAME=[iI]:int" ) for integer */
	s.s = (char*)val;
	if(s.s == NULL || s.s[0] == '\0')
		goto error;

	p = s.s;
	while(*p && *p!='=') p++;

	if(*p!='=')
		goto error;

	s.len = p - s.s;
	if(s.len == 0)
		goto error;
	p++;
	flags = 0;
	if(*p!='s' && *p!='S' && *p!='i' && *p!='I')
		goto error;

	if(*p=='s' || *p=='S')
		flags = VAR_VAL_STR;
	p++;
	if(*p!=':')
		goto error;
	p++;
	isv.s.s = p;
	isv.s.len = strlen(p);
	if(flags != VAR_VAL_STR) {
		if(str2sint(&isv.s, &ival)<0)
			goto error;
		isv.n = ival;
	}
	/* add the constant variable name and value */
	if( add_cnt_var(&s, &isv, flags) == NULL )
		goto error;

	return 0;
error:
	LM_ERR("unable to set constant parame [%s]\n", s.s);
	return -1;
}


int pv_parse_cntvar_name(pv_spec_p sp, str *in)
{
	cnt_var_t *it = 0;

	if(in==NULL || in->s==NULL || sp==NULL)
		return -1;

	sp->pvp.pvn.type    = PV_NAME_PVAR;
	sp->pvp.pvn.u.dname = (void*)0;
	/* check for constant
	 * FIXME: now a bit slow - O(n)
	 * we do not expect many constants
	 */
	for(it = cnt_vars; it; it = it->next) {
		if(it->name.len == in->len
			&& strncmp(in->s, it->name.s, in->len) == 0) {
			sp->pvp.pvn.u.dname = (void*)it;
			break;
		}
	}

	return 0;
}


int pv_get_cntvar(struct sip_msg *msg, pv_param_t *param, pv_value_t *res)
{
	cnt_var_t *cnt = NULL;

	if (param == NULL || param->pvn.u.dname == 0)
		goto error;
	/* get the constant variable */
	cnt = (cnt_var_t*)param->pvn.u.dname;

	if(cnt->v.flags & VAR_VAL_STR) {
		/* realloc the private static variable if needed */
		if(cnt_cpy.s == NULL || cnt_cpy.len < cnt->v.value.s.len) {
			GUARD_PRV_ALLOC( cnt_cpy.s = (char*)pkg_realloc(cnt_cpy.s, cnt->v.value.s.len * sizeof(char)) )
		}
		/* copy the string value */
		cnt_cpy.len = cnt->v.value.s.len;
		strncpy(cnt_cpy.s, cnt->v.value.s.s, cnt_cpy.len);
		res->rs    = cnt_cpy;
		res->flags = PV_VAL_STR;
	} else {
		int len     = 0;
		/* copy the int value */
		res->ri     = cnt->v.value.n;
		res->rs.s   = sint2str(res->ri, &len);
		res->rs.len = len;
		res->flags  = PV_VAL_STR|PV_VAL_INT|PV_TYPE_INT;
	}
	return 0;

error:
	pv_get_null(msg, param, res);
	return -1;
}


static cnt_var_t* add_cnt_var(str *name, int_str *value, int flags)
{
	cnt_var_t *it;

	if(name == 0 || name->s == 0 || name->len <= 0 || value == 0)
		return 0;

	/* first check for constant duplicates */
	for(it = cnt_vars; it; it = it->next) {
		if(it->name.len == name->len
				&& strncmp(name->s, it->name.s, name->len)==0) {
			LM_ERR("Constant %.*s was already defined\n" , name->len, name->s);
			return 0;
		}
	}

	/* alloc the constant into shared memory */
	GUARD_ALLOC( it = (cnt_var_t*)shm_malloc(sizeof(cnt_var_t)) )
	memset(it, 0, sizeof(cnt_var_t));

	/* copy the constant name */
	GUARD_ALLOC( it->name.s = (char*)shm_malloc((name->len + 1) * sizeof(char)) )
	it->name.len = name->len;
	strncpy(it->name.s, name->s, name->len);
	it->name.s[it->name.len] = '\0';

	/* add the constant value */
	it->v.flags = flags;
	if (flags & VAR_VAL_STR) {
		/* set the string value */
		str *s = &it->v.value.s;
		s->len = value->s.len;
		GUARD_ALLOC( s->s = (char*)shm_malloc((s->len + 1) * sizeof(char)) )
		strncpy(s->s, value->s.s, s->len);
		s->s[s->len] = '\0';
		LM_DBG("Added pv constant [%.*s=%.*s]\n", name->len, name->s,
				it->v.value.s.len, it->v.value.s.s);
	} else {
		/* set the integer value */
		it->v.value.n = value->n;
		LM_DBG("Added pv constant [%.*s=%d]\n", name->len, name->s, it->v.value.n);
	}

	/* insert into the list */
	it->next = cnt_vars;
	cnt_vars = it;

	return it;

error:
	if (it) {
		if (it->name.s)
			shm_free(it->name.s);
		if ( (flags & VAR_VAL_STR) && it->v.value.s.s )
			shm_free(it->v.value.s.s);
		shm_free(it);
	}
	return 0;
}
