/*
 * {app} - {purpose, description}
 *
 * Copyright (c) 2005-2006 Rack-Soft, LLC. All rights reserved.
 *
 * $Header: //itsd/devel/platform/voipnow/251/low_level/kamailio_extra_sources/1.5.4/resolve_4psa.c#1 $
 *
 * Modified: $DateTime: 2010/09/28 18:16:02 $ by $Author: bogdanc $ with $Change: 71991 $
 */

#ifndef _GNU_SOURCE
# define _GNU_SOURCE
#endif


#include "resolve.h"
#include "mem/mem.h"
#include "mem/shm_mem.h"
#include "timer.h"
#include "4psa_mem_wrappers.h"

#include <4psa_hashmacros.h>


struct fr_hash_table * g_dns_cache;
unsigned dns_cache_sweep_interval = DNS_CACHE_SWEEP_INTERVAL_DFLT;
unsigned dns_cache_entry_ttl = DNS_CACHE_ENTRY_TTL_DFLT;

static int destroy_dns_cached_enrty(void * ptr);

/* int init_dns_cache(int size) {{{ */
int init_dns_cache(int size) {
	int ret = -1;
	struct fr_allocator_api_t alloc_api;
	struct fr_locker_api_t lock_api;

	lock_api.type = PTHREAD_MUTEX_RECURSIVE;
	lock_api.shared = PTHREAD_PROCESS_SHARED;

	alloc_api.alloc = shm_malloc_f;
	alloc_api.calloc = shm_calloc_f;
	alloc_api.realloc = shm_realloc_f;
	alloc_api.free = shm_free_f;

	g_dns_cache = fr_hash_init_api(size, destroy_dns_cached_enrty, &alloc_api, &lock_api);
	if (!g_dns_cache) goto _end;

	/* Register cache timer */
	register_timer(cache_sweep_timer, 0, dns_cache_sweep_interval);

	ret = 0;
_end:

	return ret;
} /* init_dns_cache }}} */

/* void cache_sweep_timer(unsigned int ticks, void* param) {{{ */
void cache_sweep_timer(unsigned int ticks, void* param) {
	time_t now = time(NULL);
	FR_H_VARS;

	LM_DBG(" + dns: sweep cache:\n");
	FR_H_LOOP(g_dns_cache, char, struct dns_cached_he_t) {
		if (now - FR_H_VAL->time >= dns_cache_entry_ttl) {
			fr_hash_del(g_dns_cache, FR_H_KEY, FR__H_KEY_STRUCT->key_size);
		}
	}
/* _end: */
	FR_H_VARS_END;

	return;
} /* cache_sweep_timer }}} */


/*!\fn int shm_copy_hosten(struct hostent* old, struct hostent* new) {{{
 * \brief Copy the contents of the hostentry in shm, the container is already alloc'd
 * 
 * \param struct hostent* old <desc>
 * \param struct hostent* new <desc>
 * 
 * \return <desc>
 */
#if 0
from gethostbyname(3) 2.6 kernel
struct hostent {
	char    *h_name;        /* official name of host */
	char    **h_aliases;    /* alias list NULL ptr term */
	int     h_addrtype;     /* host address type */
	int     h_length;       /* length of address */
	char    **h_addr_list;  /* list of addresses NULL term */
}
#endif
int shm_copy_hosten(const struct hostent* old, struct hostent* new) {
	int ret = -1;
	/* struct hostent is already in storage */
	size_t len = 0, h_len = 0, alias_len = 0, addr_len = 0;
	char * p = NULL, **array = NULL;
	int i;
	
	h_len = strlen(old->h_name) + 1;
	len += h_len;
	
	for (i = 0; old->h_aliases[i]; ++i) {
		alias_len += sizeof(old->h_aliases[i]);
		len += strlen(old->h_aliases[i])+1;
	}
	/* + null ptr term */
	alias_len += sizeof(old->h_aliases[i]);
	for (i = 0; old->h_addr_list[i]; ++i) {
		addr_len += sizeof(old->h_addr_list[i]);
		len += old->h_length;
	}
	/* NULL ptr term */
	addr_len += sizeof(old->h_addr_list[i]);

	p = shm_malloc(len + addr_len + alias_len);
	if (!p) {
		LM_ERR("Can not allocate %ld bytes of shared memory\n", (long)(len+addr_len+alias_len));
		goto _end;
	}


	/* host */
	memcpy(p, old->h_name, h_len);
	new->h_name = p;
	p += h_len;
	
	/* array of aliases */
	array = (char **)p;
	new->h_aliases = array;
	/* p is past the array */
	p += alias_len;
	/* pile up strings after tha array, array[i] points to each block */
	for (i = 0; old->h_aliases[i]; ++i) {
		h_len = strlen(old->h_aliases[i])+1;
		array[i] = p;
		memcpy(array[i], old->h_aliases[i], h_len);
		p += h_len;
	}
	array[i] = NULL;

	new->h_addrtype = old->h_addrtype;
	new->h_length = old->h_length;

	/* array of addresses */
	array = (char **)p;
	new->h_addr_list = array;
	/* p is past the array */
	p += addr_len;
	/* pile up addesses after tha array, array[i] points to each block */
	for (i = 0; old->h_addr_list[i]; ++i) {
		array[i] = p;
		memcpy(array[i], old->h_addr_list[i], old->h_length);
		p += old->h_length;
	}
	array[i] = NULL;

	ret = 0;
_end:

	return ret;
} /* }}} shm_copy_hosten */


/* int destroy_dns_cached_enrty(void * ptr) {{{ */
int destroy_dns_cached_enrty(void * ptr) {
	struct dns_cached_he_t * dns = (struct dns_cached_he_t *)ptr;

	/* the freeing of the container will not occur */
	if (!dns) return 1;

	/* the hole chunk resides at this address; this is the only free we need */
	/* free_shm_hostent(&dns->he); */
	if (dns->he.h_name) shm_free(dns->he.h_name);

	if (dns->dn) {
		if (dns->dn->kids) 
			shm_free(dns->dn->kids);
		shm_free(dns->dn);
	}
	/* we didn't free the container; the caller should know better what to do
	 * (we don't alloc the container, thus we don't free it; its hash's duty) */
	return 0;
} /* destroy_dns_cached_enrty }}} */


/* copy the dns lookup result in shm mem;
 * the initial container is not alloc'd since it's contents will be copied anyway */
/* struct dns_cached_he_t * new_dns_cached_entry(struct dns_cached_he_t * hdns, const struct hostent* he, unsigned int proto, unsigned int port, struct dns_node * dn) {{{ */
struct dns_cached_he_t * new_dns_cached_entry(struct dns_cached_he_t * hdns, const struct hostent* he, unsigned short proto, unsigned short port, struct dns_node * dn) {

	time(&hdns->time);

	/* if he is NULL, then cache a negative dns response */
	if (!he) {
		hdns->he.h_length = 0;
		return hdns;
	}

	if (shm_copy_hosten(he, &hdns->he)!=0) return NULL;
	hdns->proto = proto;
	hdns->port = port;
	hdns->addr_idx = 0;
	hdns->dn = dn;

	return hdns;
} /* new_dns_cached_entry }}} */


/*!\fn struct hostent* resolve_dns_cached(char * full_name, size_t len, int type, unsigned short * pproto, unsigned short * pport, struct dns_node ** dn, char * save_key, unsigned short * paddr_idx) {{{
 * \brief Search for a specific name + RR type in dns cache
 *  
 * Returns hostentry and sets port, proto, dn_nodes for failolver, the key stored in cache
 * if those arguments are provided.
 * 
 * \param char * full_name: the name that is looked up in dns
 * \param size_t len: its size
 * \param int type:	the type of the RR, -1 for both A and AAAA
 * \param unsigned short * pproto: ptr to proto, to be set if in hash
 * \param unsigned short * pport: ptr to port, to be set if in hash
 * \param struct dns_node ** dn: ptr to list of dns_nodes for failover, to be \b copied if in hash
 * \param char * save_key: buffer to store the hash key. at least maxdomain + 2.
 * \param unsigned short * paddr_idx: ptr to the index used by struct proxy_l to remember wich addess from struct hostent will use next. Leave NULL if not the case.
 * 
 * \return 0 (NULL): the key is not in hash
 * \return -1 (-1UL): the key is in hash, but it says that dns gave no answer for that type
 * \return non-zero, differnt from -1: the key is in hash and the value represents the ptr to the struct hostentry stored there (additional values, passed as pointers, may have been set or copied, see above)
 */
struct hostent* resolve_dns_cached(char * full_name, size_t len, int type, unsigned short * pproto, unsigned short * pport, struct dns_node ** dn, char * save_key, unsigned short * paddr_idx) {
	struct dns_cached_he_t * res = NULL;
	struct hostent* ret = NULL;
	int bc = 0;
	int locked = 0;

	if (!full_name || len <= 0) return NULL;

	bc = snprintf(full_name + len, (MAX_DNS_CACHE_NAME) - len, "%d", type);
	if (bc <= 0 || bc >= (MAX_DNS_CACHE_NAME) - len) {
		goto _end;
	}

	LM_DBG(" + dns: try cache: %s (type:%d)\n", full_name, type);

	FR_OBJ_LOCK(g_dns_cache);
	locked = 1;

	/* we include the \0 in the key */
	res = (struct dns_cached_he_t *)fr_hash_find(g_dns_cache, full_name, len + bc + 1);
	if (!res) goto _end;

	if (time(NULL) - res->time >= dns_cache_entry_ttl) {
		fr_hash_del(g_dns_cache, full_name, len + bc + 1);
		LM_DBG(" + dns: old cache entry: %s (type:%d) was removed\n", full_name, type);
		goto _end;
	}

	/* we choosed to consider a zero value here as a response from dns
	 * saying that there are no RR for that type
	 * exit with -1 ptr, return that the user of this function will have to interpret */
	if (!res->he.h_length) {
		LM_DBG(" + dns: dns will give no result for %s (type:%d)\n", full_name, type);
		ret = (void *)-1UL;
		goto _end;
	}

	if (save_key) strncpy(save_key, full_name, MAX_DNS_CACHE_NAME);
	ret  = &res->he;
	if (pproto) *pproto = res->proto;
	if (pport) *pport = res->port;
	if (paddr_idx) *paddr_idx = res->addr_idx;
	if (dn && res->dn) *dn = dns_res_copy(res->dn);
	LM_DBG(" + dns: got result for type:%d: %s\n", type, ret->h_name);
_end:
	if (locked) FR_OBJ_UNLOCK(g_dns_cache);
	full_name[len] = '\0';

	return ret;
} /* resolve_dns_cached }}} */


/*!\fn int add_dns_cached(char * full_name, size_t len, int type, const struct hostent* he, unsigned short * pproto, unsigned short * pport, struct dns_node * dn, char * save_key) {{{
 * \brief Add a dns result to dns cache.
 *  
 * Adds different sort of details to the hash including, the struct hostent returned
 * by gethostbyname(), the port, proto (transport), the key used to store it in hash,
 * the dns_node used by struct proxy_l to do failover on SRV,
 * and the index used by struct proxy_l to remember wich address from struct hostent
 * to use.
 * 
 * \param char * full_name <desc>
 * \param size_t len <desc>
 * \param int type <desc>
 * \param const struct hostent* he <desc>
 * \param unsigned short * pproto <desc>
 * \param unsigned short * pport <desc>
 * \param struct dns_node * dn <desc>
 * \param char * save_key <desc>
 * 
 * \return <desc>
 */
int add_dns_cached(char * full_name, size_t len, int type, const struct hostent* he, unsigned short * pproto, unsigned short * pport, struct dns_node * dn, char * save_key) {
	int ret = -1;
	int bc = 0;
	struct dns_cached_he_t new;
	struct dns_node * new_dn = NULL;

	if (!full_name || len <= 0) return -1;

	bc = snprintf(full_name + len, MAX_DNS_CACHE_NAME - len, "%d", type);
	if (bc <= 0 || bc >= MAX_DNS_CACHE_NAME - len) {
		goto _end;
	}


	memset(&new, 0, sizeof(new));
	/* add a copy to hash, dn is used in a destructive way. */
	if (dn) new_dn = dns_res_copy(dn);
	if (!new_dns_cached_entry(&new, he, pproto ? *pproto:0, pport ? *pport:0, new_dn)) goto _end;

	/* this will lock inside and perform a replace if other proc already inserted this entry */
	/* we include the \0 in the key */
	if (fr_hash_add(g_dns_cache, full_name, len + bc + 1, &new, sizeof(new)) != 0) {
		destroy_dns_cached_enrty(&new);
		goto _end;
	}

	if (save_key) strncpy(save_key, full_name, MAX_DNS_CACHE_NAME);
	LM_DBG(" + dns: add cache: %s (type:%d)\n", full_name, type);
	ret = 0;
_end:
	full_name[len] = '\0';

	return ret;
} /* add_dns_cached }}} */

int remove_dns_cached(const char * dns_lookup_name) {
	int ret = fr_hash_del(g_dns_cache, dns_lookup_name, strlen(dns_lookup_name)+1);
	return ret;
}

/*!\fn int update_dns_cached(const char * dns_lookup_name, unsigned short addr_idx, struct hostent * he, struct dns_node * dn) {{{
 * \brief 
 *  
 * <details>
 * 
 * \param const char * dns_lookup_name <desc>
 * \param unsigned short addr_idx <desc>
 * \param struct hostent * he <desc>
 * \param struct dns_node * dn <desc>
 * 
 * \return <desc>
 */
int update_dns_cached(const char * dns_lookup_name, unsigned short addr_idx, struct hostent * he, struct dns_node * dn) {
	int ret = -1;
	int locked = 0;
	struct dns_cached_he_t * hdns = NULL;
	struct dns_node * new_dn = NULL;

	FR_OBJ_LOCK(g_dns_cache);
	locked = 1;

	hdns = FR_H_FIND(g_dns_cache, dns_lookup_name);
	if (!hdns) goto _end;

	if (he) {
		if (shm_copy_hosten(he, &hdns->he)!=0) goto _end;
		/* add a copy to hash, dn is used in a destructive way. */
		if (dn) new_dn = dns_res_copy(dn);
		if (hdns->dn) {
			if (hdns->dn->kids)
				shm_free(hdns->dn->kids);
			shm_free(hdns->dn);
		}
		hdns->dn = new_dn;
		LM_DBG(" + dns: update cache entry: %s with new he:%p and dn:%p\n"\
				, dns_lookup_name, he, dn);
	} else {
		hdns->addr_idx = addr_idx;
		LM_DBG(" + dns: update cache entry: %s with new addr_idx: %hd\n"\
				, dns_lookup_name, addr_idx);
	}

	ret = 0;
_end:
	if (locked) FR_OBJ_UNLOCK(g_dns_cache);

	return ret;
} /* }}} update_dns_cached */
