/*
 * $Id: script_cb.h 5003 2008-09-26 11:01:51Z henningw $
 *
 * Copyright (C) 2001-2003 FhG Fokus
 *
 * This file is part of Kamailio, a free SIP server.
 *
 * Kamailio is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * Kamailio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * History:
 * --------
 *  2005-02-13  script callbacks devided into request and reply types (bogdan)
 */

/*!
 * \file
 * \brief Script callbacks functions
 */

#ifndef _SCRIPT_CB_H_
#define _SCRIPT_CB_H_

#include "parser/msg_parser.h"

/*! Callback function */
typedef int (cb_function)( struct sip_msg *msg, void *param );

/* Callback types */
#define PRE_SCRIPT_CB    (1<<0) /*!< pre-script callback  */
#define POST_SCRIPT_CB   (1<<1) /*!< post-script callback */
#define REQ_TYPE_CB      (1<<2) /*!< request callback     */
#define RPL_TYPE_CB      (1<<3) /*!< reply callback       */


/*! Script callback list */
struct script_cb{
	cb_function *cbf;       /*!< callback function  */
	struct script_cb *next; /*!< next callback      */
	unsigned int id;        /*!< callback id        */
	void *param;            /*!< callback parameter */
};

int register_script_cb( cb_function f, int type, void *param );
void destroy_script_cb(void);

int exec_pre_req_cb( struct sip_msg *msg);
int exec_post_req_cb( struct sip_msg *msg);

int exec_pre_rpl_cb( struct sip_msg *msg);
int exec_post_rpl_cb( struct sip_msg *msg);

#endif
