/*
 * {app} - {purpose, description}
 *
 * Copyright (c) 2005-2006 Rack-Soft, LLC. All rights reserved.
 *
 * $Header: //itsd/devel/platform/voipnow/251/low_level/kamailio_extra_sources/1.5.4/udp_server_4psa.c#1 $
 *
 * Modified: $DateTime: 2010/09/28 18:16:02 $ by $Author: bogdanc $ with $Change: 71991 $
 */



#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <errno.h>
#include <arpa/inet.h>
#ifdef __linux__
	#include <linux/types.h>
	#include <linux/errqueue.h>
#endif


#include "udp_server.h"
#include "globals.h"
#include "config.h"
#include "dprint.h"
#include "receive.h"
#include "mem/mem.h"
#include "ip_addr.h"
#include "socket_info.h"
#include "resolve.h"
#include "name_alias.h"


/* int fix_socket_info(struct socket_info * si) {{{ */
int fix_socket_info(struct socket_info * si) {
	char* tmp;
	int len;
	struct hostent* he;
	char** h;

	/* fix port number, port_no should be !=0 here */
	if (si->port_no==0){
#ifdef USE_TLS
		si->port_no= (si->proto==PROTO_TLS)?tls_port_no:port_no;
#else
		si->port_no= port_no;
#endif
	}
	tmp=int2str(si->port_no, &len);
	if (len>=MAX_PORT_LEN){
		LM_ERR("bad port number: %d\n", si->port_no);
		goto error;
	}
	si->port_no_str.s=(char*)shm_malloc(len+1);
	if (si->port_no_str.s==0){
		LM_ERR("out of pkg memory.\n");
		goto error;
	}
	strncpy(si->port_no_str.s, tmp, len+1);
	si->port_no_str.len=len;
	/* get "official hostnames", all the aliases etc. */
	he=resolvehost(si->name.s,0);
	if (he==0){
		LM_ERR("could not resolve %s\n", si->name.s);
		goto error;
	}
	/* check if we got the official name */
	if (strcasecmp(he->h_name, si->name.s)!=0){
		if (auto_aliases && add_alias(si->name.s, si->name.len,
					si->port_no, si->proto)<0){
			LM_ERR("add_alias failed\n");
		}
		/* change the official name */
		shm_free(si->name.s);
		si->name.s=(char*)shm_malloc(strlen(he->h_name)+1);
		if (si->name.s==0){
			LM_ERR("out of pkg memory.\n");
			goto error;
		}
		si->name.len=strlen(he->h_name);
		strncpy(si->name.s, he->h_name, si->name.len+1);
	}
	/* add the aliases*/
	if (auto_aliases) {
		for(h=he->h_aliases; h && *h; h++)
			if (add_alias(*h, strlen(*h), si->port_no, si->proto)<0){
				LM_ERR("add_alias failed\n");
			}
	}
	hostent2ip_addr(&si->address, he, 0); /*convert to ip_addr
											format*/
	if ((tmp=ip_addr2a(&si->address))==0) goto error;
	if (si->address.af == AF_INET6) {
		si->address_str.s=(char*)shm_malloc(strlen(tmp)+1+2);
		if (si->address_str.s==0){
			LM_ERR("out of pkg memory.\n");
			goto error;
		}
		si->address_str.s[0] = '[';
		strncpy( si->address_str.s+1 , tmp, strlen(tmp));
		si->address_str.s[1+strlen(tmp)] = ']';
		si->address_str.s[2+strlen(tmp)] = '\0';
		si->address_str.len=strlen(tmp) + 2;
	} else {
		si->address_str.s=(char*)shm_malloc(strlen(tmp)+1);
		if (si->address_str.s==0){
			LM_ERR("out of pkg memory.\n");
			goto error;
		}
		strncpy(si->address_str.s, tmp, strlen(tmp)+1);
		si->address_str.len=strlen(tmp);
	}
	/* set is_ip (1 if name is an ip address, 0 otherwise) */
	if ( auto_aliases && (si->address_str.len==si->name.len) &&
			(strncasecmp(si->address_str.s, si->name.s,
						 si->address_str.len)==0)
	   ){
		si->flags|=SI_IS_IP;
		/* do rev. DNS on it (for aliases)*/
		he=rev_resolvehost(&si->address);
		if (he==0){
			LM_WARN("could not rev. resolve %s\n", si->name.s);
		}else{
			/* add the aliases*/
			if (add_alias(he->h_name, strlen(he->h_name),
						si->port_no, si->proto)<0){
				LM_ERR("add_alias failed\n");
			}
			for(h=he->h_aliases; h && *h; h++)
				if (add_alias(*h,strlen(*h),si->port_no,si->proto)<0){
					LM_ERR(" add_alias failed\n");
				}
		}
	}

	/* build and set string encoding */
	tmp = socket2str( si, 0, &si->sock_str.len);
	if (tmp==0) {
		LM_ERR("failed to convert socket to string");
		goto error;
	}
	si->sock_str.s=(char*)shm_malloc(si->sock_str.len + 1);
	if (si->sock_str.s==0) {
		LM_ERR("out of pkg memory.\n");
		goto error;
	}
	memcpy(si->sock_str.s, tmp, si->sock_str.len);

#ifdef USE_MCAST
	/* Check if it is an multicast address and
	 *          * set the flag if so
	 *                   */
	if (is_mcast(&si->address)) {
		si->flags |= SI_IS_MCAST;
	}
#endif /* USE_MCAST */

	return 0;
error:
	return -1;
} /* fix_socket_info }}} */


/* int register_local_address(struct receive_info *ri) {{{ */
int register_local_address(struct receive_info *ri) {
	int ret = -1;
	struct socket_info* si = bind_address;
	struct ifreq ifr;
	int found = 0;

	if (!bind_address) {
		LM_ERR("No default listen list set up\n");
		goto _end;
	}

	while ((si = si->next)) {
		if (si->address.u.addr32[0] == ri->dst_ip.u.addr32[0]) {
			LM_DBG("Address %.*s (using fd:%d) found in socket list\n", si->address_str.len, si->address_str.s, si->socket);
			ri->bind_address=si;
			ri->dst_port=si->port_no;

			found = 1;
			break;
		}
	}

	if (!found) {
		struct ip_addr addr = ri->dst_ip;
		char* tmp = NULL;

		si = NULL;
		/* code copied from socket_info.c */

		if ((tmp=ip_addr2a(&addr))==0) goto _end;

		memset(&ifr, 0, sizeof(ifr));
		if (ioctl(bind_address->socket, SIOCGIFFLAGS, &ifr) == -1) memset(&ifr, 0, sizeof(ifr));
		/* add it to one of the lists */
		/* put ri->dst_ip details here, assume ipv4 */
		if (!(si=new_sock_info(tmp, ri->dst_port, bind_address->proto, ifr.ifr_flags))) {
			LM_ERR("new_sock_info failed\n");
			goto _end;
		}

		/* this is a cloned socket; fd is as the initial bind address. ip is from IP_PKTINFO */
		si->su = bind_address->su;
		si->su.sin.sin_addr.s_addr = ri->dst_ip.u.addr32[0];
		si->address = ri->dst_ip;
		si->socket = bind_address->socket;

		si->address_str.s   = NULL;
		si->address_str.len = 0;
		si->port_no_str.s   = NULL;
		si->port_no_str.len = 0;
		si->sock_str.s      = NULL;
		si->sock_str.len    = 0;

		/* we call this only once per new socket in order to calculate the optimization str */
		fix_socket_info(si);
		LM_DBG("New socket containing info for %.*s (but still using fd:%d) added\n", si->address_str.len, si->address_str.s, si->socket);
		LOCK_LISTEN_LIST
		sock_listadd(&bind_address, si);
		UNLOCK_LISTEN_LIST

		/* the receive info will indicate this as the socket */
		ri->bind_address = si;

		found = 1;
	}

	if (found) ret = 0;
_end:

	return ret;
} /* register_local_address }}} */

